@@ -1,59 +1,62 @@
-# -*- coding:utf-8 -*-
-## src/chat_control.py
-##
-## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
-## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-##                         Nikos Kouremenos <kourem AT gmail.com>
-##                         Travis Shirk <travis AT pobox.com>
-## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
-##                    Julien Pivotto <roidelapluie AT gmail.com>
-## Copyright (C) 2007-2008 Brendan Taylor <whateley AT gmail.com>
-##                         Stephan Erb <steve-e AT h3c.de>
-## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+# Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+#                         Nikos Kouremenos <kourem AT gmail.com>
+#                         Travis Shirk <travis AT pobox.com>
+# Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+#                    Julien Pivotto <roidelapluie AT gmail.com>
+# Copyright (C) 2007-2008 Brendan Taylor <whateley AT gmail.com>
+#                         Stephan Erb <steve-e AT h3c.de>
+# Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+
+from typing import ClassVar  # pylint: disable=unused-import
+from typing import Type  # pylint: disable=unused-import
 
 import os
 import time
+
 from gi.repository import Gtk
 from gi.repository import Gio
-from gi.repository import GdkPixbuf
 from gi.repository import Pango
 from gi.repository import GLib
-from gajim import gtkgui_helpers
-from gajim import gui_menu_builder
-from gajim import message_control
-from gajim import dialogs
+from nbxmpp.protocol import NS_XHTML, NS_XHTML_IM, NS_FILE, NS_MUC
+from nbxmpp.protocol import NS_JINGLE_RTP_AUDIO, NS_JINGLE_RTP_VIDEO
+from nbxmpp.protocol import NS_JINGLE_ICE_UDP, NS_JINGLE_FILE_TRANSFER_5
 
 from gajim.common import app
 from gajim.common import helpers
 from gajim.common import ged
 from gajim.common import i18n
-from gajim.common.stanza_session import EncryptedStanzaSession, ArchivingStanzaSession
+from gajim.common.i18n import _
 from gajim.common.contacts import GC_Contact
-from gajim.common.logger import KindConstant
-from nbxmpp.protocol import NS_XHTML, NS_XHTML_IM, NS_FILE, NS_MUC
-from nbxmpp.protocol import NS_JINGLE_RTP_AUDIO, NS_JINGLE_RTP_VIDEO
-from nbxmpp.protocol import NS_JINGLE_ICE_UDP, NS_JINGLE_FILE_TRANSFER_5
-from nbxmpp.protocol import NS_CHATSTATES
-from gajim.common.connection_handlers_events import MessageOutgoingEvent
 from gajim.common.const import AvatarSize
+from gajim.common.const import KindConstant
+from gajim.common.const import Chatstate
+
+from gajim import gtkgui_helpers
+from gajim import gui_menu_builder
+from gajim import message_control
+from gajim import dialogs
+
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.add_contact import AddNewContactWindow
 
 from gajim.command_system.implementation.hosts import ChatCommands
+from gajim.command_system.framework import CommandHost  # pylint: disable=unused-import
 from gajim.chat_control_base import ChatControlBase
 
 ################################################################################
@@ -74,7 +77,7 @@ class ChatControl(ChatControlBase):
 
     # Set a command host to bound to. Every command given through a chat will be
     # processed with this command host.
-    COMMAND_HOST = ChatCommands
+    COMMAND_HOST = ChatCommands  # type: ClassVar[Type[CommandHost]]
 
     def __init__(self, parent_win, contact, acct, session, resource=None):
         ChatControlBase.__init__(self, self.TYPE_ID, parent_win,
@@ -125,17 +128,12 @@ class ChatControl(ChatControlBase):
         self._pep_images['mood'] = self.xml.get_object('mood_image')
         self._pep_images['activity'] = self.xml.get_object('activity_image')
         self._pep_images['tune'] = self.xml.get_object('tune_image')
-        self._pep_images['location'] = self.xml.get_object('location_image')
+        self._pep_images['geoloc'] = self.xml.get_object('location_image')
         self.update_all_pep_types()
 
         self.show_avatar()
 
         # Hook up signals
-        message_tv_buffer = self.msg_textview.get_buffer()
-        id_ = message_tv_buffer.connect('changed',
-            self._on_message_tv_buffer_changed)
-        self.handlers[id_] = message_tv_buffer
-
         widget = self.xml.get_object('avatar_eventbox')
         widget.set_property('height-request', AvatarSize.CHAT)
 
@@ -204,14 +202,11 @@ class ChatControl(ChatControlBase):
             session = app.connections[self.account].find_controlless_session(
                 self.contact.jid, resource)
 
-        self.setup_seclabel(self.xml.get_object('label_selector'))
+        self.setup_seclabel()
         if session:
             session.control = self
             self.session = session
 
-            if session.enable_encryption:
-                self.print_esession_details()
-
         # Enable encryption if needed
         self.no_autonegotiation = False
         self.add_actions()
@@ -220,7 +215,8 @@ class ChatControl(ChatControlBase):
 
         self.encryption_menu = self.xml.get_object('encryption_menu')
         self.encryption_menu.set_menu_model(
-            gui_menu_builder.get_encryption_menu(self.control_id, self.type_id))
+            gui_menu_builder.get_encryption_menu(
+                self.control_id, self.type_id, self.account == 'Local'))
         self.set_encryption_menu_icon()
         # restore previous conversation
         self.restore_conversation()
@@ -232,17 +228,21 @@ class ChatControl(ChatControlBase):
             # Dont connect this when PrivateChatControl is used
             app.ged.register_event_handler('update-roster-avatar', ged.GUI1,
                 self._nec_update_avatar)
-        app.ged.register_event_handler('failed-decrypt', ged.GUI1,
-            self._nec_failed_decrypt)
         app.ged.register_event_handler('chatstate-received', ged.GUI1,
             self._nec_chatstate_received)
-        app.ged.register_event_handler('caps-received', ged.GUI1,
+        app.ged.register_event_handler('caps-update', ged.GUI1,
             self._nec_caps_received)
         app.ged.register_event_handler('message-sent', ged.OUT_POSTCORE,
             self._message_sent)
         app.ged.register_event_handler(
             'mam-decrypted-message-received',
             ged.GUI1, self._nec_mam_decrypted_message_received)
+        app.ged.register_event_handler(
+            'decrypted-message-received',
+            ged.GUI1, self._nec_decrypted_message_received)
+        app.ged.register_event_handler(
+            'receipt-received',
+            ged.GUI1, self._receipt_received)
 
         # PluginSystem: adding GUI extension point for this ChatControl
         # instance object
@@ -263,21 +263,22 @@ class ChatControl(ChatControlBase):
             act.connect("activate", func)
             self.parent_win.window.add_action(act)
 
-        act = Gio.SimpleAction.new_stateful(
+        self.audio_action = Gio.SimpleAction.new_stateful(
             'toggle-audio-' + self.control_id, None,
             GLib.Variant.new_boolean(False))
-        act.connect('change-state', self._on_audio)
-        self.parent_win.window.add_action(act)
+        self.audio_action.connect('change-state', self._on_audio)
+        self.parent_win.window.add_action(self.audio_action)
 
-        act = Gio.SimpleAction.new_stateful(
+        self.video_action = Gio.SimpleAction.new_stateful(
             'toggle-video-' + self.control_id,
             None, GLib.Variant.new_boolean(False))
-        act.connect('change-state', self._on_video)
-        self.parent_win.window.add_action(act)
+        self.video_action.connect('change-state', self._on_video)
+        self.parent_win.window.add_action(self.video_action)
 
     def update_actions(self):
         win = self.parent_win.window
         online = app.account_is_connected(self.account)
+        con = app.connections[self.account]
 
         # Add to roster
         if not isinstance(self.contact, GC_Contact) \
@@ -301,7 +302,7 @@ class ChatControl(ChatControlBase):
         httpupload = win.lookup_action(
             'send-file-httpupload-' + self.control_id)
         httpupload.set_enabled(
-            online and app.connections[self.account].httpupload)
+            online and con.get_module('HTTPUpload').available)
 
         # Send file (Jingle)
         jingle_conditions = (
@@ -317,20 +318,9 @@ class ChatControl(ChatControlBase):
             jingle.get_enabled() or httpupload.get_enabled())
 
         # Set File Transfer Button tooltip
-        ft_pref = app.config.get_per('accounts', self.account,
-                                     'filetransfer_preference')
-
-        tooltip_text = None
-        if httpupload.get_enabled() and jingle.get_enabled():
-            if ft_pref == 'httpupload':
-                tooltip_text = _('HTTP File Upload')
-            else:
-                tooltip_text = _('Jingle File Transfer')
-        elif httpupload.get_enabled():
-            tooltip_text = _('HTTP File Upload')
-        elif jingle.get_enabled():
-            tooltip_text = _('Jingle File Transfer')
-        elif online:
+        if online and (httpupload.get_enabled() or jingle.get_enabled()):
+            tooltip_text = _('Send File…')
+        else:
             tooltip_text = _('No File Transfer available')
         self.sendfile_button.set_tooltip_text(tooltip_text)
 
@@ -351,7 +341,7 @@ class ChatControl(ChatControlBase):
             'information-' + self.control_id).set_enabled(online)
 
     def _on_add_to_roster(self, action, param):
-        dialogs.AddNewContactWindow(self.account, self.contact.jid)
+        AddNewContactWindow(self.account, self.contact.jid)
 
     def _on_information(self, action, param):
         app.interface.roster.on_info(None, self.contact, self.account)
@@ -404,7 +394,7 @@ class ChatControl(ChatControlBase):
 
         # Jingle detection
         if self.contact.supports(NS_JINGLE_ICE_UDP) and \
-        app.HAVE_FARSTREAM and self.contact.resource:
+        app.is_installed('FARSTREAM') and self.contact.resource:
             self.audio_available = self.contact.supports(NS_JINGLE_RTP_AUDIO)
             self.video_available = self.contact.supports(NS_JINGLE_RTP_VIDEO)
         else:
@@ -426,7 +416,7 @@ class ChatControl(ChatControlBase):
         img = self._pep_images[pep_type]
         if pep_type in pep:
             img.set_from_pixbuf(gtkgui_helpers.get_pep_as_pixbuf(pep[pep_type]))
-            img.set_tooltip_markup(pep[pep_type].asMarkupText())
+            img.set_tooltip_markup(pep[pep_type].as_markup_text())
             img.show()
         else:
             img.hide()
@@ -437,7 +427,7 @@ class ChatControl(ChatControlBase):
         if obj.jid != self.contact.jid:
             return
 
-        if obj.pep_type == 'nickname':
+        if obj.pep_type == 'nick':
             self.update_ui()
             self.parent_win.redraw_tab(self)
             self.parent_win.show_title()
@@ -511,9 +501,9 @@ class ChatControl(ChatControlBase):
         if jingle_type not in ('audio', 'video'):
             return
         if state in ('connecting', 'connected', 'stop', 'error') and reason:
-            str = _('%(type)s state : %(state)s, reason: %(reason)s') % {
+            info = _('%(type)s state : %(state)s, reason: %(reason)s') % {
                     'type': jingle_type.capitalize(), 'state': state, 'reason': reason}
-            self.print_conversation(str, 'info')
+            self.print_conversation(info, 'info')
 
         states = {'connecting': self.JINGLE_STATE_CONNECTING,
                 'connection_received': self.JINGLE_STATE_CONNECTION_RECEIVED,
@@ -535,7 +525,8 @@ class ChatControl(ChatControlBase):
         if state in ('connection_received', 'connecting'):
             setattr(self, jingle_type + '_sid', sid)
 
-        getattr(self, '_' + jingle_type + '_button').set_active(jingle_state != self.JINGLE_STATE_NULL)
+        v = GLib.Variant.new_boolean(jingle_state != self.JINGLE_STATE_NULL)
+        getattr(self, jingle_type + '_action').change_state(v)
 
         getattr(self, 'update_' + jingle_type)()
 
@@ -599,8 +590,8 @@ class ChatControl(ChatControlBase):
         return True
 
     def on_location_eventbox_button_release_event(self, widget, event):
-        if 'location' in self.contact.pep:
-            location = self.contact.pep['location']._pep_specific_data
+        if 'geoloc' in self.contact.pep:
+            location = self.contact.pep['geoloc'].data
             if ('lat' in location) and ('lon' in location):
                 uri = 'https://www.openstreetmap.org/?' + \
                         'mlat=%(lat)s&mlon=%(lon)s&zoom=16' % {'lat': location['lat'],
@@ -624,13 +615,12 @@ class ChatControl(ChatControlBase):
         self.update_toolbar()
 
     def _update_banner_state_image(self):
-        contact = app.contacts.get_contact_with_highest_priority(self.account,
-                self.contact.jid)
+        contact = app.contacts.get_contact_with_highest_priority(
+            self.account, self.contact.jid)
         if not contact or self.resource:
             # For transient contacts
             contact = self.contact
         show = contact.show
-        jid = contact.jid
 
         # Set banner image
         icon = gtkgui_helpers.get_iconset_name_for(show)
@@ -671,7 +661,8 @@ class ChatControl(ChatControlBase):
                 if other_contact_.get_shown_name() == \
                 self.contact.get_shown_name():
                     acct_info = i18n.direction_mark + ' (%s)' % \
-                        GLib.markup_escape_text(self.account)
+                        GLib.markup_escape_text(
+                            app.get_account_label(self.account))
                     break
 
         status = contact.status
@@ -683,9 +674,12 @@ class ChatControl(ChatControlBase):
             status_reduced = ''
         status_escaped = GLib.markup_escape_text(status_reduced)
 
-        font_attrs, font_attrs_small = self.get_font_attrs()
         st = app.config.get('displayed_chat_state_notifications')
-        cs = contact.chatstate
+        if self.TYPE_ID == 'pm':
+            cs = self.gc_contact.chatstate
+        else:
+            cs = app.contacts.get_combined_chatstate(
+                self.account, self.contact.jid)
         if cs and st in ('composing_only', 'all'):
             if contact.show == 'offline':
                 chatstate = ''
@@ -694,22 +688,21 @@ class ChatControl(ChatControlBase):
             else:
                 chatstate = ''
 
-            label_text = '<span %s>%s</span><span %s>%s %s</span>' \
-                % (font_attrs,  name, font_attrs_small, acct_info, chatstate)
+            label_text = '<span>%s</span><span size="x-small" weight="light">%s %s</span>' \
+                % (name, acct_info, chatstate)
             if acct_info:
                 acct_info = i18n.direction_mark + ' ' + acct_info
             label_tooltip = '%s%s %s' % (name, acct_info, chatstate)
         else:
-            # weight="heavy" size="x-large"
-            label_text = '<span %s>%s</span><span %s>%s</span>' % \
-                    (font_attrs, name, font_attrs_small, acct_info)
+            label_text = '<span>%s</span><span size="x-small" weight="light">%s</span>' % \
+                    (name, acct_info)
             if acct_info:
                 acct_info = i18n.direction_mark + ' ' + acct_info
             label_tooltip = '%s%s' % (name, acct_info)
 
         if status_escaped:
             status_text = self.urlfinder.sub(self.make_href, status_escaped)
-            status_text = '<span %s>%s</span>' % (font_attrs_small, status_text)
+            status_text = '<span size="x-small" weight="light">%s</span>' % status_text
             self.banner_status_label.set_tooltip_text(status)
             self.banner_status_label.set_no_show_all(False)
             self.banner_status_label.show()
@@ -735,14 +728,11 @@ class ChatControl(ChatControlBase):
             content = session.get_content(jingle_type)
             if content:
                 session.remove_content(content.creator, content.name)
-        getattr(self, '_' + jingle_type + '_button').set_active(False)
+        v = GLib.Variant.new_boolean(False)
+        getattr(self, jingle_type + '_action').change_state(v)
         getattr(self, 'update_' + jingle_type)()
 
     def on_jingle_button_toggled(self, state, jingle_type):
-        img_name = 'gajim-%s_%s' % ({'audio': 'mic', 'video': 'cam'}[jingle_type],
-                        {True: 'active', False: 'inactive'}[state])
-        path_to_img = gtkgui_helpers.get_icon_path(img_name)
-
         if state:
             if getattr(self, jingle_type + '_state') == \
             self.JINGLE_STATE_NULL:
@@ -779,12 +769,7 @@ class ChatControl(ChatControlBase):
             fixed.set_no_show_all(True)
             self.close_jingle_content(jingle_type)
 
-        img = getattr(self, '_' + jingle_type + '_button').get_property('image')
-        img.set_from_file(path_to_img)
-
     def set_lock_image(self):
-        loggable = self.session and self.session.is_loggable()
-
         encryption_state = {'visible': self.encryption is not None,
                             'enc_type': self.encryption,
                             'authenticated': False}
@@ -812,7 +797,6 @@ class ChatControl(ChatControlBase):
 
         self.authentication_button.set_tooltip_text(tooltip)
         self.widget_set_visible(self.authentication_button, not visible)
-        context = self.msg_scrolledwindow.get_style_context()
         self.lock_image.set_sensitive(visible)
 
     def _on_authentication_button_clicked(self, widget):
@@ -823,17 +807,53 @@ class ChatControl(ChatControlBase):
     def _nec_mam_decrypted_message_received(self, obj):
         if obj.conn.name != self.account:
             return
-        if obj.with_ != self.contact.jid:
-            return
+
+        if obj.muc_pm:
+            if not obj.with_ == self.contact.get_full_jid():
+                return
+        else:
+            if not obj.with_.bareMatch(self.contact.jid):
+                return
 
         kind = '' # incoming
         if obj.kind == KindConstant.CHAT_MSG_SENT:
             kind = 'outgoing'
 
-        self.print_conversation(obj.msgtxt, kind, tim=obj.timestamp,
-            encrypted=obj.encrypted, correct_id=obj.correct_id, 
+        self.print_conversation(
+            obj.msgtxt, kind, tim=obj.timestamp,
+            encrypted=obj.encrypted, correct_id=obj.correct_id,
             msg_stanza_id=obj.message_id, additional_data=obj.additional_data)
 
+    def _nec_decrypted_message_received(self, obj):
+        if not obj.msgtxt:
+            return True
+        if obj.conn.name != self.account:
+            return
+        if obj.mtype != 'chat':
+            return
+        if obj.session.control != self:
+            return
+
+        typ = ''
+        xep0184_id = None
+        if obj.mtype == 'error':
+            typ = 'error'
+        if obj.forwarded and obj.sent:
+            typ = 'out'
+            if obj.jid != app.get_jid_from_account(obj.conn.name):
+                xep0184_id = obj.id_
+        self.print_conversation(obj.msgtxt, typ,
+            tim=obj.timestamp, encrypted=obj.encrypted, subject=obj.subject,
+            xhtml=obj.xhtml, displaymarking=obj.displaymarking,
+            msg_log_id=obj.msg_log_id, msg_stanza_id=obj.id_, correct_id=obj.correct_id,
+            xep0184_id=xep0184_id, additional_data=obj.additional_data)
+        if obj.msg_log_id:
+            pw = self.parent_win
+            end = self.conv_textview.autoscroll
+            if not pw or (pw.get_active_control() and self \
+            == pw.get_active_control() and pw.is_active() and end):
+                app.logger.set_read_messages([obj.msg_log_id])
+
     def _message_sent(self, obj):
         if obj.conn.name != self.account:
             return
@@ -855,7 +875,7 @@ class ChatControl(ChatControlBase):
         if self.correcting:
             self.correcting = False
             gtkgui_helpers.remove_css_class(
-                self.msg_textview, 'msgcorrectingcolor')
+                self.msg_textview, 'gajim-msg-correcting')
 
         self.print_conversation(obj.message, self.contact.jid, tim=obj.timestamp,
             encrypted=obj.encrypted, xep0184_id=xep0184_id, xhtml=obj.xhtml,
@@ -863,8 +883,8 @@ class ChatControl(ChatControlBase):
             correct_id=obj.correct_id,
             additional_data=obj.additional_data)
 
-    def send_message(self, message, keyID='', chatstate=None, xhtml=None,
-    process_commands=True, attention=False):
+    def send_message(self, message, keyID='', xhtml=None,
+                     process_commands=True, attention=False):
         """
         Send a message to contact
         """
@@ -883,68 +903,13 @@ class ChatControl(ChatControlBase):
         contact = self.contact
         keyID = contact.keyID
 
-        chatstates_on = app.config.get('outgoing_chat_state_notifications') != \
-                'disabled'
-
-        chatstate_to_send = None
-        if contact is not None:
-            if contact.supports(NS_CHATSTATES):
-                # send active chatstate on every message (as XEP says)
-                chatstate_to_send = 'active'
-                contact.our_chatstate = 'active'
-
-                self._schedule_activity_timers()
-
-        ChatControlBase.send_message(self, message, keyID, type_='chat',
-            chatstate=chatstate_to_send, xhtml=xhtml,
-            process_commands=process_commands, attention=attention)
-
-    def on_cancel_session_negotiation(self):
-        msg = _('Session negotiation cancelled')
-        ChatControlBase.print_conversation_line(self, msg, 'status', '', None)
-
-    def print_archiving_session_details(self):
-        """
-        Print esession settings to textview
-        """
-        archiving = bool(self.session) and isinstance(self.session,
-                ArchivingStanzaSession) and self.session.archiving
-        if archiving:
-            msg = _('This session WILL be archived on server')
-        else:
-            msg = _('This session WILL NOT be archived on server')
-        ChatControlBase.print_conversation_line(self, msg, 'status', '', None)
-
-    def print_esession_details(self):
-        """
-        Print esession settings to textview
-        """
-        e2e_is_active = bool(self.session) and self.session.enable_encryption
-        if e2e_is_active:
-            msg = _('This session is encrypted')
-
-            if self.session.is_loggable():
-                msg += _(' and WILL be logged')
-            else:
-                msg += _(' and WILL NOT be logged')
-
-            ChatControlBase.print_conversation_line(self, msg, 'status', '', None)
-
-            if not self.session.verified_identity:
-                ChatControlBase.print_conversation_line(self, _("Remote contact's identity not verified. Click the shield button for more details."), 'status', '', None)
-        else:
-            msg = _('E2E encryption disabled')
-            ChatControlBase.print_conversation_line(self, msg, 'status', '', None)
-
-        self._show_lock_image(e2e_is_active, 'E2E',
-                              self.session and self.session.verified_identity)
-
-    def print_session_details(self, old_session=None):
-        if isinstance(self.session, EncryptedStanzaSession) or \
-        (old_session and isinstance(old_session, EncryptedStanzaSession)):
-            self.print_esession_details()
-        elif isinstance(self.session, ArchivingStanzaSession):
-            self.print_archiving_session_details()
+        ChatControlBase.send_message(self,
+                                     message,
+                                     keyID,
+                                     type_='chat',
+                                     xhtml=xhtml,
+                                     process_commands=process_commands,
+                                     attention=attention)
 
     def get_our_nick(self):
         return app.nicks[self.account]
@@ -961,9 +926,9 @@ class ChatControl(ChatControlBase):
                 status and error is mainly that with error, msg count as a new message
                 (in systray and in control).
         If frm is set to info: it's a information message.
-        If frm is set to print_queue: it is incomming from queue.
+        If frm is set to print_queue: it is incoming from queue.
         If frm is set to another value: it's an outgoing message.
-        If frm is not set: it's an incomming message.
+        If frm is not set: it's an incoming message.
         """
         contact = self.contact
 
@@ -1008,6 +973,14 @@ class ChatControl(ChatControlBase):
         else:
             self.old_msg_kind = kind
 
+    def _receipt_received(self, event):
+        if event.conn.name != self.account:
+            return
+        if event.jid != self.contact.jid:
+            return
+
+        self.conv_textview.show_xep0184_ack(event.receipt_id)
+
     def get_tab_label(self):
         unread = ''
         if self.resource:
@@ -1069,7 +1042,7 @@ class ChatControl(ChatControlBase):
     def prepare_context_menu(self, hide_buttonbar_items=False):
         """
         Set compact view menuitem active state sets active and sensitivity state
-        for history_menuitem (False for tranasports) and file_transfer_menuitem 
+        for history_menuitem (False for tranasports) and file_transfer_menuitem
         and hide()/show() for add_to_roster_menuitem
         """
         if app.jid_is_transport(self.contact.jid):
@@ -1082,79 +1055,6 @@ class ChatControl(ChatControlBase):
                 show_buttonbar_items=not hide_buttonbar_items)
         return menu
 
-    def send_chatstate(self, state, contact=None):
-        """
-        Send OUR chatstate as STANDLONE chat state message (eg. no body)
-        to contact only if new chatstate is different from the previous one
-        if jid is not specified, send to active tab
-        """
-        # JEP 85 does not allow resending the same chatstate
-        # this function checks for that and just returns so it's safe to call it
-        # with same state.
-
-        # This functions also checks for violation in state transitions
-        # and raises RuntimeException with appropriate message
-        # more on that http://xmpp.org/extensions/xep-0085.html#statechart
-
-        # do not send if we have chat state notifications disabled
-        # that means we won't reply to the <active/> from other peer
-        # so we do not broadcast jep85 capabalities
-        chatstate_setting = app.config.get('outgoing_chat_state_notifications')
-        if chatstate_setting == 'disabled':
-            return
-
-        # Dont leak presence to contacts
-        # which are not allowed to see our status
-        if contact and contact.sub in ('to', 'none'):
-            return
-
-        if self.contact.jid == app.get_jid_from_account(self.account):
-            return
-
-        elif chatstate_setting == 'composing_only' and state != 'active' and\
-                state != 'composing':
-            return
-
-        if contact is None:
-            contact = self.parent_win.get_active_contact()
-            if contact is None:
-                # contact was from pm in MUC, and left the room so contact is None
-                # so we cannot send chatstate anymore
-                return
-
-        # Don't send chatstates to offline contacts
-        if contact.show == 'offline':
-            return
-
-        if not contact.supports(NS_CHATSTATES):
-            return
-        if contact.our_chatstate == False:
-            return
-
-        # if the new state we wanna send (state) equals
-        # the current state (contact.our_chatstate) then return
-        if contact.our_chatstate == state:
-            return
-
-        # if wel're inactive prevent composing (XEP violation)
-        if contact.our_chatstate == 'inactive' and state == 'composing':
-            # go active before
-            app.log('chatstates').info('%-10s - %s', 'active', self.contact.jid)
-            app.nec.push_outgoing_event(MessageOutgoingEvent(None,
-                account=self.account, jid=self.contact.jid, chatstate='active',
-                control=self))
-            contact.our_chatstate = 'active'
-            self.reset_kbd_mouse_timeout_vars()
-
-        app.log('chatstates').info('%-10s - %s', state, self.contact.jid)
-        app.nec.push_outgoing_event(MessageOutgoingEvent(None,
-            account=self.account, jid=self.contact.jid, chatstate=state,
-            control=self))
-
-        contact.our_chatstate = state
-        if state == 'active':
-            self.reset_kbd_mouse_timeout_vars()
-
     def shutdown(self):
         # PluginSystem: removing GUI extension points connected with ChatControl
         # instance object
@@ -1165,24 +1065,27 @@ class ChatControl(ChatControlBase):
         if self.TYPE_ID == message_control.TYPE_CHAT:
             app.ged.remove_event_handler('update-roster-avatar', ged.GUI1,
                 self._nec_update_avatar)
-        app.ged.remove_event_handler('failed-decrypt', ged.GUI1,
-            self._nec_failed_decrypt)
         app.ged.remove_event_handler('chatstate-received', ged.GUI1,
             self._nec_chatstate_received)
-        app.ged.remove_event_handler('caps-received', ged.GUI1,
+        app.ged.remove_event_handler('caps-update', ged.GUI1,
             self._nec_caps_received)
         app.ged.remove_event_handler('message-sent', ged.OUT_POSTCORE,
             self._message_sent)
         app.ged.remove_event_handler(
             'mam-decrypted-message-received',
             ged.GUI1, self._nec_mam_decrypted_message_received)
+        app.ged.remove_event_handler(
+            'decrypted-message-received',
+            ged.GUI1, self._nec_decrypted_message_received)
+        app.ged.remove_event_handler(
+            'receipt-received',
+            ged.GUI1, self._receipt_received)
 
         self.unsubscribe_events()
 
         # Send 'gone' chatstate
-        self.send_chatstate('gone', self.contact)
-        self.contact.chatstate = None
-        self.contact.our_chatstate = None
+        con = app.connections[self.account]
+        con.get_module('Chatstate').set_chatstate(self.contact, Chatstate.GONE)
 
         for jingle_type in ('audio', 'video'):
             self.close_jingle_content(jingle_type)
@@ -1233,7 +1136,7 @@ class ChatControl(ChatControlBase):
             def on_cancel():
                 on_no(self)
 
-            dialogs.ConfirmationDialog(
+            ConfirmationDialog(
                 #%s is being replaced in the code with JID
                 _('You just received a new message from "%s"') % \
                 self.contact.jid,
@@ -1244,13 +1147,26 @@ class ChatControl(ChatControlBase):
             return
         on_yes(self)
 
-    def _nec_chatstate_received(self, obj):
-        """
-        Handle incoming chatstate that jid SENT TO us
-        """
+    def _nec_chatstate_received(self, event):
+        if event.account != self.account:
+            return
+
+        if self.TYPE_ID == 'pm':
+            if event.contact != self.gc_contact:
+                return
+        else:
+            if event.contact.jid != self.contact.jid:
+                return
+
         self.draw_banner_text()
+
         # update chatstate in tab for this chat
-        self.parent_win.redraw_tab(self, self.contact.chatstate)
+        if event.contact.is_gc_contact:
+            chatstate = event.contact.chatstate
+        else:
+            chatstate = app.contacts.get_combined_chatstate(
+                self.account, self.contact.jid)
+        self.parent_win.redraw_tab(self, chatstate)
 
     def _nec_caps_received(self, obj):
         if obj.conn.name != self.account:
@@ -1261,14 +1177,16 @@ class ChatControl(ChatControlBase):
             return
         self.update_ui()
 
-    def _nec_ping_reply(self, obj):
-        if obj.control:
-            if obj.control != self:
-                return
-        else:
-            if self.contact != obj.contact:
-                return
-        self.print_conversation(_('Pong! (%s s.)') % obj.seconds, 'status')
+    def _nec_ping(self, obj):
+        if self.contact != obj.contact:
+            return
+        if obj.name == 'ping-sent':
+            self.print_conversation(_('Ping?'), 'status')
+        elif obj.name == 'ping-reply':
+            self.print_conversation(
+                _('Pong! (%s seconds)') % obj.seconds, 'status')
+        elif obj.name == 'ping-error':
+            self.print_conversation(_('Error.'), 'status')
 
     def show_avatar(self):
         if not app.config.get('show_avatar_in_chat'):
@@ -1328,12 +1246,6 @@ class ChatControl(ChatControlBase):
 
             dialogs.TransformChatToMUC(self.account, [c.jid], [dropped_jid])
 
-    def _on_message_tv_buffer_changed(self, textbuffer):
-        super()._on_message_tv_buffer_changed(textbuffer)
-        if textbuffer.get_char_count() and self.encryption:
-            app.plugin_manager.extension_point(
-                'typing' + self.encryption, self)
-
     def restore_conversation(self):
         jid = self.contact.jid
         # don't restore lines if it's a transport
@@ -1390,7 +1302,7 @@ class ChatControl(ChatControlBase):
                 local_old_kind = None
             else:
                 local_old_kind = kind
-        if len(rows):
+        if rows:
             self.conv_textview.print_empty_line()
 
     def read_queue(self):
@@ -1441,7 +1353,7 @@ class ChatControl(ChatControlBase):
             typ = 'pm'
 
         self.redraw_after_event_removed(jid)
-        if (self.contact.show in ('offline', 'error')):
+        if self.contact.show in ('offline', 'error'):
             show_offline = app.config.get('showoffline')
             show_transports = app.config.get('show_transports_group')
             if (not show_transports and app.jid_is_transport(jid)) or \
@@ -1458,57 +1370,6 @@ class ChatControl(ChatControlBase):
         """
         dialogs.TransformChatToMUC(self.account, [self.contact.jid])
 
-    def activate_esessions(self):
-        if not (self.session and self.session.enable_encryption):
-            self.begin_e2e_negotiation()
-
-    def terminate_esessions(self):
-        if not (self.session and self.session.enable_encryption):
-            return
-        # e2e was enabled, disable it
-        jid = str(self.session.jid)
-        thread_id = self.session.thread_id
-
-        self.session.terminate_e2e()
-
-        app.connections[self.account].delete_session(jid, thread_id)
-
-        # presumably the user had a good reason to shut it off, so
-        # disable autonegotiation too
-        self.no_autonegotiation = True
-
-    def begin_negotiation(self):
-        self.no_autonegotiation = True
-
-        if not self.session:
-            fjid = self.contact.get_full_jid()
-            new_sess = app.connections[self.account].make_new_session(fjid, type_=self.type_id)
-            self.set_session(new_sess)
-
-    def begin_e2e_negotiation(self):
-        self.begin_negotiation()
-        self.session.resource = self.contact.resource
-        self.session.negotiate_e2e(False)
-
-    def _nec_failed_decrypt(self, obj):
-        if obj.session != self.session:
-            return
-
-        details = _('Unable to decrypt message from %s\nIt may have been '
-            'tampered with.') % obj.fjid
-        self.print_conversation_line(details, 'status', '', obj.timestamp)
-
-        # terminate the session
-        thread_id = self.session.thread_id
-        self.session.terminate_e2e()
-        obj.conn.delete_session(obj.fjid, thread_id)
-
-        # restart the session
-        self.begin_e2e_negotiation()
-
-        # Stop emission so it doesn't go to gui_interface
-        return True
-
     def got_connected(self):
         ChatControlBase.got_connected(self)
         # Refreshing contact
@@ -1532,13 +1393,11 @@ class ChatControl(ChatControlBase):
         self.update_ui()
         self.parent_win.redraw_tab(self)
 
-        self.print_conversation(_('%(name)s is now %(status)s') % {'name': name,
-                'status': uf_show}, 'status')
-
         if status:
-            self.print_conversation(' (', 'status', simple=True)
-            self.print_conversation('%s' % (status), 'status', simple=True)
-            self.print_conversation(')', 'status', simple=True)
+            status = '- %s' % status
+        status_line = _('%(name)s is now %(show)s %(status)s') % {
+            'name': name, 'show': uf_show, 'status': status or ''}
+        self.print_conversation(status_line, 'status')
 
     def _info_bar_show_message(self):
         if self.info_bar.get_visible():
@@ -1546,7 +1405,7 @@ class ChatControl(ChatControlBase):
             return
         if not self.info_bar_queue:
             return
-        markup, buttons, args, type_ = self.info_bar_queue[0]
+        markup, buttons, _args, type_ = self.info_bar_queue[0]
         self.info_bar_label.set_markup(markup)
 
         # Remove old buttons
