@@ -1,31 +1,28 @@
-# -*- coding:utf-8 -*-
-## src/common/events.py
-##
-## Copyright (C) 2006 Jean-Marie Traissard <jim AT lapin.org>
-##                    Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006-2017 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
-## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
-## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
-##                    Jonathan Schleifer <js-gajim AT webkeks.org>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2006 Jean-Marie Traissard <jim AT lapin.org>
+#                    Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
+# Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+# Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+#                    Jonathan Schleifer <js-gajim AT webkeks.org>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 import time
 
+
 class Event:
     """
     Information concerning each event
@@ -64,16 +61,16 @@ class Event:
 
 class ChatEvent(Event):
     type_ = 'chat'
-    def __init__ (self, message, subject, kind, time, encrypted, resource,
+    def __init__(self, message, subject, kind, time_, encrypted, resource,
     msg_log_id, correct_id=None, xhtml=None, session=None, form_node=None,
-    displaymarking=None, sent_forwarded=False, time_=None, show_in_roster=False,
-    show_in_systray=True):
+    displaymarking=None, sent_forwarded=False, show_in_roster=False,
+    show_in_systray=True, additional_data=None):
         Event.__init__(self, time_, show_in_roster=show_in_roster,
             show_in_systray=show_in_systray)
         self.message = message
         self.subject = subject
         self.kind = kind
-        self.time = time
+        self.time = time_
         self.encrypted = encrypted
         self.resource = resource
         self.msg_log_id = msg_log_id
@@ -83,6 +80,9 @@ class ChatEvent(Event):
         self.form_node = form_node
         self.displaymarking = displaymarking
         self.sent_forwarded = sent_forwarded
+        if additional_data is None:
+            additional_data = {}
+        self.additional_data = additional_data
 
 class NormalEvent(ChatEvent):
     type_ = 'normal'
@@ -129,7 +129,7 @@ class UnsubscribedEvent(Event):
 
 class GcInvitationtEvent(Event):
     type_ = 'gc-invitation'
-    def __init__(self, room_jid, reason, password, is_continued, from_jid,
+    def __init__(self, room_jid, reason, password, is_continued, jid_from,
     time_=None, show_in_roster=False, show_in_systray=True):
         Event.__init__(self, time_, show_in_roster=show_in_roster,
             show_in_systray=show_in_systray)
@@ -137,7 +137,7 @@ class GcInvitationtEvent(Event):
         self.reason = reason
         self.password = password
         self.is_continued = is_continued
-        self.from_jid = from_jid
+        self.jid_from = jid_from
 
 class FileRequestEvent(Event):
     type_ = 'file-request'
@@ -246,11 +246,13 @@ class Events:
         event.account = account
         self.fire_event_added(event)
 
-    def remove_events(self, account, jid, event = None, types = []):
+    def remove_events(self, account, jid, event=None, types=None):
         """
         If event is not specified, remove all events from this jid, optionally
         only from given type return True if no such event found
         """
+        if types is None:
+            types = []
         if account not in self._events:
             return True
         if jid not in self._events[account]:
@@ -296,15 +298,19 @@ class Events:
             self._events[account][new_jid] = self._events[account][old_jid]
         del self._events[account][old_jid]
 
-    def get_nb_events(self, types = [], account = None):
-        return self._get_nb_events(types = types, account = account)
+    def get_nb_events(self, types=None, account=None):
+        if types is None:
+            types = []
+        return self._get_nb_events(types=types, account=account)
 
-    def get_events(self, account, jid = None, types = []):
+    def get_events(self, account, jid=None, types=None):
         """
         Return all events from the given account of the form {jid1: [], jid2:
         []}. If jid is given, returns all events from the given jid in a list: []
         optionally only from given type
         """
+        if types is None:
+            types = []
         if account not in self._events:
             return []
         if not jid:
@@ -341,11 +347,12 @@ class Events:
                 first_event = event
         return first_event
 
-    def _get_nb_events(self, account = None, jid = None, attribute = None, types
-                    = []):
+    def _get_nb_events(self, account=None, jid=None, attribute=None, types=None):
         """
         Return the number of pending events
         """
+        if types is None:
+            types = []
         nb = 0
         if account:
             accounts = [account]
@@ -410,11 +417,13 @@ class Events:
                         first_event = event
         return first_account, first_jid, first_event
 
-    def get_nb_systray_events(self, types = []):
+    def get_nb_systray_events(self, types=None):
         """
         Return the number of events displayed in roster
         """
-        return self._get_nb_events(attribute = 'systray', types = types)
+        if types is None:
+            types = []
+        return self._get_nb_events(attribute='systray', types=types)
 
     def get_systray_events(self):
         """
@@ -427,12 +436,14 @@ class Events:
         events = self.get_systray_events()
         return self._get_first_event_with_attribute(events)
 
-    def get_nb_roster_events(self, account = None, jid = None, types = []):
+    def get_nb_roster_events(self, account=None, jid=None, types=None):
         """
         Return the number of events displayed in roster
         """
-        return self._get_nb_events(attribute = 'roster', account = account,
-                jid = jid, types = types)
+        if types is None:
+            types = []
+        return self._get_nb_events(attribute='roster', account=account,
+                jid=jid, types=types)
 
     def get_roster_events(self):
         """
