@@ -1,33 +1,29 @@
-# -*- coding:utf-8 -*-
-## src/common/helpers.py
-##
-## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
-##                         Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006 Alex Mauer <hawke AT hawkesnest.net>
-## Copyright (C) 2006-2007 Travis Shirk <travis AT pobox.com>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
-##                    James Newton <redshodan AT gmail.com>
-##                    Julien Pivotto <roidelapluie AT gmail.com>
-## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
-## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
-##                    Jonathan Schleifer <js-gajim AT webkeks.org>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
+#                         Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2006 Alex Mauer <hawke AT hawkesnest.net>
+# Copyright (C) 2006-2007 Travis Shirk <travis AT pobox.com>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+#                    James Newton <redshodan AT gmail.com>
+#                    Julien Pivotto <roidelapluie AT gmail.com>
+# Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+# Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+#                    Jonathan Schleifer <js-gajim AT webkeks.org>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 import sys
 import re
@@ -40,32 +36,30 @@ import select
 import base64
 import hashlib
 import shlex
-from gajim.common import caps_cache
 import socket
 import time
-from datetime import datetime, timedelta, timezone, tzinfo
+import logging
+import json
+from datetime import datetime, timedelta
 from distutils.version import LooseVersion as V
-
 from encodings.punycode import punycode_encode
 from string import Template
 
 import nbxmpp
 
+from gajim.common import caps_cache
+from gajim.common import configpaths
 from gajim.common.i18n import Q_
+from gajim.common.i18n import _
 from gajim.common.i18n import ngettext
+from gajim.common.caps_cache import muc_caps_cache
 
 try:
-    import precis_i18n.codec
+    import precis_i18n.codec  # pylint: disable=unused-import
     HAS_PRECIS_I18N = True
 except ImportError:
     HAS_PRECIS_I18N = False
 
-try:
-    import idna
-    HAS_IDNA = True
-except ImportError:
-    HAS_IDNA = False
-
 HAS_SOUND = True
 if sys.platform == 'win32':
     try:
@@ -89,86 +83,15 @@ try:
 except Exception:
     pass
 
-import logging
 log = logging.getLogger('gajim.c.helpers')
 
 special_groups = (_('Transports'), _('Not in Roster'), _('Observers'), _('Groupchats'))
 
-# Patterns for DateTime parsing XEP-0082
-PATTERN_DATETIME = re.compile(
-        r'([0-9]{4}-[0-9]{2}-[0-9]{2})'
-        r'T'
-        r'([0-9]{2}:[0-9]{2}:[0-9]{2})'
-        r'(\.[0-9]{0,6})?'
-        r'(?:[0-9]+)?'
-        r'(?:(Z)|(?:([-+][0-9]{2}):([0-9]{2})))$'
-        )
-
-PATTERN_DELAY = re.compile(
-        r'([0-9]{4}-[0-9]{2}-[0-9]{2})'
-        r'T'
-        r'([0-9]{2}:[0-9]{2}:[0-9]{2})'
-        r'(\.[0-9]{0,6})?'
-        r'(?:[0-9]+)?'
-        r'(?:(Z)|(?:([-+][0]{2}):([0]{2})))$'
-        )
-
-ZERO = timedelta(0)
-HOUR = timedelta(hours=1)
-SECOND = timedelta(seconds=1)
-
-STDOFFSET = timedelta(seconds=-time.timezone)
-if time.daylight:
-    DSTOFFSET = timedelta(seconds=-time.altzone)
-else:
-    DSTOFFSET = STDOFFSET
-
-DSTDIFF = DSTOFFSET - STDOFFSET
-
-
-class LocalTimezone(tzinfo):
-    '''
-    A class capturing the platform's idea of local time.
-    May result in wrong values on historical times in
-    timezones where UTC offset and/or the DST rules had
-    changed in the past.
-    '''
-    def fromutc(self, dt):
-        assert dt.tzinfo is self
-        stamp = (dt - datetime(1970, 1, 1, tzinfo=self)) // SECOND
-        args = time.localtime(stamp)[:6]
-        dst_diff = DSTDIFF // SECOND
-        # Detect fold
-        fold = (args == time.localtime(stamp - dst_diff))
-        return datetime(*args, microsecond=dt.microsecond,
-                        tzinfo=self, fold=fold)
-
-    def utcoffset(self, dt):
-        if self._isdst(dt):
-            return DSTOFFSET
-        else:
-            return STDOFFSET
-
-    def dst(self, dt):
-        if self._isdst(dt):
-            return DSTDIFF
-        else:
-            return ZERO
-
-    def tzname(self, dt):
-        return 'local'
-
-    def _isdst(self, dt):
-        tt = (dt.year, dt.month, dt.day,
-              dt.hour, dt.minute, dt.second,
-              dt.weekday(), 0, 0)
-        stamp = time.mktime(tt)
-        tt = time.localtime(stamp)
-        return tt.tm_isdst > 0
 
 class InvalidFormat(Exception):
     pass
 
+
 def decompose_jid(jidstring):
     user = None
     server = None
@@ -259,9 +182,8 @@ def parse_resource(resource):
         try:
             if HAS_PRECIS_I18N:
                 return resource.encode('Nickname').decode('utf-8')
-            else:
-                from nbxmpp.stringprepare import resourceprep
-                return resourceprep.prepare(resource)
+            from nbxmpp.stringprepare import resourceprep
+            return resourceprep.prepare(resource)
         except UnicodeError:
             raise InvalidFormat('Invalid character in resource.')
 
@@ -290,23 +212,20 @@ def prep(user, server, resource):
 
     if not ip_address:
         if server is not None:
-            if server.endswith('.'): # RFC7622, 3.2
+            if server.endswith('.'):  # RFC7622, 3.2
                 server = server[:-1]
-            if len(server) < 1 or len(server.encode('utf-8')) > 1023:
+            if not server or len(server.encode('utf-8')) > 1023:
                 raise InvalidFormat(_('Server must be between 1 and 1023 bytes'))
             try:
-                if HAS_IDNA:
-                    server = idna.encode(server).decode('utf-8')
-                else:
-                    from nbxmpp.stringprepare import nameprep
-                    server = nameprep.prepare(server)
+                from nbxmpp.stringprepare import nameprep
+                server = nameprep.prepare(server)
             except UnicodeError:
                 raise InvalidFormat(_('Invalid character in hostname.'))
         else:
             raise InvalidFormat(_('Server address required.'))
 
     if user is not None:
-        if len(user) < 1 or len(user.encode('utf-8')) > 1023:
+        if not user or len(user.encode('utf-8')) > 1023:
             raise InvalidFormat(_('Username must be between 1 and 1023 bytes'))
         try:
             if HAS_PRECIS_I18N:
@@ -320,7 +239,7 @@ def prep(user, server, resource):
         user = None
 
     if resource is not None:
-        if len(resource) < 1 or len(resource.encode('utf-8')) > 1023:
+        if not resource or len(resource.encode('utf-8')) > 1023:
             raise InvalidFormat(_('Resource must be between 1 and 1023 bytes'))
         try:
             if HAS_PRECIS_I18N:
@@ -336,13 +255,11 @@ def prep(user, server, resource):
     if user:
         if resource:
             return '%s@%s/%s' % (user, server, resource)
-        else:
-            return '%s@%s' % (user, server)
-    else:
-        if resource:
-            return '%s/%s' % (server, resource)
-        else:
-            return server
+        return '%s@%s' % (user, server)
+
+    if resource:
+        return '%s/%s' % (server, resource)
+    return server
 
 def windowsify(s):
     if os.name == 'nt':
@@ -359,7 +276,7 @@ def temp_failure_retry(func, *args, **kwargs):
             else:
                 raise
 
-def get_uf_show(show, use_mnemonic = False):
+def get_uf_show(show, use_mnemonic=False):
     """
     Return a userfriendly string for dnd/xa/chat and make all strings
     translatable
@@ -415,11 +332,11 @@ def get_uf_show(show, use_mnemonic = False):
 def get_css_show_color(show):
     if show in ('online', 'chat', 'invisible'):
         return 'status-online'
-    elif show in ('offline', 'not in roster', 'requested'):
+    if show in ('offline', 'not in roster', 'requested'):
         return None
-    elif show in ('xa', 'dnd'):
+    if show in ('xa', 'dnd'):
         return 'status-dnd'
-    elif show in ('away'):
+    if show == 'away':
         return 'status-away'
 
 def get_uf_sub(sub):
@@ -446,7 +363,7 @@ def get_uf_ask(ask):
 
     return uf_ask
 
-def get_uf_role(role, plural = False):
+def get_uf_role(role, plural=False):
     ''' plural determines if you get Moderators or Moderator'''
     if role == 'none':
         role_name = Q_('?Group Chat Contact Role:None')
@@ -516,14 +433,14 @@ def get_uf_chatstate(chatstate):
     """
     if chatstate == 'active':
         return _('is paying attention to the conversation')
-    elif chatstate == 'inactive':
+    if chatstate == 'inactive':
         return _('is doing something else')
-    elif chatstate == 'composing':
+    if chatstate == 'composing':
         return _('is composing a message…')
-    elif chatstate == 'paused':
+    if chatstate == 'paused':
         #paused means he or she was composing but has stopped for a while
         return _('paused composing a message')
-    elif chatstate == 'gone':
+    if chatstate == 'gone':
         return _('has closed the chat window or tab')
     return ''
 
@@ -538,8 +455,7 @@ def is_in_path(command, return_abs_path=False):
             if command in os.listdir(directory):
                 if return_abs_path:
                     return os.path.join(directory, command)
-                else:
-                    return True
+                return True
         except OSError:
             # If the user has non directories in his path
             pass
@@ -577,16 +493,6 @@ def get_file_path_from_dnd_dropped_uri(uri):
         path = path[5:] # 5 is len('file:')
     return path
 
-def from_xs_boolean_to_python_boolean(value):
-    # this is xs:boolean so 'true', 'false', '1', '0'
-    # convert those to True/False (python booleans)
-    if value in ('1', 'true'):
-        val = True
-    else: # '0', 'false' or anything else
-        val = False
-
-    return val
-
 def get_xmpp_show(show):
     if show in ('online', 'offline'):
         return None
@@ -599,8 +505,8 @@ def sanitize_filename(filename):
     """
     # 48 is the limit
     if len(filename) > 48:
-        hash = hashlib.md5(filename.encode('utf-8'))
-        filename = base64.b64encode(hash.digest()).decode('utf-8')
+        hash_ = hashlib.md5(filename.encode('utf-8'))
+        filename = base64.b64encode(hash_.digest()).decode('utf-8')
 
     # make it latin chars only
     filename = punycode_encode(filename).decode('utf-8')
@@ -612,7 +518,7 @@ def sanitize_filename(filename):
 
     return filename
 
-def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
+def reduce_chars_newlines(text, max_chars=0, max_lines=0):
     """
     Cut the chars after 'max_chars' on each line and show only the first
     'max_lines'
@@ -680,59 +586,9 @@ def datetime_tuple(timestamp):
         tim = tim.timetuple()
     return tim
 
-def parse_datetime(timestring, check_utc=False, convert='utc', epoch=False):
-    '''
-    Parse a XEP-0082 DateTime Profile String
-    https://xmpp.org/extensions/xep-0082.html
-
-    :param timestring: a XEP-0082 DateTime profile formated string
-
-    :param check_utc:  if True, returns None if timestring is not
-                       a timestring expressing UTC
-
-    :param convert:    convert the given timestring to utc or local time
-
-    :param epoch:      if True, returns the time in epoch
-
-    Examples:
-    '2017-11-05T01:41:20Z'
-    '2017-11-05T01:41:20.123Z'
-    '2017-11-05T01:41:20.123+05:00'
-
-    return a datetime or epoch
-    '''
-    if convert not in (None, 'utc', 'local'):
-        raise TypeError('"%s" is not a valid value for convert')
-    if check_utc:
-        match = PATTERN_DELAY.match(timestring)
-    else:
-        match = PATTERN_DATETIME.match(timestring)
-
-    if match:
-        timestring = ''.join(match.groups(''))
-        strformat = '%Y-%m-%d%H:%M:%S%z'
-        if match.group(3):
-            # Fractional second addendum to Time
-            strformat = '%Y-%m-%d%H:%M:%S.%f%z'
-        if match.group(4):
-            # UTC string denoted by addition of the character 'Z'
-            timestring = timestring[:-1] + '+0000'
-        try:
-            date_time = datetime.strptime(timestring, strformat)
-        except ValueError:
-            pass
-        else:
-            if not check_utc and convert == 'utc':
-                date_time = date_time.astimezone(timezone.utc)
-            if convert == 'local':
-                date_time = date_time.astimezone(LocalTimezone())
-            if epoch:
-                return date_time.timestamp()
-            return date_time
-    return None
 
 from gajim.common import app
-if app.HAVE_PYCURL:
+if app.is_installed('PYCURL'):
     import pycurl
     from io import StringIO
 
@@ -827,8 +683,6 @@ def launch_browser_mailer(kind, uri):
 
 
 def launch_file_manager(path_to_open):
-    if not path_to_open.startswith('file://'):
-        uri = 'file://' + path_to_open
     if os.name == 'nt':
         try:
             os.startfile(path_to_open) # if pywin32 is installed we open
@@ -853,8 +707,7 @@ def play_sound(event):
     path_to_soundfile = app.config.get_per('soundevents', event, 'path')
     play_sound_file(path_to_soundfile)
 
-def check_soundfile_path(file_, dirs=(app.gajimpaths.data_root,
-app.DATA_DIR)):
+def check_soundfile_path(file_, dirs=None):
     """
     Check if the sound file exists
 
@@ -863,9 +716,13 @@ app.DATA_DIR)):
                                      (eg: ~/.gajim/sounds/, DATADIR/sounds...).
     :return      the path to file or None if it doesn't exists.
     """
+    if dirs is None:
+        dirs = [configpaths.get('MY_DATA'),
+                configpaths.get('DATA')]
+
     if not file_:
         return None
-    elif os.path.exists(file_):
+    if os.path.exists(file_):
         return file_
 
     for d in dirs:
@@ -874,8 +731,7 @@ app.DATA_DIR)):
             return d
     return None
 
-def strip_soundfile_path(file_, dirs=(app.gajimpaths.data_root,
-app.DATA_DIR), abs=True):
+def strip_soundfile_path(file_, dirs=None, abs_=True):
     """
     Remove knowns paths from a sound file
 
@@ -883,15 +739,19 @@ app.DATA_DIR), abs=True):
     So config have no hardcoded path        to DATA_DIR and text in textfield is shorther.
     param: file_: the filename to strip.
     param: dirs: list of knowns paths from which the filename should be stripped.
-    param:  abs: force absolute path on dirs
+    param:  abs_: force absolute path on dirs
     """
     if not file_:
         return None
 
+    if dirs is None:
+        dirs = [configpaths.get('MY_DATA'),
+                configpaths.get('DATA')]
+
     name = os.path.basename(file_)
     for d in dirs:
         d = os.path.join(d, 'sounds', name)
-        if abs:
+        if abs_:
             d = os.path.abspath(d)
         if file_ == d:
             return name
@@ -904,7 +764,7 @@ def play_sound_file(path_to_soundfile):
     path_to_soundfile = check_soundfile_path(path_to_soundfile)
     if path_to_soundfile is None:
         return
-    elif sys.platform == 'win32' and HAS_SOUND:
+    if sys.platform == 'win32' and HAS_SOUND:
         try:
             winsound.PlaySound(path_to_soundfile,
                     winsound.SND_FILENAME|winsound.SND_ASYNC)
@@ -914,7 +774,7 @@ def play_sound_file(path_to_soundfile):
         if app.config.get('soundplayer') == '':
             def _oss_play():
                 sndfile = wave.open(path_to_soundfile, 'rb')
-                (nc, sw, fr, nf, comptype, compname) = sndfile.getparams()
+                nc, sw, fr, nf, _comptype, _compname = sndfile.getparams()
                 dev = oss.open('/dev/dsp', 'w')
                 dev.setparameters(sw * 8, nc, fr)
                 dev.write(sndfile.readframes(nf))
@@ -969,14 +829,14 @@ def statuses_unified():
             return False
     return True
 
-def get_icon_name_to_show(contact, account = None):
+def get_icon_name_to_show(contact, account=None):
     """
     Get the icon name to show in online, away, requested, etc
     """
     if account and app.events.get_nb_roster_events(account, contact.jid):
         return 'event'
-    if account and app.events.get_nb_roster_events(account,
-    contact.get_full_jid()):
+    if account and app.events.get_nb_roster_events(
+        account, contact.get_full_jid()):
         return 'event'
     if account and account in app.interface.minimized_controls and \
     contact.jid in app.interface.minimized_controls[account] and app.interface.\
@@ -985,8 +845,7 @@ def get_icon_name_to_show(contact, account = None):
     if account and contact.jid in app.gc_connected[account]:
         if app.gc_connected[account][contact.jid]:
             return 'muc_active'
-        else:
-            return 'muc_inactive'
+        return 'muc_inactive'
     if contact.jid.find('@') <= 0: # if not '@' or '@' starts the jid ==> agent
         return contact.show
     if contact.sub in ('both', 'to'):
@@ -1004,6 +863,9 @@ def get_full_jid_from_iq(iq_obj):
     """
     Return the full jid (with resource) from an iq
     """
+    jid = iq_obj.getFrom()
+    if jid is None:
+        return None
     return parse_jid(str(iq_obj.getFrom()))
 
 def get_jid_from_iq(iq_obj):
@@ -1091,7 +953,7 @@ def get_chat_control(account, contact):
     if contact.resource:
         full_jid_with_resource += '/' + contact.resource
     highest_contact = app.contacts.get_contact_with_highest_priority(
-            account, contact.jid)
+        account, contact.jid)
 
     # Look for a chat control that has the given resource, or default to
     # one without resource
@@ -1100,12 +962,14 @@ def get_chat_control(account, contact):
 
     if ctrl:
         return ctrl
-    elif highest_contact and highest_contact.resource and \
-    contact.resource != highest_contact.resource:
+
+    if (highest_contact and
+        highest_contact.resource and
+            contact.resource != highest_contact.resource):
         return None
-    else:
-        # unknown contact or offline message
-        return app.interface.msg_win_mgr.get_control(contact.jid, account)
+
+    # unknown contact or offline message
+    return app.interface.msg_win_mgr.get_control(contact.jid, account)
 
 def get_notification_icon_tooltip_dict():
     """
@@ -1122,7 +986,7 @@ def get_notification_icon_tooltip_dict():
         account_name = account['name']
         account['event_lines'] = []
         # Gather events per-account
-        pending_events = app.events.get_events(account = account_name)
+        pending_events = app.events.get_events(account=account_name)
         messages, non_messages, total_messages, total_non_messages = {}, {}, 0, 0
         for jid in pending_events:
             for event in pending_events[jid]:
@@ -1143,7 +1007,7 @@ def get_notification_icon_tooltip_dict():
                         total_messages, total_messages, total_messages)
                 account['event_lines'].append(text)
             else:
-                for jid in messages.keys():
+                for jid in messages:
                     text = ngettext(
                             '%d message pending',
                             '%d messages pending',
@@ -1166,10 +1030,10 @@ def get_notification_icon_tooltip_dict():
                 text = ngettext(
                     '%d event pending',
                     '%d events pending',
-                    total_non_messages, total_non_messages,total_non_messages)
+                    total_non_messages, total_non_messages, total_non_messages)
                 account['event_lines'].append(text)
             else:
-                for jid in non_messages.keys():
+                for jid in non_messages:
                     text = ngettext('%d event pending', '%d events pending',
                         non_messages[jid], non_messages[jid], non_messages[jid])
                     text += ' ' + _('from user %s') % (jid)
@@ -1186,7 +1050,7 @@ def get_notification_icon_tooltip_text():
 
     accounts = get_notification_icon_tooltip_dict()
 
-    if len(accounts) == 0:
+    if not accounts:
         # No configured account
         return _('Gajim')
 
@@ -1208,7 +1072,7 @@ def get_notification_icon_tooltip_text():
     for account in accounts:
         account_name = account['name']
         # Set account status, if not set above
-        if (show_more_accounts):
+        if show_more_accounts:
             message = '\n' + indent_with + ' %s - %s'
             text += message % (account_name, get_account_status(account))
             # Account list shown, messages need to be indented more
@@ -1240,42 +1104,47 @@ def get_accounts_info():
             message = message.strip()
         if message != '':
             single_line += ': ' + message
-        account_label = app.config.get_per(
-            'accounts', account, 'account_label')
+        account_label = app.get_account_label(account)
         accounts.append({'name': account,
-                         'account_label': account_label or account,
+                         'account_label': account_label,
                          'status_line': single_line,
                          'show': status,
                          'message': message})
     return accounts
 
+def get_current_show(account):
+    if account not in app.connections:
+        return 'offline'
+    status = app.connections[account].connected
+    return app.SHOW_LIST[status]
 
 def get_iconset_path(iconset):
-    if os.path.isdir(os.path.join(app.DATA_DIR, 'iconsets', iconset)):
-        return os.path.join(app.DATA_DIR, 'iconsets', iconset)
-    elif os.path.isdir(os.path.join(app.MY_ICONSETS_PATH, iconset)):
-        return os.path.join(app.MY_ICONSETS_PATH, iconset)
+    if os.path.isdir(os.path.join(configpaths.get('DATA'), 'iconsets', iconset)):
+        return os.path.join(configpaths.get('DATA'), 'iconsets', iconset)
+    if os.path.isdir(os.path.join(configpaths.get('MY_ICONSETS'), iconset)):
+        return os.path.join(configpaths.get('MY_ICONSETS'), iconset)
 
 def get_mood_iconset_path(iconset):
-    if os.path.isdir(os.path.join(app.DATA_DIR, 'moods', iconset)):
-        return os.path.join(app.DATA_DIR, 'moods', iconset)
-    elif os.path.isdir(os.path.join(app.MY_MOOD_ICONSETS_PATH, iconset)):
-        return os.path.join(app.MY_MOOD_ICONSETS_PATH, iconset)
+    if os.path.isdir(os.path.join(configpaths.get('DATA'), 'moods', iconset)):
+        return os.path.join(configpaths.get('DATA'), 'moods', iconset)
+    if os.path.isdir(
+            os.path.join(configpaths.get('MY_MOOD_ICONSETS'), iconset)):
+        return os.path.join(configpaths.get('MY_MOOD_ICONSETS'), iconset)
 
 def get_activity_iconset_path(iconset):
-    if os.path.isdir(os.path.join(app.DATA_DIR, 'activities', iconset)):
-        return os.path.join(app.DATA_DIR, 'activities', iconset)
-    elif os.path.isdir(os.path.join(app.MY_ACTIVITY_ICONSETS_PATH,
+    if os.path.isdir(os.path.join(configpaths.get('DATA'), 'activities', iconset)):
+        return os.path.join(configpaths.get('DATA'), 'activities', iconset)
+    if os.path.isdir(os.path.join(configpaths.get('MY_ACTIVITY_ICONSETS'),
     iconset)):
-        return os.path.join(app.MY_ACTIVITY_ICONSETS_PATH, iconset)
+        return os.path.join(configpaths.get('MY_ACTIVITY_ICONSETS'), iconset)
 
 def get_transport_path(transport):
-    if os.path.isdir(os.path.join(app.DATA_DIR, 'iconsets', 'transports',
+    if os.path.isdir(os.path.join(configpaths.get('DATA'), 'iconsets', 'transports',
     transport)):
-        return os.path.join(app.DATA_DIR, 'iconsets', 'transports', transport)
-    elif os.path.isdir(os.path.join(app.MY_ICONSETS_PATH, 'transports',
+        return os.path.join(configpaths.get('DATA'), 'iconsets', 'transports', transport)
+    if os.path.isdir(os.path.join(configpaths.get('MY_ICONSETS'), 'transports',
     transport)):
-        return os.path.join(app.MY_ICONSETS_PATH, 'transports', transport)
+        return os.path.join(configpaths.get('MY_ICONSETS'), 'transports', transport)
     # No transport folder found, use default jabber one
     return get_iconset_path(app.config.get('iconset'))
 
@@ -1327,7 +1196,7 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
             keyID = 'UNKNOWN'
     return keyID
 
-def update_optional_features(account = None):
+def update_optional_features(account=None):
     if account:
         accounts = [account]
     else:
@@ -1354,13 +1223,10 @@ def update_optional_features(account = None):
             app.gajim_optional_features[a].append(nbxmpp.NS_CHATSTATES)
         if not app.config.get('ignore_incoming_xhtml'):
             app.gajim_optional_features[a].append(nbxmpp.NS_XHTML_IM)
-        if app.HAVE_PYCRYPTO \
-        and app.config.get_per('accounts', a, 'enable_esessions'):
-            app.gajim_optional_features[a].append(nbxmpp.NS_ESESSION)
         if app.config.get_per('accounts', a, 'answer_receipts'):
             app.gajim_optional_features[a].append(nbxmpp.NS_RECEIPTS)
         app.gajim_optional_features[a].append(nbxmpp.NS_JINGLE)
-        if app.HAVE_FARSTREAM:
+        if app.is_installed('FARSTREAM'):
             app.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_RTP)
             app.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_RTP_AUDIO)
             app.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_RTP_VIDEO)
@@ -1383,12 +1249,15 @@ def update_optional_features(account = None):
                     app.connections[a].status)
 
 def jid_is_blocked(account, jid):
-    return ((jid in app.connections[account].blocked_contacts) or \
-            app.connections[account].blocked_all)
+    con = app.connections[account]
+    return (jid in con.get_module('Blocking').blocked or
+            jid in con.get_module('PrivacyLists').blocked_contacts or
+            con.get_module('PrivacyLists').blocked_all)
 
 def group_is_blocked(account, group):
-    return ((group in app.connections[account].blocked_groups) or \
-            app.connections[account].blocked_all)
+    con = app.connections[account]
+    return (group in con.get_module('PrivacyLists').blocked_groups or
+            con.get_module('PrivacyLists').blocked_all)
 
 def get_subscription_request_msg(account=None):
     s = app.config.get_per('accounts', account, 'subscription_request_msg')
@@ -1397,15 +1266,7 @@ def get_subscription_request_msg(account=None):
     s = _('I would like to add you to my contact list.')
     if account:
         s = _('Hello, I am $name.') + ' ' + s
-        our_jid = app.get_jid_from_account(account)
-        vcard = app.connections[account].own_vcard
-        name = ''
-        if vcard:
-            if 'N' in vcard:
-                if 'GIVEN' in vcard['N'] and 'FAMILY' in vcard['N']:
-                    name = vcard['N']['GIVEN'] + ' ' + vcard['N']['FAMILY']
-            if not name and 'FN' in vcard:
-                name = vcard['FN']
+        name = app.connections[account].get_module('VCardTemp').get_vard_name()
         nick = app.nicks[account]
         if name and nick:
             name += ' (%s)' % nick
@@ -1415,7 +1276,6 @@ def get_subscription_request_msg(account=None):
         return s
 
 def replace_dataform_media(form, stanza):
-    import nbxmpp
     found = False
     for field in form.getTags('field'):
         for media in field.getTags('media'):
@@ -1492,8 +1352,7 @@ def _get_img_direct(attrs):
         req.add_header('User-Agent', 'Gajim ' + app.version)
         f = urllib.request.urlopen(req)
     except Exception as ex:
-        log.debug('Error loading image %s ' % attrs['src']  + str(ex))
-        pixbuf = None
+        log.debug('Error loading image %s ', attrs['src']  + str(ex))
         alt = attrs.get('alt', 'Broken image')
     else:
         # Wait 2s between each byte
@@ -1505,7 +1364,7 @@ def _get_img_direct(attrs):
         deadline = time.time() + (10 * (max_size / 1048576))
         while True:
             if time.time() > deadline:
-                log.debug('Timeout loading image %s ' % attrs['src'])
+                log.debug('Timeout loading image %s ', attrs['src'])
                 mem = ''
                 alt = attrs.get('alt', '')
                 if alt:
@@ -1515,7 +1374,7 @@ def _get_img_direct(attrs):
             try:
                 temp = f.read(100)
             except socket.timeout as ex:
-                log.debug('Timeout loading image %s ' % attrs['src'] + str(ex))
+                log.debug('Timeout loading image %s ', attrs['src'] + str(ex))
                 alt = attrs.get('alt', '')
                 if alt:
                     alt += '\n'
@@ -1539,9 +1398,9 @@ def _get_img_proxy(attrs, proxy):
     Download an image through a proxy. This function should be launched in a
     separated thread.
     """
-    if not app.HAVE_PYCURL:
+    if not app.is_installed('PYCURL'):
         return '', _('PyCURL is not installed')
-    mem, alt, max_size = '', '', 2*1024*1024
+    alt, max_size = '', 2*1024*1024
     if 'max_size' in attrs:
         max_size = attrs['max_size']
     try:
@@ -1567,7 +1426,6 @@ def _get_img_proxy(attrs, proxy):
             c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_HTTP)
         elif proxy['type'] == 'socks5':
             c.setopt(pycurl.PROXYTYPE, pycurl.PROXYTYPE_SOCKS5)
-        x = c.perform()
         c.close()
         t = b.getvalue()
         return (t, attrs.get('alt', ''))
@@ -1582,8 +1440,7 @@ def _get_img_proxy(attrs, proxy):
         else:
             alt += _('Error loading image')
     except Exception as ex:
-        log.debug('Error loading image %s ' % attrs['src']  + str(ex))
-        pixbuf = None
+        log.debug('Error loading image %s ', attrs['src']  + str(ex))
         alt = attrs.get('alt', 'Broken image')
     return ('', alt)
 
@@ -1599,51 +1456,52 @@ def version_condition(current_version, required_version):
     return True
 
 def get_available_emoticon_themes():
-    emoticons_themes = []
-    emoticons_data_path = os.path.join(app.DATA_DIR, 'emoticons')
-    font_theme_path = os.path.join(
-        app.DATA_DIR, 'emoticons', 'font-emoticons', 'emoticons_theme.py')
+    emoticons_themes = ['font']
+    files = []
+    dir_iterator = os.scandir(configpaths.get('EMOTICONS'))
+    for folder in dir_iterator:
+        if not folder.is_dir():
+            continue
+        file_iterator = os.scandir(folder.path)
+        for theme in file_iterator:
+            if theme.is_file():
+                files.append(theme.name)
 
-    folders = os.listdir(emoticons_data_path)
-    if os.path.isdir(app.MY_EMOTS_PATH):
-        folders += os.listdir(app.MY_EMOTS_PATH)
+    if os.path.isdir(configpaths.get('MY_EMOTS')):
+        files += os.listdir(configpaths.get('MY_EMOTS'))
 
-    file = 'emoticons_theme.py'
-    if os.name == 'nt' and not os.path.exists(font_theme_path):
-        # When starting Gajim from source .py files are available
-        # We test this with font-emoticons and fallback to .pyc files otherwise
-        file = 'emoticons_theme.pyc'
-
-    for theme in folders:
-        theme_path = os.path.join(emoticons_data_path, theme, file)
-        if os.path.exists(theme_path):
-            emoticons_themes.append(theme)
+    for file in files:
+        if file.endswith('.png'):
+            emoticons_themes.append(file[:-4])
     emoticons_themes.sort()
     return emoticons_themes
 
-def get_emoticon_theme_path(theme):
-    emoticons_data_path = os.path.join(app.DATA_DIR, 'emoticons', theme)
-    if os.path.exists(emoticons_data_path):
-        return emoticons_data_path
+def call_counter(func):
+    def helper(self, restart=False):
+        if restart:
+            self._connect_machine_calls = 0
+        self._connect_machine_calls += 1
+        return func(self, restart=False)
+    return helper
 
-    emoticons_user_path = os.path.join(app.MY_EMOTS_PATH, theme)
-    if os.path.exists(emoticons_user_path):
-        return emoticons_user_path
+def get_sync_threshold(jid, archive_info):
+    if archive_info is None or archive_info.sync_threshold is None:
+        if muc_caps_cache.supports(jid, 'muc#roomconfig_membersonly'):
+            threshold = app.config.get('private_room_sync_threshold')
+        else:
+            threshold = app.config.get('public_room_sync_threshold')
+        app.logger.set_archive_infos(jid, sync_threshold=threshold)
+        return threshold
+    return archive_info.sync_threshold
 
-def add_to_mam_blacklist(jid):
-    config_value = app.config.get('mam_blacklist')
-    if not config_value:
-        config_value = [jid]
-    else:
-        if jid in config_value:
-            return
-        config_value = config_value.split(',')
-        config_value.append(jid)
-    log.warning('Found not-compliant MUC. %s added to MAM Blacklist', jid)
-    app.config.set('mam_blacklist', ','.join(config_value))
+def load_json(path, key=None, default=None):
+    try:
+        with open(path, 'r') as file:
+            json_dict = json.loads(file.read())
+    except Exception:
+        log.exception('Parsing error')
+        return default
 
-def get_mam_blacklist():
-    config_value = app.config.get('mam_blacklist')
-    if not config_value:
-        return []
-    return config_value.split(',')
+    if key is None:
+        return json_dict
+    return json_dict.get(key, default)
