@@ -1,17 +1,16 @@
-##
-## Copyright (C) 2006 Gajim Team
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 """
 Handles Jingle RTP sessions (XEP 0167)
@@ -28,7 +27,7 @@ from gi.repository import Gst
 from gi.repository import GLib
 
 from gajim.common import app
-
+from gajim.common.i18n import _
 from gajim.common.jingle_transport import JingleTransportICEUDP
 from gajim.common.jingle_content import contents, JingleContent, JingleContentSetupException
 from gajim.common.connection_handlers_events import InformationEvent
@@ -43,7 +42,7 @@ class JingleRTPContent(JingleContent):
     def __init__(self, session, media, transport=None):
         if transport is None:
             transport = JingleTransportICEUDP(None)
-        JingleContent.__init__(self, session, transport)
+        JingleContent.__init__(self, session, transport, None)
         self.media = media
         self._dtmf_running = False
         self.farstream_media = {
@@ -121,7 +120,7 @@ class JingleRTPContent(JingleContent):
                 InformationEvent(
                     None, conn=self.session.connection, level='error',
                     pri_txt=_('%s configuration error') % text.capitalize(),
-                    sec_txt=_('Couldn’t setup %(text)s. Check your '
+                    sec_txt=_('Couldn’t set up %(text)s. Check your '
                     'configuration.\n\nPipeline was:\n%(pipeline)s\n\n'
                     'Error was:\n%(error)s') % {'text': text,
                     'pipeline': pipeline, 'error': str(e)}))
@@ -243,7 +242,7 @@ class JingleRTPContent(JingleContent):
                 # Add fallback source
                 self.src_bin = self.get_fallback_src()
                 self.pipeline.add(self.src_bin)
-                self.src_bin.link(sink_pad)
+                self.src_bin.get_static_pad('src').link(sink_pad)
                 self.stream_failed_once = True
             else:
                 reason = nbxmpp.Node('reason')
@@ -413,7 +412,7 @@ class JingleVideo(JingleRTPContent):
             framerate = ''
         try:
             w, h = app.config.get('video_size').split('x')
-        except:
+        except Exception:
             w = h = None
         if w and h:
             video_size = 'video/x-raw,width=%s,height=%s ! ' % (w, h)
@@ -471,7 +470,7 @@ class JingleVideo(JingleRTPContent):
 def get_content(desc):
     if desc['media'] == 'audio':
         return JingleAudio
-    elif desc['media'] == 'video':
+    if desc['media'] == 'video':
         return JingleVideo
 
 contents[nbxmpp.NS_JINGLE_RTP] = get_content
