@@ -1,28 +1,25 @@
-# -*- coding:utf-8 -*-
-## src/common/logger.py
-##
-## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2004-2005 Vincent Hanquez <tab AT snarc.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2007 Tomasz Melcer <liori AT exroot.org>
-##                    Julien Pivotto <roidelapluie AT gmail.com>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2004-2005 Vincent Hanquez <tab AT snarc.org>
+# Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2007 Tomasz Melcer <liori AT exroot.org>
+#                    Julien Pivotto <roidelapluie AT gmail.com>
+# Copyright (C) 2018 Philipp Hörist <philipp AT hoerist.com>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 """
 This module allows to access the on-disk database of logs
@@ -34,96 +31,232 @@ import time
 import datetime
 import calendar
 import json
+import logging
+import sqlite3 as sqlite
 from collections import namedtuple
 from gzip import GzipFile
 from io import BytesIO
 from gi.repository import GLib
-from enum import IntEnum, unique
 
 from gajim.common import exceptions
 from gajim.common import app
+from gajim.common import configpaths
+from gajim.common.i18n import _
+from gajim.common.const import (
+    JIDConstant, KindConstant, ShowConstant, TypeConstant,
+    SubscriptionConstant)
 
-import sqlite3 as sqlite
 
-LOG_DB_PATH = app.gajimpaths['LOG_DB']
-LOG_DB_FOLDER, LOG_DB_FILE = os.path.split(LOG_DB_PATH)
-CACHE_DB_PATH = app.gajimpaths['CACHE_DB']
+LOGS_SQL_STATEMENT = '''
+    CREATE TABLE jids(
+            jid_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
+            jid TEXT UNIQUE,
+            type INTEGER
+    );
+    CREATE TABLE unread_messages(
+            message_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
+            jid_id INTEGER,
+            shown BOOLEAN default 0
+    );
+    CREATE INDEX idx_unread_messages_jid_id ON unread_messages (jid_id);
+    CREATE TABLE logs(
+            log_line_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
+            account_id INTEGER,
+            jid_id INTEGER,
+            contact_name TEXT,
+            time INTEGER,
+            kind INTEGER,
+            show INTEGER,
+            message TEXT,
+            subject TEXT,
+            additional_data TEXT,
+            stanza_id TEXT,
+            encryption TEXT,
+            encryption_state TEXT,
+            marker INTEGER
+    );
+    CREATE TABLE last_archive_message(
+            jid_id INTEGER PRIMARY KEY UNIQUE,
+            last_mam_id TEXT,
+            oldest_mam_timestamp TEXT,
+            last_muc_timestamp TEXT,
+            sync_threshold INTEGER
+    );
+    CREATE INDEX idx_logs_jid_id_time ON logs (jid_id, time DESC);
+    CREATE INDEX idx_logs_stanza_id ON logs (stanza_id);
+    PRAGMA user_version=2;
+    '''
+
+CACHE_SQL_STATEMENT = '''
+    CREATE TABLE transports_cache (
+            transport TEXT UNIQUE,
+            type INTEGER
+    );
+    CREATE TABLE caps_cache (
+            hash_method TEXT,
+            hash TEXT,
+            data BLOB,
+            last_seen INTEGER);
+    CREATE TABLE rooms_last_message_time(
+            jid_id INTEGER PRIMARY KEY UNIQUE,
+            time INTEGER
+    );
+    CREATE TABLE roster_entry(
+            account_jid_id INTEGER,
+            jid_id INTEGER,
+            name TEXT,
+            subscription INTEGER,
+            ask BOOLEAN,
+            avatar_sha TEXT,
+            PRIMARY KEY (account_jid_id, jid_id)
+    );
+    CREATE TABLE roster_group(
+            account_jid_id INTEGER,
+            jid_id INTEGER,
+            group_name TEXT,
+            PRIMARY KEY (account_jid_id, jid_id, group_name)
+    );
+    PRAGMA user_version=1;
+    '''
 
-import logging
 log = logging.getLogger('gajim.c.logger')
 
-@unique
-class JIDConstant(IntEnum):
-    NORMAL_TYPE = 0
-    ROOM_TYPE = 1
-
-@unique
-class KindConstant(IntEnum):
-    STATUS = 0
-    GCSTATUS = 1
-    GC_MSG = 2
-    SINGLE_MSG_RECV = 3
-    CHAT_MSG_RECV = 4
-    SINGLE_MSG_SENT = 5
-    CHAT_MSG_SENT = 6
-    ERROR = 7
-
-    def __str__(self):
-        return str(self.value)
-
-@unique
-class ShowConstant(IntEnum):
-    ONLINE = 0
-    CHAT = 1
-    AWAY = 2
-    XA = 3
-    DND = 4
-    OFFLINE = 5
-
-@unique
-class TypeConstant(IntEnum):
-    AIM = 0
-    GG = 1
-    HTTP_WS = 2
-    ICQ = 3
-    MSN = 4
-    QQ = 5
-    SMS = 6
-    SMTP = 7
-    TLEN = 8
-    YAHOO = 9
-    NEWMAIL = 10
-    RSS = 11
-    WEATHER = 12
-    MRIM = 13
-    NO_TRANSPORT = 14
-
-@unique
-class SubscriptionConstant(IntEnum):
-    NONE = 0
-    TO = 1
-    FROM = 2
-    BOTH = 3
 
 class Logger:
     def __init__(self):
         self._jid_ids = {}
-        self.con = None
-        self.commit_timout_id = None
+        self._jid_ids_reversed = {}
+        self._con = None
+        self._commit_timout_id = None
+        self._log_db_path = configpaths.get('LOG_DB')
+        self._cache_db_path = configpaths.get('CACHE_DB')
 
-        if not os.path.exists(LOG_DB_PATH):
-            # this can happen only the first time (the time we create the db)
-            # db is not created here but in src/common/checks_paths.py
-            return
-        self.init_vars()
-        if not os.path.exists(CACHE_DB_PATH):
-            # this can happen cache database is not present when gajim is launched
-            # db will be created in src/common/checks_paths.py
-            return
-        self.attach_cache_database()
+        self._create_databases()
+        self._migrate_databases()
+        self._connect_databases()
+        self._get_jid_ids_from_db()
+
+    def _create_databases(self):
+        if os.path.isdir(self._log_db_path):
+            log.error(_('%s is a directory but should be a file'),
+                      self._log_db_path)
+            sys.exit()
+
+        if os.path.isdir(self._cache_db_path):
+            log.error(_('%s is a directory but should be a file'),
+                      self._cache_db_path)
+            sys.exit()
+
+        if not os.path.exists(self._log_db_path):
+            if os.path.exists(self._cache_db_path):
+                os.remove(self._cache_db_path)
+            self._create(LOGS_SQL_STATEMENT, self._log_db_path)
+
+        if not os.path.exists(self._cache_db_path):
+            self._create(CACHE_SQL_STATEMENT, self._cache_db_path)
 
     @staticmethod
-    def namedtuple_factory(cursor, row):
+    def _connect(*args, **kwargs):
+        con = sqlite.connect(*args, **kwargs)
+        con.execute("PRAGMA secure_delete=1")
+        return con
+
+    @classmethod
+    def _create(cls, statement, path):
+        log.info(_('Creating %s'), path)
+        con = cls._connect(path)
+        os.chmod(path, 0o600)
+
+        try:
+            con.executescript(statement)
+        except Exception:
+            log.exception('Error')
+            con.close()
+            os.remove(path)
+            sys.exit()
+
+        con.commit()
+        con.close()
+
+    @staticmethod
+    def _get_user_version(con: sqlite.Connection) -> int:
+        """ Return the value of PRAGMA user_version. """
+        return con.execute('PRAGMA user_version').fetchone()[0]
+
+    def _migrate_databases(self):
+        try:
+            con = self._connect(self._log_db_path)
+            self._migrate_logs(con)
+            con.close()
+
+            con = self._connect(self._cache_db_path)
+            self._migrate_cache(con)
+            con.close()
+        except Exception:
+            log.exception('Error')
+            sys.exit()
+
+    def _migrate_logs(self, con):
+        if self._get_user_version(con) == 0:
+            # All migrations from 0.16.9 until 1.0.0
+            statements = [
+                'ALTER TABLE logs ADD COLUMN "account_id" INTEGER',
+                'ALTER TABLE logs ADD COLUMN "stanza_id" TEXT',
+                'ALTER TABLE logs ADD COLUMN "encryption" TEXT',
+                'ALTER TABLE logs ADD COLUMN "encryption_state" TEXT',
+                'ALTER TABLE logs ADD COLUMN "marker" INTEGER',
+                'ALTER TABLE logs ADD COLUMN "additional_data" TEXT',
+                '''CREATE TABLE IF NOT EXISTS last_archive_message(
+                    jid_id INTEGER PRIMARY KEY UNIQUE,
+                    last_mam_id TEXT,
+                    oldest_mam_timestamp TEXT,
+                    last_muc_timestamp TEXT
+                    )''',
+
+                '''CREATE INDEX IF NOT EXISTS idx_logs_stanza_id
+                    ON logs(stanza_id)''',
+                'PRAGMA user_version=1'
+            ]
+
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 2:
+            statements = [
+                'ALTER TABLE last_archive_message ADD COLUMN "sync_threshold" INTEGER',
+                'PRAGMA user_version=2'
+            ]
+            self._execute_multiple(con, statements)
+
+    def _migrate_cache(self, con):
+        if self._get_user_version(con) == 0:
+            # All migrations from 0.16.9 until 1.0.0
+            statements = [
+                'ALTER TABLE roster_entry ADD COLUMN "avatar_sha" TEXT',
+                'PRAGMA user_version=1'
+                ]
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 2:
+            pass
+
+    @staticmethod
+    def _execute_multiple(con, statements):
+        """
+        Execute mutliple statements with the option to fail on duplicates
+        but still continue
+        """
+        for sql in statements:
+            try:
+                con.execute(sql)
+                con.commit()
+            except sqlite.OperationalError as error:
+                if str(error).startswith('duplicate column name:'):
+                    log.info(error)
+                else:
+                    log.exception('Error')
+                    sys.exit()
+
+    def namedtuple_factory(self, cursor, row):
         """
         Usage:
         con.row_factory = namedtuple_factory
@@ -134,60 +267,45 @@ class Logger:
         if 'additional_data' in fields:
             named_row = named_row._replace(
                 additional_data=json.loads(named_row.additional_data or '{}'))
+
+        # if an alias `account` for the field `account_id` is used for the
+        # query, the account_id is converted to the account jid
+        if 'account' in fields:
+            if named_row.account:
+                jid = self._jid_ids_reversed[named_row.account].jid
+                named_row = named_row._replace(account=jid)
         return named_row
 
     def dispatch(self, event, error):
         app.ged.raise_event(event, None, str(error))
 
-    def close_db(self):
-        if self.con:
-            self.con.close()
-        self.con = None
-        self.cur = None
+    def _connect_databases(self):
+        self._con = self._connect(
+            self._log_db_path, timeout=20.0, isolation_level='IMMEDIATE')
 
-    def open_db(self):
-        self.close_db()
-
-        # FIXME: sqlite3_open wants UTF8 strings. So a path with
-        # non-ascii chars doesn't work. See #2812 and
-        # http://lists.initd.org/pipermail/pysqlite/2005-August/000134.html
-        back = os.getcwd()
-        os.chdir(LOG_DB_FOLDER)
-
-        # if locked, wait up to 20 sec to unlock
-        # before raise (hopefully should be enough)
-
-        self.con = sqlite.connect(LOG_DB_FILE, timeout=20.0,
-                isolation_level='IMMEDIATE')
-        os.chdir(back)
-        self.con.row_factory = self.namedtuple_factory
+        self._con.row_factory = self.namedtuple_factory
 
         # DB functions
-        self.con.create_function("like", 1, self._like)
-        self.con.create_function("get_timeout", 0, self._get_timeout)
+        self._con.create_function("like", 1, self._like)
+        self._con.create_function("get_timeout", 0, self._get_timeout)
 
-        self.cur = self.con.cursor()
-        self.set_synchronous(False)
-
-    def attach_cache_database(self):
+        self._set_synchronous(False)
         try:
-            self.cur.execute("ATTACH DATABASE '%s' AS cache" % \
-                CACHE_DB_PATH.replace("'", "''"))
-        except sqlite.Error as e:
-            log.debug("Failed to attach cache database: %s" % str(e))
+            self._con.execute("ATTACH DATABASE '%s' AS cache" %
+                              self._cache_db_path.replace("'", "''"))
+        except Exception:
+            log.exception('Error')
+            self._con.close()
+            sys.exit()
 
-    def set_synchronous(self, sync):
+    def _set_synchronous(self, sync):
         try:
             if sync:
-                self.cur.execute("PRAGMA synchronous = NORMAL")
+                self._con.execute("PRAGMA synchronous = NORMAL")
             else:
-                self.cur.execute("PRAGMA synchronous = OFF")
-        except sqlite.Error as e:
-            log.debug("Failed to set_synchronous(%s): %s" % (sync, str(e)))
-
-    def init_vars(self):
-        self.open_db()
-        self.get_jid_ids_from_db()
+                self._con.execute("PRAGMA synchronous = OFF")
+        except sqlite.Error:
+            log.exception('Error')
 
     @staticmethod
     def _get_timeout():
@@ -207,32 +325,33 @@ class Logger:
 
     def commit(self):
         try:
-            self.con.commit()
+            self._con.commit()
         except sqlite.OperationalError as e:
             print(str(e), file=sys.stderr)
-        self.commit_timout_id = None
+        self._commit_timout_id = None
         return False
 
     def _timeout_commit(self):
-        if self.commit_timout_id:
+        if self._commit_timout_id:
             return
-        self.commit_timout_id = GLib.timeout_add(500, self.commit)
+        self._commit_timout_id = GLib.timeout_add(500, self.commit)
 
     def simple_commit(self, sql_to_commit):
         """
         Helper to commit
         """
-        self.cur.execute(sql_to_commit)
+        self._con.execute(sql_to_commit)
         self._timeout_commit()
 
-    def get_jid_ids_from_db(self):
+    def _get_jid_ids_from_db(self):
         """
         Load all jid/jid_id tuples into a dict for faster access
         """
-        rows = self.con.execute(
+        rows = self._con.execute(
             'SELECT jid_id, jid, type FROM jids').fetchall()
         for row in rows:
             self._jid_ids[row.jid] = row
+            self._jid_ids_reversed[row.jid_id] = row
 
     def get_jids_in_db(self):
         return self._jid_ids.keys()
@@ -249,22 +368,17 @@ class Logger:
         if jid.find('/') > -1:
             possible_room_jid = jid.split('/', 1)[0]
             return self.jid_is_room_jid(possible_room_jid)
-        else:
-            # it's not a full jid, so it's not a pm one
-            return False
+        # it's not a full jid, so it's not a pm one
+        return False
 
     def jid_is_room_jid(self, jid):
         """
         Return True if it's a room jid, False if it's not, None if we don't know
         """
-        self.cur.execute('SELECT type FROM jids WHERE jid=?', (jid,))
-        row = self.cur.fetchone()
-        if row is None:
-            return None
-        else:
-            if row.type == JIDConstant.ROOM_TYPE:
-                return True
-            return False
+        jid_ = self._jid_ids.get(jid)
+        if jid_ is None:
+            return
+        return jid_.type == JIDConstant.ROOM_TYPE
 
     @staticmethod
     def _get_family_jids(account, jid):
@@ -310,7 +424,7 @@ class Logger:
             return result.jid_id
 
         sql = 'SELECT jid_id, jid, type FROM jids WHERE jid = ?'
-        row = self.con.execute(sql, [jid]).fetchone()
+        row = self._con.execute(sql, [jid]).fetchone()
         if row is not None:
             self._jid_ids[jid] = row
             return row.jid_id
@@ -320,7 +434,7 @@ class Logger:
                 'Unable to insert new JID because type is missing')
 
         sql = 'INSERT INTO jids (jid, type) VALUES (?, ?)'
-        lastrowid = self.con.execute(sql, (jid, type_)).lastrowid
+        lastrowid = self._con.execute(sql, (jid, type_)).lastrowid
         Row = namedtuple('Row', 'jid_id jid type')
         self._jid_ids[jid] = Row(lastrowid, jid, type_)
         self._timeout_commit()
@@ -356,21 +470,21 @@ class Logger:
 
         if show == 'online':
             return ShowConstant.ONLINE
-        elif show == 'chat':
+        if show == 'chat':
             return ShowConstant.CHAT
-        elif show == 'away':
+        if show == 'away':
             return ShowConstant.AWAY
-        elif show == 'xa':
+        if show == 'xa':
             return ShowConstant.XA
-        elif show == 'dnd':
+        if show == 'dnd':
             return ShowConstant.DND
-        elif show == 'offline':
+        if show == 'offline':
             return ShowConstant.OFFLINE
-        elif show is None:
+        if show is None:
             return ShowConstant.ONLINE
-        else: # invisible in GC when someone goes invisible
-            # it's a RFC violation .... but we should not crash
-            return None
+        # invisible in GC when someone goes invisible
+        # it's a RFC violation .... but we should not crash
+        return None
 
     def convert_human_transport_type_to_db_api_values(self, type_):
         """
@@ -394,8 +508,6 @@ class Logger:
             return TypeConstant.SMTP
         if type_ in ('tlen', 'x-tlen'):
             return TypeConstant.TLEN
-        if type_ == 'yahoo':
-            return TypeConstant.YAHOO
         if type_ == 'newmail':
             return TypeConstant.NEWMAIL
         if type_ == 'rss':
@@ -430,8 +542,6 @@ class Logger:
             return 'smtp'
         if type_id == TypeConstant.TLEN:
             return 'tlen'
-        if type_id == TypeConstant.YAHOO:
-            return 'yahoo'
         if type_id == TypeConstant.NEWMAIL:
             return 'newmail'
         if type_id == TypeConstant.RSS:
@@ -443,7 +553,7 @@ class Logger:
         if type_id == TypeConstant.NO_TRANSPORT:
             return 'jabber'
 
-    def convert_human_subscription_values_to_db_api_values(self, sub):
+    def convert_xmpp_sub(self, sub):
         """
         Convert from string style to constant ints for db
         """
@@ -456,7 +566,7 @@ class Logger:
         if sub == 'both':
             return SubscriptionConstant.BOTH
 
-    def convert_db_api_values_to_human_subscription_values(self, sub):
+    def convert_db_sub(self, sub):
         """
         Convert from constant ints for db to string style
         """
@@ -473,9 +583,10 @@ class Logger:
         """
         Add unread message with id: message_id
         """
-        sql = 'INSERT INTO unread_messages VALUES (%d, %d, 0)' % (message_id,
-                jid_id)
-        self.simple_commit(sql)
+        sql = '''INSERT INTO unread_messages (message_id, jid_id, shown)
+                 VALUES (?, ?, 0)'''
+        self._con.execute(sql, (message_id, jid_id))
+        self._timeout_commit()
 
     def set_read_messages(self, message_ids):
         """
@@ -506,9 +617,8 @@ class Logger:
         """
         all_messages = []
         try:
-            self.cur.execute(
-                    'SELECT message_id, shown from unread_messages')
-            unread_results = self.cur.fetchall()
+            unread_results = self._con.execute(
+                'SELECT message_id, shown from unread_messages').fetchall()
         except Exception:
             unread_results = []
         for message in unread_results:
@@ -517,20 +627,19 @@ class Logger:
             # here we get infos for that message, and related jid from jids table
             # do NOT change order of SELECTed things, unless you change function(s)
             # that called this function
-            self.cur.execute('''
+            result = self._con.execute('''
                     SELECT logs.log_line_id, logs.message, logs.time, logs.subject,
                     jids.jid, logs.additional_data
                     FROM logs, jids
                     WHERE logs.log_line_id = %d AND logs.jid_id = jids.jid_id
                     ''' % msg_log_id
-                    )
-            results = self.cur.fetchone()
-            if len(results) == 0:
+                    ).fetchone()
+            if result is None:
                 # Log line is no more in logs table. remove it from unread_messages
                 self.set_read_messages([msg_log_id])
                 continue
 
-            all_messages.append((results, shown))
+            all_messages.append((result, shown))
         return all_messages
 
     def get_last_conversation_lines(self, account, jid, pending):
@@ -574,11 +683,11 @@ class Logger:
                        kinds=', '.join(kinds))
 
         try:
-            messages = self.con.execute(
+            messages = self._con.execute(
                 sql, tuple(jids) + (restore, pending)).fetchall()
         except sqlite.DatabaseError:
             self.dispatch('DB_ERROR',
-                          exceptions.DatabaseMalformed(LOG_DB_PATH))
+                          exceptions.DatabaseMalformed(self._log_db_path))
             return []
 
         messages.reverse()
@@ -621,7 +730,7 @@ class Logger:
             ORDER BY time, log_line_id
             '''.format(jids=', '.join('?' * len(jids)))
 
-        return self.con.execute(sql, tuple(jids) +
+        return self._con.execute(sql, tuple(jids) +
                                       (date.timestamp(),
                                       (date + delta).timestamp())).fetchall()
 
@@ -664,7 +773,7 @@ class Logger:
         '''.format(jids=', '.join('?' * len(jids)),
                    date_search=between if date else '')
 
-        return self.con.execute(sql, tuple(jids) + (query,)).fetchall()
+        return self._con.execute(sql, tuple(jids) + (query,)).fetchall()
 
     def get_days_with_logs(self, account, jid, year, month):
         """
@@ -702,7 +811,7 @@ class Logger:
             """.format(jids=', '.join('?' * len(jids)),
                        kinds=', '.join(kinds))
 
-        return self.con.execute(sql, tuple(jids) +
+        return self._con.execute(sql, tuple(jids) +
                                       (date.timestamp(),
                                       (date + delta).timestamp())).fetchall()
 
@@ -730,7 +839,7 @@ class Logger:
 
         # fetchone() returns always at least one Row with all
         # attributes set to None because of the MAX() function
-        return self.con.execute(sql, tuple(jids)).fetchone().time
+        return self._con.execute(sql, tuple(jids)).fetchone().time
 
     def get_first_date_that_has_logs(self, account, jid):
         """
@@ -756,7 +865,7 @@ class Logger:
 
         # fetchone() returns always at least one Row with all
         # attributes set to None because of the MIN() function
-        return self.con.execute(sql, tuple(jids)).fetchone().time
+        return self._con.execute(sql, tuple(jids)).fetchone().time
 
     def get_date_has_logs(self, account, jid, date):
         """
@@ -774,21 +883,20 @@ class Logger:
         """
         jids = self._get_family_jids(account, jid)
 
-        kinds = map(str, [KindConstant.STATUS,
-                          KindConstant.GCSTATUS])
-
         delta = datetime.timedelta(
             hours=23, minutes=59, seconds=59, microseconds=999999)
 
+        start = date.timestamp()
+        end = (date + delta).timestamp()
+
         sql = '''
             SELECT time
             FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
             AND time BETWEEN ? AND ?
             '''.format(jids=', '.join('?' * len(jids)))
 
-        return self.con.execute(sql, tuple(jids) +
-                                      (date.timestamp(),
-                                      (date + delta).timestamp())).fetchone()
+        return self._con.execute(
+            sql, tuple(jids) + (start, end)).fetchone()
 
     def get_room_last_message_time(self, account, jid):
         """
@@ -805,7 +913,7 @@ class Logger:
             NATURAL JOIN jids WHERE jid = ?
             '''
 
-        row = self.con.execute(sql, (jid,)).fetchone()
+        row = self._con.execute(sql, (jid,)).fetchone()
         if not row:
             return self.get_last_date_that_has_logs(account, jid)
         return row.time
@@ -826,7 +934,7 @@ class Logger:
                  (SELECT time FROM rooms_last_message_time
                   WHERE jid_id = :jid_id AND time >= :time), :time))'''
 
-        self.con.execute(sql, {"jid_id": jid_id, "time": timestamp})
+        self._con.execute(sql, {"jid_id": jid_id, "time": timestamp})
         self._timeout_commit()
 
     def save_transport_type(self, jid, type_):
@@ -837,26 +945,24 @@ class Logger:
         if not type_id:
             # unknown type
             return
-        self.cur.execute(
-                'SELECT type from transports_cache WHERE transport = "%s"' % jid)
-        results = self.cur.fetchone()
-        if results:
-            if results.type == type_id:
+        result = self._con.execute(
+            'SELECT type from transports_cache WHERE transport = "%s"' % jid).fetchone()
+        if result:
+            if result.type == type_id:
                 return
             sql = 'UPDATE transports_cache SET type = %d WHERE transport = "%s"' %\
                     (type_id, jid)
             self.simple_commit(sql)
             return
-        sql = 'INSERT INTO transports_cache VALUES ("%s", %d)' % (jid, type_id)
-        self.simple_commit(sql)
+        sql = 'INSERT INTO transports_cache (transport, type) VALUES (?, ?)'
+        self._con.execute(sql, (jid, type_id))
+        self._timeout_commit()
 
     def get_transports_type(self):
         """
         Return all the type of the transports in DB
         """
-        self.cur.execute(
-                'SELECT * from transports_cache')
-        results = self.cur.fetchall()
+        results = self._con.execute('SELECT * from transports_cache').fetchall()
         if not results:
             return {}
         answer = {}
@@ -884,7 +990,7 @@ class Logger:
         # the data field contains binary object (gzipped data), this is a hack
         # to get that data without trying to convert it to unicode
         try:
-            self.cur.execute('SELECT hash_method, hash, data FROM caps_cache;')
+            rows = self._con.execute('SELECT hash_method, hash, data FROM caps_cache;')
         except sqlite.OperationalError:
             # might happen when there's no caps_cache table yet
             # -- there's no data to read anyway then
@@ -892,7 +998,7 @@ class Logger:
 
         # list of corrupted entries that will be removed
         to_be_removed = []
-        for row in self.cur:
+        for row in rows:
             # for each row: unpack the data field
             # (format: (category, type, name, category, type, name, ...
             #   ..., 'FEAT', feature1, feature2, ...).join(' '))
@@ -912,9 +1018,9 @@ class Logger:
                 lang = data[i + 2]
                 name = data[i + 3]
                 identities.append({'category': category, 'type': type_,
-                        'xml:lang': lang, 'name': name})
+                                   'xml:lang': lang, 'name': name})
                 i += 4
-            i+=1
+            i += 1
             while i < len(data):
                 features.append(data[i])
                 i += 1
@@ -943,7 +1049,7 @@ class Logger:
         gzip.write(data.encode('utf-8'))
         gzip.close()
         data = string.getvalue()
-        self.cur.execute('''
+        self._con.execute('''
                 INSERT INTO caps_cache ( hash_method, hash, data, last_seen )
                 VALUES (?, ?, ?, ?);
                 ''', (hash_method, hash_, memoryview(data), int(time.time())))
@@ -1006,16 +1112,16 @@ class Logger:
             jid_id = self.get_jid_id(jid)
         except exceptions.PysqliteOperationalError as e:
             raise exceptions.PysqliteOperationalError(str(e))
-        self.cur.execute(
+        self._con.execute(
                 'DELETE FROM roster_group WHERE account_jid_id=? AND jid_id=?',
                 (account_jid_id, jid_id))
-        self.cur.execute(
+        self._con.execute(
                 'DELETE FROM roster_entry WHERE account_jid_id=? AND jid_id=?',
                 (account_jid_id, jid_id))
         self._timeout_commit()
 
     def add_or_update_contact(self, account_jid, jid, name, sub, ask, groups,
-    commit=True):
+                              commit=True):
         """
         Add or update a contact from account_jid roster
         """
@@ -1026,29 +1132,30 @@ class Logger:
         try:
             account_jid_id = self.get_jid_id(account_jid)
             jid_id = self.get_jid_id(jid, type_=JIDConstant.NORMAL_TYPE)
-        except exceptions.PysqliteOperationalError as e:
-            raise exceptions.PysqliteOperationalError(str(e))
+        except exceptions.PysqliteOperationalError as error:
+            raise exceptions.PysqliteOperationalError(str(error))
 
         # Update groups information
         # First we delete all previous groups information
-        self.cur.execute(
-                'DELETE FROM roster_group WHERE account_jid_id=? AND jid_id=?',
-                (account_jid_id, jid_id))
+        sql = 'DELETE FROM roster_group WHERE account_jid_id=? AND jid_id=?'
+        self._con.execute(sql, (account_jid_id, jid_id))
         # Then we add all new groups information
+        sql = '''INSERT INTO roster_group (account_jid_id, jid_id, group_name)
+                 VALUES (?, ?, ?)'''
         for group in groups:
-            self.cur.execute('INSERT INTO roster_group VALUES(?, ?, ?)',
-                    (account_jid_id, jid_id, group))
+            self._con.execute(sql, (account_jid_id, jid_id, group))
 
         if name is None:
             name = ''
 
-        self.cur.execute('''
-            REPLACE INTO roster_entry
-            (account_jid_id, jid_id, name, subscription, ask)
-            VALUES(?, ?, ?, ?, ?)''', (
-                account_jid_id, jid_id, name,
-                self.convert_human_subscription_values_to_db_api_values(sub),
-                bool(ask)))
+        sql = '''REPLACE INTO roster_entry
+                 (account_jid_id, jid_id, name, subscription, ask)
+                 VALUES(?, ?, ?, ?, ?)'''
+        self._con.execute(sql, (account_jid_id,
+                                jid_id,
+                                name,
+                                self.convert_xmpp_sub(sub),
+                                bool(ask)))
         if commit:
             self._timeout_commit()
 
@@ -1060,11 +1167,11 @@ class Logger:
         account_jid_id = self.get_jid_id(account_jid, type_=JIDConstant.NORMAL_TYPE)
 
         # First we fill data with roster_entry informations
-        self.cur.execute('''
+        rows = self._con.execute('''
                 SELECT j.jid, re.jid_id, re.name, re.subscription, re.ask, re.avatar_sha
                 FROM roster_entry re, jids j
                 WHERE re.account_jid_id=? AND j.jid_id=re.jid_id''', (account_jid_id,))
-        for row in self.cur:
+        for row in rows:
             #jid, jid_id, name, subscription, ask
             jid = row.jid
             name = row.name
@@ -1074,9 +1181,7 @@ class Logger:
                 data[jid]['name'] = name
             else:
                 data[jid]['name'] = None
-            data[jid]['subscription'] = \
-                    self.convert_db_api_values_to_human_subscription_values(
-                    row.subscription)
+            data[jid]['subscription'] = self.convert_db_sub(row.subscription)
             data[jid]['groups'] = []
             data[jid]['resources'] = {}
             if row.ask:
@@ -1087,11 +1192,11 @@ class Logger:
 
         # Then we add group for roster entries
         for jid in data:
-            self.cur.execute('''
+            rows = self._con.execute('''
                     SELECT group_name FROM roster_group
                     WHERE account_jid_id=? AND jid_id=?''',
                     (account_jid_id, data[jid]['id']))
-            for row in self.cur:
+            for row in rows:
                 group_name = row.group_name
                 data[jid]['groups'].append(group_name)
             del data[jid]['id']
@@ -1116,7 +1221,7 @@ class Logger:
             DELETE FROM roster_group WHERE account_jid_id = {jid_id};
             '''.format(jid_id=jid_id)
 
-        self.con.executescript(sql)
+        self._con.executescript(sql)
         self._timeout_commit()
 
     def search_for_duplicate(self, account, jid, timestamp, msg):
@@ -1133,8 +1238,8 @@ class Logger:
         """
 
         # Add 10 seconds around the timestamp
-        start_time = timestamp - 10
-        end_time = timestamp + 10
+        start_time = timestamp - 30
+        end_time = timestamp + 30
 
         account_id = self.get_account_id(account)
         log.debug('start: %s, end: %s, jid: %s, message: %s',
@@ -1146,7 +1251,7 @@ class Logger:
             AND time BETWEEN ? AND ?
             '''
 
-        result = self.con.execute(
+        result = self._con.execute(
             sql, (jid, msg, account_id, start_time, end_time)).fetchone()
 
         if result is not None:
@@ -1181,7 +1286,11 @@ class Logger:
         if not ids:
             return False
 
-        archive_id = self.get_jid_id(archive_jid)
+        type_ = JIDConstant.NORMAL_TYPE
+        if groupchat:
+            type_ = JIDConstant.ROOM_TYPE
+
+        archive_id = self.get_jid_id(archive_jid, type_=type_)
         account_id = self.get_account_id(account)
 
         if groupchat:
@@ -1193,14 +1302,14 @@ class Logger:
                 WHERE stanza_id IN ({values})
                 AND jid_id = ? AND account_id = ? LIMIT 1
                 '''.format(values=', '.join('?' * len(ids)))
-            result = self.con.execute(
+            result = self._con.execute(
                 sql, tuple(ids) + (archive_id, account_id)).fetchone()
         else:
             sql = '''
                 SELECT stanza_id FROM logs
                 WHERE stanza_id IN ({values}) AND account_id = ? AND kind != ? LIMIT 1
                 '''.format(values=', '.join('?' * len(ids)))
-            result = self.con.execute(
+            result = self._con.execute(
                 sql, tuple(ids) + (account_id, KindConstant.GC_MSG)).fetchone()
 
         if result is not None:
@@ -1254,7 +1363,7 @@ class Logger:
               '''.format(columns=', '.join(kwargs.keys()),
                          values=', '.join('?' * len(kwargs)))
 
-        lastrowid = self.con.execute(
+        lastrowid = self._con.execute(
             sql, (account_id, jid_id, time_, kind) + tuple(kwargs.values())).lastrowid
 
         log.info('Insert into DB: jid: %s, time: %s, kind: %s, stanza_id: %s',
@@ -1263,7 +1372,7 @@ class Logger:
         if unread and kind == KindConstant.CHAT_MSG_RECV:
             sql = '''INSERT INTO unread_messages (message_id, jid_id)
                      VALUES (?, (SELECT jid_id FROM jids WHERE jid = ?))'''
-            self.con.execute(sql, (lastrowid, jid))
+            self._con.execute(sql, (lastrowid, jid))
 
         self._timeout_commit()
 
@@ -1288,23 +1397,23 @@ class Logger:
             UPDATE roster_entry SET avatar_sha = ?
             WHERE account_jid_id = ? AND jid_id = ?
             '''
-        self.con.execute(sql, (sha, account_jid_id, jid_id))
+        self._con.execute(sql, (sha, account_jid_id, jid_id))
         self._timeout_commit()
 
-    def get_archive_timestamp(self, jid, type_=None):
+    def get_archive_infos(self, jid):
         """
-        Get the last archive id/timestamp for a jid
+        Get the archive infos
 
         :param jid:     The jid that belongs to the avatar
 
         """
-        jid_id = self.get_jid_id(jid, type_=type_)
+        jid_id = self.get_jid_id(jid, type_=JIDConstant.ROOM_TYPE)
         sql = '''SELECT * FROM last_archive_message WHERE jid_id = ?'''
-        return self.con.execute(sql, (jid_id,)).fetchone()
+        return self._con.execute(sql, (jid_id,)).fetchone()
 
-    def set_archive_timestamp(self, jid, **kwargs):
+    def set_archive_infos(self, jid, **kwargs):
         """
-        Set the last archive id/timestamp
+        Set archive infos
 
         :param jid:                     The jid that belongs to the avatar
 
@@ -1316,19 +1425,28 @@ class Logger:
         :param last_muc_timestamp:      The timestamp of the last message we
                                         received in a MUC
 
+        :param sync_threshold:          The max days that we request from a
+                                        MUC archive
+
         """
         jid_id = self.get_jid_id(jid)
-        exists = self.get_archive_timestamp(jid)
+        exists = self.get_archive_infos(jid)
         if not exists:
-            sql = '''INSERT INTO last_archive_message VALUES (?, ?, ?, ?)'''
-            self.con.execute(sql, (jid_id,
-                                   kwargs.get('last_mam_id', None),
-                                   kwargs.get('oldest_mam_timestamp', None),
-                                   kwargs.get('last_muc_timestamp', None)))
+            sql = '''INSERT INTO last_archive_message
+                     (jid_id, last_mam_id, oldest_mam_timestamp,
+                      last_muc_timestamp, sync_threshold)
+                      VALUES (?, ?, ?, ?, ?)'''
+            self._con.execute(sql, (
+                jid_id,
+                kwargs.get('last_mam_id', None),
+                kwargs.get('oldest_mam_timestamp', None),
+                kwargs.get('last_muc_timestamp', None),
+                kwargs.get('sync_threshold', None)
+            ))
         else:
             args = ' = ?, '.join(kwargs.keys()) + ' = ?'
             sql = '''UPDATE last_archive_message SET {}
                      WHERE jid_id = ?'''.format(args)
-            self.con.execute(sql, tuple(kwargs.values()) + (jid_id,))
-        log.info('Save archive timestamps: %s', kwargs)
+            self._con.execute(sql, tuple(kwargs.values()) + (jid_id,))
+        log.info('Save archive infos: %s', kwargs)
         self._timeout_commit()
