@@ -1,33 +1,33 @@
-##      common/zeroconf/client_zeroconf.py
-##
-## Copyright (C) 2006 Stefan Bethge <stefan@lanpartei.de>
-##                              2006 Dimitur Kirov <dkirov@gmail.com>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
-##
-from common import gajim
+# Copyright (C) 2006 Stefan Bethge <stefan@lanpartei.de>
+#               2006 Dimitur Kirov <dkirov@gmail.com>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+
+from gajim.common import app
 import nbxmpp
 from nbxmpp.idlequeue import IdleObject
 from nbxmpp import dispatcher_nb, simplexml
 from nbxmpp.plugin import PlugIn
-from nbxmpp.simplexml import ustr
+from nbxmpp.plugin import *
 from nbxmpp.transports_nb import DATA_RECEIVED, DATA_SENT, DATA_ERROR
-from common.zeroconf import zeroconf
+from gajim.common.zeroconf import zeroconf
 
-from nbxmpp.protocol import Node, NS_STREAMS
+from nbxmpp.protocol import *
 import socket
+import platform
+import ssl
 import errno
 import sys
 import os
@@ -37,7 +37,7 @@ from random import Random
 import logging
 log = logging.getLogger('gajim.c.z.client_zeroconf')
 
-from common.zeroconf import roster_zeroconf
+from gajim.common.zeroconf import roster_zeroconf
 
 MAX_BUFF_LEN = 65536
 TYPE_SERVER, TYPE_CLIENT = range(2)
@@ -73,8 +73,7 @@ class ZeroconfListener(IdleObject):
         self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
         self._serv.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         if os.name == 'nt':
-            ver = os.sys.getwindowsversion()
-            if (ver[3], ver[0]) == (2, 6): # Win Vista +
+            if int(platform.win32_ver()[0]) >= 6:  # Win Vista +
                 # 47 is socket.IPPROTO_IPV6
                 # 27 is socket.IPV6_V6ONLY under windows, but not defined ...
                 self._serv.setsockopt(41, 27, 0)
@@ -87,7 +86,7 @@ class ZeroconfListener(IdleObject):
         self._serv.listen(socket.SOMAXCONN)
         self._serv.setblocking(False)
         self.fd = self._serv.fileno()
-        gajim.idlequeue.plug_idle(self, False, True)
+        app.idlequeue.plug_idle(self, False, True)
         self.started = True
 
     def pollend(self):
@@ -107,7 +106,7 @@ class ZeroconfListener(IdleObject):
         for jid in self.conn_holder.getRoster().keys():
             entry = self.conn_holder.getRoster().getItem(jid)
             for address in entry['addresses']:
-                if (address['address'] == ipaddr):
+                if address['address'] == ipaddr:
                     from_jid = jid
                     break
         P2PClient(sock[0], [{'host': ipaddr, 'address': ipaddr, 'port': sock[1][1]}], self.conn_holder, [], from_jid)
@@ -116,9 +115,9 @@ class ZeroconfListener(IdleObject):
         """
         Free all resources, we are not listening anymore
         """
-        log.info('Disconnecting ZeroconfListener: %s' % message)
-        gajim.idlequeue.remove_timeout(self.fd)
-        gajim.idlequeue.unplug_idle(self.fd)
+        log.info('Disconnecting ZeroconfListener: %s', message)
+        app.idlequeue.remove_timeout(self.fd)
+        app.idlequeue.unplug_idle(self.fd)
         self.fd = -1
         self.started = False
         try:
@@ -182,7 +181,7 @@ class P2PClient(IdleObject):
                 id_ = stanza.getID()
                 if not id_:
                     id_ = self.Dispatcher.getAnID()
-                if self.conn_holder.ids_of_awaiting_messages.has_key(self.fd):
+                if self.fd in self.conn_holder.ids_of_awaiting_messages:
                     self.conn_holder.ids_of_awaiting_messages[self.fd].append((
                         id_, thread_id))
                 else:
@@ -204,7 +203,7 @@ class P2PClient(IdleObject):
             id_ = stanza.getID()
             if not id_:
                 id_ = self.Dispatcher.getAnID()
-            if self.conn_holder.ids_of_awaiting_messages.has_key(self.fd):
+            if self.fd in self.conn_holder.ids_of_awaiting_messages:
                 self.conn_holder.ids_of_awaiting_messages[self.fd].append((id_,
                     thread_id))
             else:
@@ -214,7 +213,7 @@ class P2PClient(IdleObject):
         return True
 
     def on_message_sent(self, connection_id):
-        id_, thread_id = \
+        id_, _thread_id = \
             self.conn_holder.ids_of_awaiting_messages[connection_id].pop(0)
         if self.on_ok:
             self.on_ok(id_)
@@ -254,18 +253,18 @@ class P2PClient(IdleObject):
 
     def _check_stream_start(self, ns, tag, attrs):
         if ns != NS_STREAMS or tag != 'stream':
-            log.error('Incorrect stream start: (%s,%s).Terminating!' % (tag,
-                ns), 'error')
+            log.error('Incorrect stream start: (%s,%s).Terminating!',
+                      tag, ns)
             self.Connection.disconnect()
             if self.on_not_ok:
                 self.on_not_ok('Connection to host could not be established: '
                     'Incorrect answer from server.')
             return
         if self.sock_type == TYPE_SERVER:
-            if attrs.has_key('from'):
+            if 'from' in attrs:
                 self.to = attrs['from']
             self.send_stream_header()
-            if attrs.has_key('version') and attrs['version'] == '1.0':
+            if 'version' in attrs and attrs['version'] == '1.0':
                 # other part supports stream features
                 features = Node('stream:features')
                 self.Dispatcher.send(features)
@@ -279,12 +278,12 @@ class P2PClient(IdleObject):
 
     def on_disconnect(self):
         if self.conn_holder:
-            if self.conn_holder.ids_of_awaiting_messages.has_key(self.fd):
+            if self.fd in self.conn_holder.ids_of_awaiting_messages:
                 del self.conn_holder.ids_of_awaiting_messages[self.fd]
             self.conn_holder.remove_connection(self.sock_hash)
-        if self.__dict__.has_key('Dispatcher'):
+        if 'Dispatcher' in self.__dict__:
             self.Dispatcher.PlugOut()
-        if self.__dict__.has_key('P2PConnection'):
+        if 'P2PConnection' in self.__dict__:
             self.P2PConnection.PlugOut()
         self.Connection = None
         self._caller = None
@@ -303,7 +302,7 @@ class P2PClient(IdleObject):
                 self.Dispatcher.Stream._document_attrs is None:
             return
         self.onreceive(None)
-        if self.Dispatcher.Stream._document_attrs.has_key('version') and \
+        if 'version' in self.Dispatcher.Stream._document_attrs and \
         self.Dispatcher.Stream._document_attrs['version'] == '1.0':
                 #~ self.onreceive(self._on_receive_stream_features)
                 #XXX continue with TLS
@@ -317,7 +316,7 @@ class P2PClient(IdleObject):
     def _register_handlers(self):
         self._caller.peerhost = self.Connection._sock.getsockname()
         self.RegisterHandler('message', lambda conn,
-            data:self._caller._messageCB(self.Server, conn, data))
+            data: self._caller._messageCB(self.Server, conn, data))
         self.RegisterHandler('iq', self._caller._siSetCB, 'set', nbxmpp.NS_SI)
         self.RegisterHandler('iq', self._caller._siErrorCB, 'error',
             nbxmpp.NS_SI)
@@ -329,8 +328,6 @@ class P2PClient(IdleObject):
             nbxmpp.NS_BYTESTREAM)
         self.RegisterHandler('iq', self._caller._bytestreamErrorCB, 'error',
             nbxmpp.NS_BYTESTREAM)
-        self.RegisterHandler('iq', self._caller._DiscoverItemsGetCB, 'get',
-            nbxmpp.NS_DISCO_ITEMS)
         self.RegisterHandler('iq', self._caller._JingleCB, 'result')
         self.RegisterHandler('iq', self._caller._JingleCB, 'error')
         self.RegisterHandler('iq', self._caller._JingleCB, 'set',
@@ -374,22 +371,24 @@ class P2PConnection(IdleObject, PlugIn):
         try:
             self.ais = socket.getaddrinfo(address['host'], address['port'], socket.AF_UNSPEC,
                     socket.SOCK_STREAM)
-        except socket.gaierror, e:
-            log.info('Lookup failure for %s: %s[%s]', host, e[1],
+        except socket.gaierror as e:
+            log.info('Lookup failure for %s: %s[%s]', self.host, e[1],
                 repr(e[0]), exc_info=True)
-            if len(self.addresses_) > 0: return self.get_next_addrinfo()
+            if self.addresses_:
+                return self.get_next_addrinfo()
         else:
             self.connect_to_next_ip()
 
     def connect_to_next_ip(self):
-        if len(self.ais) == 0:
+        if not self.ais:
             log.error('Connection failure to %s', str(self.host), exc_info=True)
-            if len(self.addresses_) > 0: return self.get_next_addrinfo()
+            if self.addresses_:
+                return self.get_next_addrinfo()
             self.disconnect()
             return
         ai = self.ais.pop(0)
-        log.info('Trying to connect to %s through %s:%s', str(self.host),
-            ai[4][0], ai[4][1], exc_info=True)
+        log.info('Trying to connect to %s through %s:%s',
+                 str(self.host), ai[4][0], ai[4][1], exc_info=True)
         try:
             self._sock = socket.socket(*ai[:3])
             self._sock.setblocking(False)
@@ -400,14 +399,14 @@ class P2PConnection(IdleObject, PlugIn):
                 self.connect_to_next_ip()
                 return
         self.fd = self._sock.fileno()
-        gajim.idlequeue.plug_idle(self, True, False)
+        app.idlequeue.plug_idle(self, True, False)
         self.set_timeout(CONNECT_TIMEOUT_SECONDS)
         self.do_connect()
 
     def set_timeout(self, timeout):
-        gajim.idlequeue.remove_timeout(self.fd)
+        app.idlequeue.remove_timeout(self.fd)
         if self.state >= 0:
-            gajim.idlequeue.set_read_timeout(self.fd, timeout)
+            app.idlequeue.set_read_timeout(self.fd, timeout)
 
     def plugin(self, owner):
         self.onreceive(owner._on_receive_document_attrs)
@@ -438,18 +437,11 @@ class P2PConnection(IdleObject, PlugIn):
         """
         Append stanza to the queue of messages to be send if now is False, else
         send it instantly
-
-        If supplied data is unicode string, encode it to UTF-8.
         """
         if self.state <= 0:
             return
 
-        r = packet
-
-        if isinstance(r, unicode):
-            r = r.encode('utf-8')
-        elif not isinstance(r, str):
-            r = ustr(r).encode('utf-8')
+        r = str(packet).encode('utf-8')
 
         if now:
             self.sendqueue.insert(0, (r, is_message))
@@ -460,8 +452,8 @@ class P2PConnection(IdleObject, PlugIn):
 
     def read_timeout(self):
         ids = self.client.conn_holder.ids_of_awaiting_messages
-        if self.fd in ids and len(ids[self.fd]) > 0:
-            for (id_, thread_id) in ids[self.fd]:
+        if self.fd in ids and ids[self.fd]:
+            for (_id, thread_id) in ids[self.fd]:
                 if hasattr(self._owner, 'Dispatcher'):
                     self._owner.Dispatcher.Event('', DATA_ERROR, (
                         self.client.to, thread_id))
@@ -475,21 +467,24 @@ class P2PConnection(IdleObject, PlugIn):
         try:
             self._sock.connect(self._server[:2])
             self._sock.setblocking(False)
-        except Exception, ee:
-            (errnum, errstr) = ee
+        except Exception as ee:
+            errnum = ee.errno
+            errstr = ee.strerror
         errors = (errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK)
         if 'WSAEINVAL' in errno.__dict__:
             errors += (errno.WSAEINVAL,)
         if errnum in errors:
             return
+
         # win32 needs this
-        elif errnum not in (0, 10056, errno.EISCONN) or self.state != 0:
+        if errnum not in (0, 10056, errno.EISCONN) or self.state != 0:
             log.error('Could not connect to %s: %s [%s]', str(self.host),
                 errnum, errstr)
             self.connect_to_next_ip()
             return
-        else: # socket is already connected
-            self._sock.setblocking(False)
+
+        # socket is already connected
+        self._sock.setblocking(False)
         self.state = 1 # connected
         # we are connected
         self.on_connect(self)
@@ -498,7 +493,7 @@ class P2PConnection(IdleObject, PlugIn):
         if self.state == 0:
             self.do_connect()
             return
-        gajim.idlequeue.remove_timeout(self.fd)
+        app.idlequeue.remove_timeout(self.fd)
         self._do_send()
 
     def pollend(self):
@@ -519,18 +514,17 @@ class P2PConnection(IdleObject, PlugIn):
         try:
             # get as many bites, as possible, but not more than RECV_BUFSIZE
             received = self._sock.recv(MAX_BUFF_LEN)
-        except Exception, e:
-            if len(e.args) > 0 and isinstance(e.args[0], int):
-                errnum = e[0]
+        except Exception as e:
+            errnum = e.errno
             # "received" will be empty anyhow
-        if errnum == socket.SSL_ERROR_WANT_READ:
+        if errnum == ssl.SSL_ERROR_WANT_READ:
             pass
         elif errnum in [errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN]:
             self.pollend()
             # don't proccess result, cas it will raise error
             return
-        elif not received :
-            if errnum != socket.SSL_ERROR_EOF:
+        elif not received:
+            if errnum != ssl.SSL_ERROR_EOF:
                 # 8 EOF occurred in violation of protocol
                 self.pollend()
             if self.state >= 0:
@@ -539,6 +533,9 @@ class P2PConnection(IdleObject, PlugIn):
 
         if self.state < 0:
             return
+
+        received = received.decode('utf-8')
+
         if self.on_receive:
             if self._owner.sock_type == TYPE_CLIENT:
                 self.set_timeout(ACTIVITY_TIMEOUT_SECONDS)
@@ -549,7 +546,7 @@ class P2PConnection(IdleObject, PlugIn):
             self.on_receive(received)
         else:
             # This should never happed, so we need the debug
-            log.error('Unhandled data received: %s' % received)
+            log.error('Unhandled data received: %s', received)
             self.disconnect()
         return True
 
@@ -557,8 +554,8 @@ class P2PConnection(IdleObject, PlugIn):
         """
         Close the socket
         """
-        gajim.idlequeue.remove_timeout(self.fd)
-        gajim.idlequeue.unplug_idle(self.fd)
+        app.idlequeue.remove_timeout(self.fd)
+        app.idlequeue.unplug_idle(self.fd)
         try:
             self._sock.shutdown(socket.SHUT_RDWR)
             self._sock.close()
@@ -582,7 +579,7 @@ class P2PConnection(IdleObject, PlugIn):
                 self.sendbuff = self.sendbuff[send_count:]
                 if not self.sendbuff and not self.sendqueue:
                     if self.state < 0:
-                        gajim.idlequeue.unplug_idle(self.fd)
+                        app.idlequeue.unplug_idle(self.fd)
                         self._on_send()
                         self.disconnect()
                         return
@@ -590,8 +587,8 @@ class P2PConnection(IdleObject, PlugIn):
                     self._plug_idle()
                 self._on_send()
 
-        except socket.error, e:
-            if e[0] == socket.SSL_ERROR_WANT_WRITE:
+        except socket.error as e:
+            if e.errno == ssl.SSL_ERROR_WANT_WRITE:
                 return True
             if self.state < 0:
                 self.disconnect()
@@ -604,19 +601,17 @@ class P2PConnection(IdleObject, PlugIn):
 
     def _plug_idle(self):
         readable = self.state != 0
-        if self.sendqueue or self.sendbuff:
-            writable = True
-        else:
-            writable = False
+        writable = self.sendqueue or self.sendbuff
         if self.writable != writable or self.readable != readable:
-            gajim.idlequeue.plug_idle(self, writable, readable)
+            app.idlequeue.plug_idle(self, writable, readable)
 
 
     def _on_send(self):
         if self.sent_data and self.sent_data.strip():
-            log.debug('sent: %s' % self.sent_data)
+            log.debug('sent: %s', self.sent_data)
             if hasattr(self._owner, 'Dispatcher'):
-                self._owner.Dispatcher.Event('', DATA_SENT, self.sent_data)
+                self._owner.Dispatcher.Event(
+                    '', DATA_SENT, self.sent_data.decode('utf-8'))
         self.sent_data = None
         if self.buff_is_message:
             self._owner.on_message_sent(self.fd)
@@ -710,7 +705,7 @@ class ClientZeroconf:
             self.roster = None
         self.disconnecting = True
         for i in reversed(self.disconnect_handlers):
-            log.debug('Calling disconnect handler %s' % i)
+            log.debug('Calling disconnect handler %s', i)
             i()
         self.disconnecting = False
 
@@ -718,11 +713,11 @@ class ClientZeroconf:
         self.disconnect()
 
     def kill_all_connections(self):
-        for connection in self.connections.values():
+        for connection in list(self.connections.values()):
             connection.force_disconnect()
 
     def add_connection(self, connection, ip, port, recipient):
-        sock_hash=connection.sock_hash
+        sock_hash = connection.sock_hash
         if sock_hash not in self.connections:
             self.connections[sock_hash] = connection
         self.ip_to_hash[ip] = sock_hash
@@ -741,7 +736,7 @@ class ClientZeroconf:
             if self.ip_to_hash[i] == sock_hash:
                 del self.ip_to_hash[i]
                 break
-        if self.hash_to_port.has_key(sock_hash):
+        if sock_hash in self.hash_to_port:
             del self.hash_to_port[sock_hash]
 
     def start_listener(self, port):
@@ -760,9 +755,12 @@ class ClientZeroconf:
 
     def send(self, stanza, is_message=False, now=False, on_ok=None,
     on_not_ok=None):
+        to = stanza.getTo()
+        if to is None:
+            # Can’t send undirected stanza over Zeroconf.
+            return -1
+        to = to.getStripped()
         stanza.setFrom(self.roster.zeroconf.name)
-        to = unicode(stanza.getTo())
-        to = gajim.get_jid_without_resource(to)
 
         try:
             item = self.roster[to]
@@ -806,7 +804,7 @@ class ClientZeroconf:
         """
         Generate a random id
         """
-        return ''.join(Random().sample(string.letters + string.digits, 6))
+        return ''.join(Random().sample(string.ascii_letters + string.digits, 6))
 
     def RegisterDisconnectHandler(self, handler):
         """
@@ -833,8 +831,8 @@ class ClientZeroconf:
         def on_ok(_waitid):
 #            if timeout:
 #                self._owner.set_timeout(timeout)
-            to = unicode(stanza.getTo())
-            to = gajim.get_jid_without_resource(to)
+            to = stanza.getTo()
+            to = app.get_jid_without_resource(to)
 
             try:
                 item = self.roster[to]
