@@ -1,68 +1,65 @@
-# -*- coding:utf-8 -*-
-## src/conversation_textview.py
-##
-## Copyright (C) 2005 Norman Rasmussen <norman AT rasmussen.co.za>
-## Copyright (C) 2005-2006 Alex Mauer <hawke AT hawkesnest.net>
-##                         Travis Shirk <travis AT pobox.com>
-## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2005-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
-##                    Julien Pivotto <roidelapluie AT gmail.com>
-##                    Stephan Erb <steve-e AT h3c.de>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2005 Norman Rasmussen <norman AT rasmussen.co.za>
+# Copyright (C) 2005-2006 Alex Mauer <hawke AT hawkesnest.net>
+#                         Travis Shirk <travis AT pobox.com>
+# Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2005-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
+#                    Julien Pivotto <roidelapluie AT gmail.com>
+#                    Stephan Erb <steve-e AT h3c.de>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from threading import Timer # for smooth scrolling
+import time
+import os
+import queue
+import urllib
+import logging
+from calendar import timegm
 
 from gi.repository import Gtk
-from gi.repository import Gdk
 from gi.repository import Pango
 from gi.repository import GObject
 from gi.repository import GLib
-import time
-import os
-from gajim import dialogs
-import queue
-import urllib
 
-from gajim import gtkgui_helpers
 from gajim.common import app
 from gajim.common import helpers
 from gajim.common import i18n
-from calendar import timegm
+from gajim.common.i18n import _
 from gajim.common.fuzzyclock import FuzzyClock
-from gajim import emoticons
+from gajim.common.const import StyleAttr
 
 from gajim.htmltextview import HtmlTextView
 
+from gajim.gtk import util
+from gajim.gtk.util import load_icon
+from gajim.gtk.util import get_cursor
+from gajim.gtk.emoji_data import emoji_pixbufs
+from gajim.gtk.emoji_data import is_emoji
+from gajim.gtk.emoji_data import get_emoji_pixbuf
+
 NOT_SHOWN = 0
 ALREADY_RECEIVED = 1
 SHOWN = 2
 
-import logging
 log = logging.getLogger('gajim.conversation_textview')
 
 def is_selection_modified(mark):
     name = mark.get_name()
-    if name and name in ('selection_bound', 'insert'):
-        return True
-    else:
-        return False
+    return name in ('selection_bound', 'insert')
 
 def has_focus(widget):
     return widget.get_state_flags() & Gtk.StateFlags.FOCUSED == \
@@ -81,14 +78,12 @@ class TextViewImage(Gtk.Image):
 
     def _get_selected(self):
         parent = self.get_parent()
-        if not parent or not self.anchor: return False
+        if not parent or not self.anchor:
+            return False
         buffer_ = parent.get_buffer()
         position = buffer_.get_iter_at_child_anchor(self.anchor)
         bounds = buffer_.get_selection_bounds()
-        if bounds and position.in_range(*bounds):
-            return True
-        else:
-            return False
+        return bounds and position.in_range(*bounds)
 
     def get_state(self):
         parent = self.get_parent()
@@ -97,10 +92,8 @@ class TextViewImage(Gtk.Image):
         if self._selected:
             if has_focus(parent):
                 return Gtk.StateType.SELECTED
-            else:
-                return Gtk.StateType.ACTIVE
-        else:
-            return Gtk.StateType.NORMAL
+            return Gtk.StateType.ACTIVE
+        return Gtk.StateType.NORMAL
 
     def _update_selected(self):
         selected = self._get_selected()
@@ -159,23 +152,13 @@ class ConversationTextview(GObject.GObject):
     Class for the conversation textview (where user reads already said messages)
     for chat/groupchat windows
     """
-    __gsignals__ = dict(
-            quote = (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
-                    None, # return value
-                    (str, ) # arguments
-            )
-    )
+    __gsignals__ = dict(quote=(
+        GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
+        None, # return value
+        (str, ) # arguments
+        ))
 
-    MESSAGE_CORRECTED_PIXBUF = gtkgui_helpers.get_icon_pixmap(
-        'document-edit-symbolic')
-    MESSAGE_ENCRYPTED_PIXBUF = gtkgui_helpers.get_icon_pixmap(
-        'channel-secure-croped-symbolic')
-
-    # smooth scroll constants
-    MAX_SCROLL_TIME = 0.4 # seconds
-    SCROLL_DELAY = 33 # milliseconds
-
-    def __init__(self, account, used_in_history_window = False):
+    def __init__(self, account, used_in_history_window=False):
         """
         If used_in_history_window is True, then we do not show Clear menuitem in
         context menu
@@ -188,10 +171,7 @@ class ConversationTextview(GObject.GObject):
         self.fc = FuzzyClock()
 
         # no need to inherit TextView, use it as atrribute is safer
-        self.tv = HtmlTextView()
-        # we have to override HtmlTextView Event handlers
-        # because we dont inherit
-        self.tv.hyperlink_handler = self.hyperlink_handler
+        self.tv = HtmlTextView(account)
         self.tv.connect_tooltip(self.query_tooltip)
 
         # set properties
@@ -202,8 +182,8 @@ class ConversationTextview(GObject.GObject):
         self.tv.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
         self.tv.set_left_margin(2)
         self.tv.set_right_margin(2)
+        self._buffer = self.tv.get_buffer()
         self.handlers = {}
-        self.images = []
         self.image_cache = {}
         self.xep0184_marks = {}
         # self.last_sent_message_id = msg_stanza_id
@@ -221,44 +201,50 @@ class ConversationTextview(GObject.GObject):
         self.account = account
         self.cursor_changed = False
         self.last_time_printout = 0
+        self.encryption_enabled = False
 
         style = self.tv.get_style_context()
-        style.add_class('font_custom')
+        style.add_class('gajim-conversation-font')
         buffer_ = self.tv.get_buffer()
         end_iter = buffer_.get_end_iter()
         buffer_.create_mark('end', end_iter, False)
 
         self.tagIn = buffer_.create_tag('incoming')
-        color = app.config.get('inmsgcolor')
-        font = Pango.FontDescription(app.config.get('inmsgfont'))
+        color = app.css_config.get_value(
+            '.gajim-incoming-nickname', StyleAttr.COLOR)
         self.tagIn.set_property('foreground', color)
-        self.tagIn.set_property('font-desc', font)
+        desc = app.css_config.get_font('.gajim-incoming-nickname')
+        self.tagIn.set_property('font-desc', desc)
 
         self.tagOut = buffer_.create_tag('outgoing')
-        color = app.config.get('outmsgcolor')
-        font = Pango.FontDescription(app.config.get('outmsgfont'))
+        color = app.css_config.get_value(
+            '.gajim-outgoing-nickname', StyleAttr.COLOR)
         self.tagOut.set_property('foreground', color)
-        self.tagOut.set_property('font-desc', font)
+        desc = app.css_config.get_font('.gajim-outgoing-nickname')
+        self.tagOut.set_property('font-desc', desc)
 
         self.tagStatus = buffer_.create_tag('status')
-        color = app.config.get('statusmsgcolor')
-        font = Pango.FontDescription(app.config.get('satusmsgfont'))
+        color = app.css_config.get_value(
+            '.gajim-status-message', StyleAttr.COLOR)
         self.tagStatus.set_property('foreground', color)
-        self.tagStatus.set_property('font-desc', font)
+        desc = app.css_config.get_font('.gajim-status-message')
+        self.tagStatus.set_property('font-desc', desc)
 
         self.tagInText = buffer_.create_tag('incomingtxt')
-        color = app.config.get('inmsgtxtcolor')
-        font = Pango.FontDescription(app.config.get('inmsgtxtfont'))
+        color = app.css_config.get_value(
+            '.gajim-incoming-message-text', StyleAttr.COLOR)
         if color:
             self.tagInText.set_property('foreground', color)
-        self.tagInText.set_property('font-desc', font)
+        desc = app.css_config.get_font('.gajim-incoming-message-text')
+        self.tagInText.set_property('font-desc', desc)
 
         self.tagOutText = buffer_.create_tag('outgoingtxt')
-        color = app.config.get('outmsgtxtcolor')
+        color = app.css_config.get_value(
+            '.gajim-outgoing-message-text', StyleAttr.COLOR)
         if color:
-            font = Pango.FontDescription(app.config.get('outmsgtxtfont'))
-        self.tagOutText.set_property('foreground', color)
-        self.tagOutText.set_property('font-desc', font)
+            self.tagOutText.set_property('foreground', color)
+        desc = app.css_config.get_font('.gajim-outgoing-message-text')
+        self.tagOutText.set_property('font-desc', desc)
 
         colors = app.config.get('gc_nicknames_colors')
         colors = colors.split(':')
@@ -268,10 +254,14 @@ class ConversationTextview(GObject.GObject):
             tag.set_property('foreground', color)
 
         self.tagMarked = buffer_.create_tag('marked')
-        color = app.config.get('markedmsgcolor')
+        color = app.css_config.get_value(
+            '.gajim-highlight-message', StyleAttr.COLOR)
         self.tagMarked.set_property('foreground', color)
         self.tagMarked.set_property('weight', Pango.Weight.BOLD)
 
+        textview_icon = buffer_.create_tag('textview-icon')
+        textview_icon.set_property('rise', Pango.units_from_double(-4.45))
+
         tag = buffer_.create_tag('time_sometimes')
         tag.set_property('foreground', 'darkgrey')
         #Pango.SCALE_SMALL
@@ -283,7 +273,7 @@ class ConversationTextview(GObject.GObject):
         tag.set_property('scale', 0.8333333333333)
 
         tag = buffer_.create_tag('restored_message')
-        color = app.config.get('restored_messages_color')
+        color = app.css_config.get_value('.gajim-restored-message', StyleAttr.COLOR)
         tag.set_property('foreground', color)
 
         self.tv.create_tags()
@@ -297,7 +287,7 @@ class ConversationTextview(GObject.GObject):
         tag = buffer_.create_tag('underline')
         tag.set_property('underline', Pango.Underline.SINGLE)
 
-        buffer_.create_tag('focus-out-line', justification = Gtk.Justification.CENTER)
+        buffer_.create_tag('focus-out-line', justification=Gtk.Justification.CENTER)
         self.displaymarking_tags = {}
 
         tag = buffer_.create_tag('xep0184-received')
@@ -336,11 +326,11 @@ class ConversationTextview(GObject.GObject):
                     if len(text) > 50:
                         text = text[:47] + '…'
                     tooltip.set_text(text)
-                    window.set_cursor(gtkgui_helpers.get_cursor('HAND2'))
+                    window.set_cursor(get_cursor('HAND2'))
                     self.cursor_changed = True
                     return True
             if tag_name in ('url', 'mail', 'xmpp', 'sth_at_sth'):
-                window.set_cursor(gtkgui_helpers.get_cursor('HAND2'))
+                window.set_cursor(get_cursor('HAND2'))
                 self.cursor_changed = True
                 return False
             try:
@@ -350,25 +340,25 @@ class ConversationTextview(GObject.GObject):
             except KeyError:
                 pass
         if self.cursor_changed:
-            window.set_cursor(gtkgui_helpers.get_cursor('XTERM'))
+            window.set_cursor(get_cursor('XTERM'))
             self.cursor_changed = False
         return False
 
     def del_handlers(self):
-        for i in self.handlers.keys():
+        for i in self.handlers:
             if self.handlers[i].handler_is_connected(i):
                 self.handlers[i].disconnect(i)
         del self.handlers
         self.tv.destroy()
 
     def update_tags(self):
-        self.tagIn.set_property('foreground', app.config.get('inmsgcolor'))
-        self.tagOut.set_property('foreground', app.config.get('outmsgcolor'))
+        self.tagIn.set_property('foreground', app.css_config.get_value('.gajim-incoming-nickname', StyleAttr.COLOR))
+        self.tagOut.set_property('foreground', app.css_config.get_value('.gajim-outgoing-nickname', StyleAttr.COLOR))
         self.tagStatus.set_property('foreground',
-            app.config.get('statusmsgcolor'))
+            app.css_config.get_value('.gajim-status-message', StyleAttr.COLOR))
         self.tagMarked.set_property('foreground',
-            app.config.get('markedmsgcolor'))
-        color = app.config.get('urlmsgcolor')
+            app.css_config.get_value('.gajim-highlight-message', StyleAttr.COLOR))
+        color = app.css_config.get_value('.gajim-url', StyleAttr.COLOR)
         self.tv.tagURL.set_property('foreground', color)
         self.tv.tagMail.set_property('foreground', color)
         self.tv.tagXMPP.set_property('foreground', color)
@@ -376,7 +366,7 @@ class ConversationTextview(GObject.GObject):
 
     def scroll_to_end(self, force=False):
         if self.autoscroll or force:
-            gtkgui_helpers.scroll_to_end(self.tv.get_parent())
+            util.scroll_to_end(self.tv.get_parent())
 
     def correct_message(self, correct_id, kind, name):
         allowed = True
@@ -397,7 +387,7 @@ class ConversationTextview(GObject.GObject):
             allowed = False
 
         if not allowed:
-            log.debug('Message correctiong not allowed')
+            log.debug('Message correction not allowed')
             return None
 
         end_mark, index = self.get_end_mark(correct_id, start_mark)
@@ -504,7 +494,7 @@ class ConversationTextview(GObject.GObject):
             buffer_.end_user_action()
             self.scroll_to_end()
 
-    def clear(self, tv = None):
+    def clear(self, tv=None):
         """
         Clear text in the textview
         """
@@ -568,7 +558,7 @@ class ConversationTextview(GObject.GObject):
                         % phrase_for_url
             else:
                 link = 'http://%s.wikipedia.org/wiki/Special:Search?search=%s'\
-                        % (app.LANG, phrase_for_url)
+                        % (i18n.LANG, phrase_for_url)
             item = Gtk.MenuItem.new_with_mnemonic(_('Read _Wikipedia Article'))
             id_ = item.connect('activate', self.visit_url_from_menuitem, link)
             self.handlers[id_] = item
@@ -584,7 +574,7 @@ class ConversationTextview(GObject.GObject):
                             % phrase_for_url
                 else:
                     link = 'http://%s.wiktionary.org/wiki/Special:Search?search=%s'\
-                            % (app.LANG, phrase_for_url)
+                            % (i18n.LANG, phrase_for_url)
                 id_ = item.connect('activate', self.visit_url_from_menuitem, link)
                 self.handlers[id_] = item
             else:
@@ -609,7 +599,7 @@ class ConversationTextview(GObject.GObject):
                 item.set_property('sensitive', False)
             else:
                 item = Gtk.MenuItem.new_with_mnemonic(_('Web _Search for it'))
-                link =  search_link % phrase_for_url
+                link = search_link % phrase_for_url
                 id_ = item.connect('activate', self.visit_url_from_menuitem, link)
                 self.handlers[id_] = item
             submenu.append(item)
@@ -625,9 +615,9 @@ class ConversationTextview(GObject.GObject):
         self.emit('quote', self.selected_phrase)
 
     def on_textview_button_press_event(self, widget, event):
-        # If we clicked on a taged text do NOT open the standard popup menu
+        # If we clicked on a tagged text do NOT open the standard popup menu
         # if normal text check if we have sth selected
-        self.selected_phrase = '' # do not move belove event button check!
+        self.selected_phrase = '' # do not move below event button check!
 
         if event.button != 3: # if not right click
             return False
@@ -664,146 +654,6 @@ class ConversationTextview(GObject.GObject):
                 finish_sel.forward_word_end()
             self.selected_phrase = buffer_.get_text(start_sel, finish_sel, True)
 
-    def on_open_link_activate(self, widget, kind, text):
-        helpers.launch_browser_mailer(kind, text)
-
-    def on_copy_link_activate(self, widget, text):
-        clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-        clip.set_text(text, -1)
-
-    def on_start_chat_activate(self, widget, jid):
-        app.interface.new_chat_from_jid(self.account, jid)
-
-    def on_join_group_chat_menuitem_activate(self, widget, room_jid):
-        # Remove ?join
-        room_jid = room_jid.split('?')[0]
-        app.interface.join_gc_minimal(self.account, room_jid)
-
-    def on_add_to_roster_activate(self, widget, jid):
-        dialogs.AddNewContactWindow(self.account, jid)
-
-    def make_link_menu(self, event, kind, text):
-        xml = gtkgui_helpers.get_gtk_builder('chat_context_menu.ui')
-        menu = xml.get_object('chat_context_menu')
-        childs = menu.get_children()
-        if kind == 'url':
-            id_ = childs[0].connect('activate', self.on_copy_link_activate, text)
-            self.handlers[id_] = childs[0]
-            id_ = childs[1].connect('activate', self.on_open_link_activate, kind,
-                    text)
-            self.handlers[id_] = childs[1]
-            childs[2].hide() # copy mail address
-            childs[3].hide() # open mail composer
-            childs[4].hide() # jid section separator
-            childs[5].hide() # start chat
-            childs[6].hide() # join group chat
-            childs[7].hide() # add to roster
-        else: # It's a mail or a JID
-            # load muc icon
-            join_group_chat_menuitem = xml.get_object('join_group_chat_menuitem')
-
-            text = text.lower()
-            if text.startswith('xmpp:'):
-                text = text[5:]
-            id_ = childs[2].connect('activate', self.on_copy_link_activate, text)
-            self.handlers[id_] = childs[2]
-            id_ = childs[3].connect('activate', self.on_open_link_activate, kind,
-                    text)
-            self.handlers[id_] = childs[3]
-            id_ = childs[5].connect('activate', self.on_start_chat_activate, text)
-            self.handlers[id_] = childs[5]
-            id_ = childs[6].connect('activate',
-                    self.on_join_group_chat_menuitem_activate, text)
-            self.handlers[id_] = childs[6]
-
-            if self.account and app.connections[self.account].\
-            roster_supported:
-                id_ = childs[7].connect('activate',
-                    self.on_add_to_roster_activate, text)
-                self.handlers[id_] = childs[7]
-                childs[7].show() # show add to roster menuitem
-            else:
-                childs[7].hide() # hide add to roster menuitem
-
-            if kind == 'xmpp':
-                id_ = childs[0].connect('activate', self.on_copy_link_activate,
-                    'xmpp:' + text)
-                self.handlers[id_] = childs[0]
-                childs[2].hide() # copy mail address
-                childs[3].hide() # open mail composer
-                childs[4].hide() # jid section separator
-            elif kind == 'mail':
-                childs[4].hide() # jid section separator
-                childs[5].hide() # start chat
-                childs[6].hide() # join group chat
-                childs[7].hide() # add to roster
-
-            if kind != 'xmpp':
-                childs[0].hide() # copy link location
-            childs[1].hide() # open link in browser
-
-        menu.attach_to_widget(self.tv, None)
-        menu.popup(None, None, None, None, event.button.button, event.time)
-
-    def hyperlink_handler(self, texttag, widget, event, iter_, kind):
-        if event.type == Gdk.EventType.BUTTON_PRESS:
-            begin_iter = iter_.copy()
-            # we get the begining of the tag
-            while not begin_iter.begins_tag(texttag):
-                begin_iter.backward_char()
-            end_iter = iter_.copy()
-            # we get the end of the tag
-            while not end_iter.ends_tag(texttag):
-                end_iter.forward_char()
-
-            # Detect XHTML-IM link
-            word = getattr(texttag, 'href', None)
-            if word:
-                if word.startswith('xmpp'):
-                    kind = 'xmpp'
-                elif word.startswith('mailto:'):
-                    kind = 'mail'
-                elif app.interface.sth_at_sth_dot_sth_re.match(word):
-                    # it's a JID or mail
-                    kind = 'sth_at_sth'
-            else:
-                word = self.tv.get_buffer().get_text(begin_iter, end_iter, True)
-            if event.button.button == 3: # right click
-                self.make_link_menu(event, kind, word)
-                return True
-            else:
-                self.plugin_modified = False
-                app.plugin_manager.extension_point(
-                    'hyperlink_handler', word, kind, self,
-                    self.tv.get_toplevel())
-                if self.plugin_modified:
-                    return
-
-                # we launch the correct application
-                if kind == 'xmpp':
-                    word = word[5:]
-                    if '?' in word:
-                        (jid, action) = word.split('?')
-                        if action == 'join':
-                            app.interface.join_gc_minimal(None, jid)
-                        else:
-                            self.on_start_chat_activate(None, jid)
-                    else:
-                        self.on_start_chat_activate(None, word)
-                # handle geo:-URIs
-                elif word[:4] == 'geo:':
-                    location = word[4:]
-                    lat, _, lon = location.partition(',')
-                    if lon == '':
-                        return
-                    uri = 'https://www.openstreetmap.org/?' \
-                          'mlat=%(lat)s&mlon=%(lon)s&zoom=16' % \
-                          {'lat': lat, 'lon': lon}
-                    helpers.launch_browser_mailer(kind, uri)
-                # other URIs
-                else:
-                    helpers.launch_browser_mailer(kind, word)
-
     def detect_and_print_special_text(self, otext, other_tags, graphics=True,
     iter_=None, additional_data=None):
         """
@@ -925,16 +775,32 @@ class ConversationTextview(GObject.GObject):
         else:
             end_iter = buffer_.get_end_iter()
 
-        pixbuf = emoticons.get_pixbuf(special_text)
-        if app.config.get('emoticons_theme') and pixbuf and graphics:
+        theme = app.config.get('emoticons_theme')
+        show_emojis = theme and theme != 'font'
+        if show_emojis and graphics and is_emoji(special_text):
             # it's an emoticon
-            anchor = buffer_.create_child_anchor(end_iter)
-            img = TextViewImage(anchor,
-                GLib.markup_escape_text(special_text))
-            img.set_from_pixbuf(pixbuf)
-            img.show()
-            self.images.append(img)
-            self.tv.add_child_at_anchor(img, anchor)
+            if emoji_pixbufs.complete:
+                # only search for the pixbuf if we are sure
+                # that loading is completed
+                pixbuf = get_emoji_pixbuf(special_text)
+                if pixbuf is None:
+                    buffer_.insert(end_iter, special_text)
+                else:
+                    pixbuf = pixbuf.copy()
+                    anchor = buffer_.create_child_anchor(end_iter)
+                    anchor.plaintext = special_text
+                    img = Gtk.Image.new_from_pixbuf(pixbuf)
+                    img.show()
+                    self.tv.add_child_at_anchor(img, anchor)
+            else:
+                # Set marks and save them so we can replace the emojis
+                # once the loading is complete
+                start_mark = buffer_.create_mark(None, end_iter, True)
+                buffer_.insert(end_iter, special_text)
+                end_mark = buffer_.create_mark(None, end_iter, True)
+                emoji_pixbufs.append_marks(
+                    self.tv, start_mark, end_mark, special_text)
+
         elif special_text.startswith('www.') or \
             special_text.startswith('ftp.') or \
             text_is_valid_uri and not is_xhtml_link:
@@ -1142,16 +1008,17 @@ class ConversationTextview(GObject.GObject):
         if kind == 'status':
             direction_mark = i18n.direction_mark
 
+        # print the encryption icon
+        if kind in ('incoming', 'outgoing'):
+            self.print_encryption_status(iter_, additional_data)
+
         # print the time stamp
         self.print_time(text, kind, tim, simple, direction_mark,
             other_tags_for_time, iter_)
 
-        if encrypted:
-            buffer_.insert_pixbuf(iter_, self.MESSAGE_ENCRYPTED_PIXBUF)
-
         # If there's a displaymarking, print it here.
         if displaymarking:
-            self.print_displaymarking(displaymarking, iter_=iter_)
+            self.print_displaymarking(displaymarking, iter_)
 
         # kind = info, we print things as if it was a status: same color, ...
         if kind in ('error', 'info'):
@@ -1181,7 +1048,7 @@ class ConversationTextview(GObject.GObject):
                 text_tags.append('outgoingtxt')
 
         self.print_subject(subject, iter_=iter_)
-        
+
         iter_ = self.print_real_text(text, text_tags, name, xhtml, graphics=graphics,
             mark=insert_mark, additional_data=additional_data)
 
@@ -1189,8 +1056,9 @@ class ConversationTextview(GObject.GObject):
             # Show Correction Icon
             buffer_.create_tag(tag_name=msg_stanza_id)
             buffer_.insert(iter_, ' ')
+            icon = load_icon('document-edit-symbolic', self.tv, pixbuf=True)
             buffer_.insert_pixbuf(
-                iter_, ConversationTextview.MESSAGE_CORRECTED_PIXBUF)
+                iter_, icon)
             tag_start_iter = iter_.copy()
             tag_start_iter.backward_chars(2)
             buffer_.apply_tag_by_name(msg_stanza_id, tag_start_iter, iter_)
@@ -1205,7 +1073,7 @@ class ConversationTextview(GObject.GObject):
         new_mark = buffer_.create_mark(
             str(self.line), temp_iter, left_gravity=False)
 
-        if not index:
+        if index is None:
             # New Message
             self.message_list.append((tim, new_mark, msg_stanza_id))
         elif corrected:
@@ -1265,15 +1133,61 @@ class ConversationTextview(GObject.GObject):
     def detect_other_text_tag(self, text, kind):
         if kind == 'status':
             return kind
-        elif text.startswith('/me ') or text.startswith('/me\n'):
+        if text.startswith('/me ') or text.startswith('/me\n'):
             return kind
 
+    def print_encryption_status(self, iter_, additional_data):
+        details = self._get_encryption_details(additional_data)
+        if details is None:
+            # Message was not encrypted
+            if not self.encryption_enabled:
+                return
+            icon = 'channel-insecure-symbolic'
+            color = 'unencrypted-color'
+            tooltip = _('Not encrypted')
+        else:
+            icon = 'channel-secure-symbolic'
+            color = 'encrypted-color'
+            name, fingerprint = details
+            if fingerprint is None:
+                tooltip = name
+            else:
+                tooltip = '%s %s' % (name, fingerprint)
+
+        temp_mark = self._buffer.create_mark(None, iter_, True)
+        self._buffer.insert(iter_, ' ')
+        anchor = self._buffer.create_child_anchor(iter_)
+        anchor.plaintext = ''
+        self._buffer.insert(iter_, ' ')
+
+        # Apply mark to vertically center the icon
+        start = self._buffer.get_iter_at_mark(temp_mark)
+        self._buffer.apply_tag_by_name('textview-icon', start, iter_)
+
+        image = Gtk.Image.new_from_icon_name(icon, Gtk.IconSize.MENU)
+        image.show()
+        image.set_tooltip_text(tooltip)
+        image.get_style_context().add_class(color)
+        self.tv.add_child_at_anchor(image, anchor)
+
+    @staticmethod
+    def _get_encryption_details(additional_data):
+        encrypted = additional_data.get('encrypted')
+        if encrypted is None:
+            return
+
+        name = encrypted.get('name')
+        if name is None:
+            return
+        fingerprint = encrypted.get('fingerprint')
+        return name, fingerprint
+
     def print_time(self, text, kind, tim, simple, direction_mark, other_tags_for_time, iter_):
         local_tim = time.localtime(tim)
         buffer_ = self.tv.get_buffer()
         current_print_time = app.config.get('print_time')
 
-        if current_print_time == 'always' and kind != 'info' and not simple:
+        if current_print_time == 'always' and not simple:
             timestamp_str = self.get_time_to_show(local_tim, direction_mark)
             timestamp = time.strftime(timestamp_str, local_tim)
             timestamp = direction_mark + timestamp + direction_mark
@@ -1282,7 +1196,7 @@ class ConversationTextview(GObject.GObject):
                     *other_tags_for_time)
             else:
                 buffer_.insert(iter_, timestamp)
-        elif current_print_time == 'sometimes' and kind != 'info' and not simple:
+        elif current_print_time == 'sometimes' and not simple:
             every_foo_seconds = 60 * app.config.get(
                 'print_ichat_every_foo_minutes')
             seconds_passed = tim - self.last_time_printout
@@ -1296,21 +1210,16 @@ class ConversationTextview(GObject.GObject):
                 buffer_.insert_with_tags_by_name(iter_, tim_format + '\n',
                     'time_sometimes')
 
-    def print_displaymarking(self, displaymarking, iter_=None):
+    def print_displaymarking(self, displaymarking, iter_):
         bgcolor = displaymarking.getAttr('bgcolor') or '#FFF'
         fgcolor = displaymarking.getAttr('fgcolor') or '#000'
         text = displaymarking.getData()
         if text:
             buffer_ = self.tv.get_buffer()
-            if iter_:
-                end_iter = iter_
-            else:
-                end_iter = buffer_.get_end_iter()
             tag = self.displaymarking_tags.setdefault(bgcolor + '/' + fgcolor,
                 buffer_.create_tag(None, background=bgcolor, foreground=fgcolor))
-            buffer_.insert_with_tags(end_iter, '[' + text + ']', tag)
-            end_iter = buffer_.get_end_iter()
-            buffer_.insert_with_tags(end_iter, ' ')
+            buffer_.insert_with_tags(iter_, '[' + text + ']', tag)
+            buffer_.insert_with_tags(iter_, ' ')
 
     def print_name(self, name, kind, other_tags_for_name, direction_mark='',
     iter_=None):
@@ -1362,10 +1271,10 @@ class ConversationTextview(GObject.GObject):
                 if name and (text.startswith('/me ') or text.startswith('/me\n')):
                     xhtml = xhtml.replace('/me', '<i>* %s</i>' % (name,), 1)
                 self.tv.display_html(xhtml, self.tv, self, iter_=iter_)
-                return
-            except Exception as e:
-                log.debug('Error processing xhtml: ' + str(e))
-                log.debug('with |' + xhtml + '|')
+                return iter_
+            except Exception as error:
+                log.debug('Error processing xhtml: %s', error)
+                log.debug('with |%s|', xhtml)
 
         # /me is replaced by name if name is given
         if name and (text.startswith('/me ') or text.startswith('/me\n')):
@@ -1380,8 +1289,7 @@ class ConversationTextview(GObject.GObject):
         if self.plugin_modified:
             if not mark:
                 return buffer_.get_end_iter()
-            else:
-                return buffer_.get_iter_at_mark(mark)
+            return buffer_.get_iter_at_mark(mark)
 
         if not mark:
             iter_ = buffer_.get_end_iter()
