@@ -36,29 +36,25 @@ from urllib.request import urlopen
 
 from gi.repository import Gtk
 from gi.repository import GdkPixbuf
-from gi.repository import Pango
 from gi.repository import GLib
 
-try:
-    from common import gajim as app
-    from plugins import GajimPlugin
-    from plugins.gui import GajimPluginConfigDialog
-    from htmltextview import HtmlTextView
-    from dialogs import WarningDialog, HigDialog, YesNoDialog
-    from gtkgui_helpers import get_action
-except ImportError:
-    from gajim.common import app
-    from gajim.plugins import GajimPlugin
-    from gajim.plugins.gui import GajimPluginConfigDialog
-    from gajim.htmltextview import HtmlTextView
-    from gajim.dialogs import WarningDialog, HigDialog, YesNoDialog
-    from gajim.gtkgui_helpers import get_action
+from gajim.common import app
+from gajim.common import configpaths
+from gajim.plugins import GajimPlugin
+from gajim.plugins.gui import GajimPluginConfigDialog
+from gajim.plugins.plugins_i18n import _
+from gajim.plugins.helpers import get_builder
+from gajim.gtk.dialogs import WarningDialog
+from gajim.gtk.dialogs import HigDialog
+from gajim.gtk.dialogs import YesNoDialog
+from gajim.gtk.dialogs import ConfirmationDialogCheck
+from gajim.gtkgui_helpers import get_action
 
 log = logging.getLogger('gajim.plugin_system.plugin_installer')
 
-PLUGINS_URL = 'https://ftp.gajim.org/plugins_1/'
-MANIFEST_URL = 'https://ftp.gajim.org/plugins_1/manifests.zip'
-MANIFEST_IMAGE_URL = 'https://ftp.gajim.org/plugins_1/manifests_images.zip'
+PLUGINS_URL = 'https://ftp.gajim.org/plugins_1.1_zip/'
+MANIFEST_URL = 'https://ftp.gajim.org/plugins_1.1_zip/manifests.zip'
+MANIFEST_IMAGE_URL = 'https://ftp.gajim.org/plugins_1.1_zip/manifests_images.zip'
 MANDATORY_FIELDS = ['name', 'version', 'description', 'authors', 'homepage']
 FALLBACK_ICON = Gtk.IconTheme.get_default().load_icon(
     'preferences-system', Gtk.IconSize.MENU, 0)
@@ -93,7 +89,7 @@ def get_local_version(plugin_manifest):
     if not active:
         return
     manifest_path = os.path.join(
-        app.PLUGINS_DIRS[1], short_name, 'manifest.ini')
+        configpaths.get('PLUGINS_USER'), short_name, 'manifest.ini')
     if not os.path.exists(manifest_path):
         return
     conf = configparser.ConfigParser()
@@ -113,7 +109,9 @@ class PluginInstaller(GajimPlugin):
     def init(self):
         self.description = _('Install and Upgrade Plugins')
         self.config_dialog = PluginInstallerPluginConfigDialog(self)
-        self.config_default_values = {'check_update': (True, '')}
+        self.config_default_values = {'check_update': (True, ''),
+                                      'auto_update': (False, ''),
+                                      'auto_update_feedback': (True, '')}
         self.gui_extension_points = {'plugin_window': (self.on_activate, None)}
         self.window = None
         self.progressbar = None
@@ -128,17 +126,20 @@ class PluginInstaller(GajimPlugin):
             self.on_activate(app.interface.instances['plugins'])
 
     def warn_update(self, plugins):
-        def open_update(dummy):
+        def open_update(checked):
+            if checked:
+                self.config['auto_update'] = True
             get_action('plugins').activate()
-            page = self.notebook.page_num(self.paned)
+            page = self.notebook.page_num(self.available_plugins_box)
             self.notebook.set_current_page(page)
         if plugins:
             plugins_str = '\n' + '\n'.join(plugins)
             YesNoDialog(
-                _('Plugins updates'),
-                _('Some updates are available for your installer plugins. '
-                  'Do you want to update those plugins:\n%s')
-                % plugins_str, on_response_yes=open_update)
+                _('Plugin updates'),
+                _('There are updates available for plugins you have installed.\n'
+                  'Do you want to update those plugins:\n%s') % plugins_str,
+                checktext=_('Update plugins automatically next time'),
+                on_response_yes=open_update)
         else:
             log.info('No updates found')
             if hasattr(self, 'thread'):
@@ -148,12 +149,13 @@ class PluginInstaller(GajimPlugin):
         if hasattr(self, 'thread'):
             return
         log.info('Checking for Updates...')
-        self.start_download(check_update=True)
+        auto_update = self.config['auto_update']
+        self.start_download(check_update=True, auto_update=auto_update)
         self.timeout_id = 0
 
     def deactivate(self):
         if hasattr(self, 'available_page'):
-            self.notebook.remove_page(self.notebook.page_num(self.paned))
+            self.notebook.remove_page(self.notebook.page_num(self.available_plugins_box))
             self.notebook.set_current_page(0)
             for id_, widget in list(self.connected_ids.items()):
                 widget.disconnect(id_)
@@ -177,40 +179,27 @@ class PluginInstaller(GajimPlugin):
         self.window = plugin_win.window
         id_ = self.window.connect('destroy', self.on_win_destroy)
         self.connected_ids[id_] = self.window
-        self.Gtk_BUILDER_FILE_PATH = self.local_file_path('config_dialog.ui')
-        self.xml = Gtk.Builder()
-        self.xml.set_translation_domain('gajim_plugins')
-        self.xml.add_objects_from_file(self.Gtk_BUILDER_FILE_PATH,
-                                       ['refresh', 'paned', 'plugin_store'])
+        path = self.local_file_path('installer.ui')
+        self._ui = get_builder(
+            path, widgets=['refresh', 'available_plugins_box', 'plugin_store'])
 
         widgets_to_extract = (
-            'name_label', 'available_treeview', 'progressbar', 'paned',
-            'install_button', 'authors_label', 'homepage_linkbutton',
-            'version_label', 'scrolled_description_window')
+            'available_plugins_box', 'install_plugin_button', 'plugin_name_label',
+            'plugin_version_label', 'plugin_authors_label', 'plugin_description',
+            'plugin_homepage_linkbutton', 'progressbar', 'available_plugins_treeview',
+            'available_text', 'available_text_label')
 
         for widget_name in widgets_to_extract:
-            setattr(self, widget_name, self.xml.get_object(widget_name))
-
-        # Make Link in LinkButton not centered
-        style_provider = Gtk.CssProvider()
-        css = '.link { padding-left: 0px; padding-right: 0px; }'
-        style_provider.load_from_data(css.encode())
-        context = self.homepage_linkbutton.get_style_context()
-        context.add_provider(style_provider,
-                             Gtk.STYLE_PROVIDER_PRIORITY_USER)
+            setattr(self, widget_name, self._ui.get_object(widget_name))
 
         self.available_page = self.notebook.append_page(
-            self.paned, Gtk.Label.new(_('Available')))
+            self.available_plugins_box, Gtk.Label.new(_('Available')))
 
-        self.available_plugins_model = self.xml.get_object('plugin_store')
+        self.available_plugins_model = self._ui.plugin_store
         self.available_plugins_model.set_sort_column_id(
             2, Gtk.SortType.ASCENDING)
 
-        self.description_textview = HtmlTextView()
-        self.description_textview.set_wrap_mode(Gtk.WrapMode.WORD)
-        self.scrolled_description_window.add(self.description_textview)
-
-        self.xml.connect_signals(self)
+        self._ui.connect_signals(self)
         self.window.show_all()
 
     def on_win_destroy(self, widget):
@@ -226,7 +215,7 @@ class PluginInstaller(GajimPlugin):
         for i in range(len(self.available_plugins_model)):
             if self.available_plugins_model[i][Column.UPGRADE]:
                 dir_list.append(self.available_plugins_model[i][Column.DIR])
-        self.install_button.set_property('sensitive', bool(dir_list))
+        self.install_plugin_button.set_property('sensitive', bool(dir_list))
 
     def on_notebook_switch_page(self, widget, page, page_num):
         tab_label_text = self.notebook.get_tab_label_text(page)
@@ -237,25 +226,24 @@ class PluginInstaller(GajimPlugin):
             self.start_download(upgrading=True)
 
     def on_install_upgrade_clicked(self, widget):
-        self.install_button.set_property('sensitive', False)
+        self.install_plugin_button.set_property('sensitive', False)
         dir_list = []
         for i in range(len(self.available_plugins_model)):
             if self.available_plugins_model[i][Column.UPGRADE]:
                 dir_list.append(self.available_plugins_model[i][Column.DIR])
 
-        self.start_download(remote_dirs=dir_list)
+        self.start_download(remote_dirs=dir_list, auto_update=False)
 
     def on_error(self, reason):
         if reason == 'CERTIFICATE_VERIFY_FAILED':
             YesNoDialog(
                 _('Security error during download'),
-                _('A security error occurred when '
+                _('A security error occurred while '
                   'downloading. The certificate of the '
                   'plugin archive could not be verified. '
-                  'this might be a security attack. '
-                  '\n\nYou can continue at your risk. '
-                  'Do you want to do so? '
-                  '(not recommended)'
+                  'This might be a security attack. '
+                  '\n\nYou can continue at your own risk (not recommended). '
+                  'Do you want to continue?'
                   ),
                 on_response_yes=lambda dlg:
                 self.start_download(secure=False, upgrading=True))
@@ -269,111 +257,99 @@ class PluginInstaller(GajimPlugin):
                           _('An error occurred when downloading\n\n'
                           '<tt>[%s]</tt>' % (str(text))), self.window)
 
-    def start_download(self, secure=True, remote_dirs=False,
-                       upgrading=False, check_update=False):
+    def start_download(self, secure=True, remote_dirs=False, upgrading=False,
+                       check_update=False, auto_update=False):
         log.info('Start Download...')
         log.debug(
-            'secure: %s, remote_dirs: %s, upgrading: %s, check_update: %s',
-            secure, remote_dirs, upgrading, check_update)
+            'secure: %s, remote_dirs: %s, upgrading: %s, check_update: %s, auto_update: %s',
+            secure, remote_dirs, upgrading, check_update, auto_update)
         self.thread = DownloadAsync(
-            self, secure=secure, remote_dirs=remote_dirs,
-            upgrading=upgrading, check_update=check_update)
+            self, secure=secure, remote_dirs=remote_dirs, upgrading=upgrading,
+            check_update=check_update, auto_update=auto_update)
         self.thread.start()
 
-    def on_plugin_downloaded(self, plugin_dirs):
+    def on_plugin_downloaded(self, plugin_dirs, auto_update):
+        need_restart = False
         for _dir in plugin_dirs:
-            is_active = False
-            plugins = None
-            plugin_dir = os.path.join(app.PLUGINS_DIRS[1], _dir)
-            plugin = app.plugin_manager.get_plugin_by_path(plugin_dir)
-            if plugin:
-                if plugin.active:
-                    is_active = True
-                    log.info('Deactivate Plugin: %s', plugin)
-                    app.plugin_manager.deactivate_plugin(plugin)
-                app.plugin_manager.plugins.remove(plugin)
+            updated = app.plugin_manager.update_plugins(replace=False, activate=True, plugin_name=_dir)
+            if updated:
+                if not auto_update:
+                    plugin = app.plugin_manager.get_active_plugin(updated[0])
+                    for row in range(len(self.available_plugins_model)):
+                        model_row = self.available_plugins_model[row]
+                        if plugin.name == model_row[Column.NAME]:
+                            model_row[Column.LOCAL_VERSION] = plugin.version
+                            model_row[Column.UPGRADE] = False
+                            break
+                if not auto_update:
+                    # Get plugin icon
+                    icon_file = os.path.join(plugin.__path__, os.path.split(
+                        plugin.__path__)[1]) + '.png'
+                    icon = FALLBACK_ICON
+                    if os.path.isfile(icon_file):
+                        icon = GdkPixbuf.Pixbuf.new_from_file_at_size(icon_file, 16, 16)
+                    row = [plugin, plugin.name, True, plugin.activatable, icon]
+                    self.installed_plugins_model.append(row)
+            else:
+                need_restart = True
 
-                model = self.installed_plugins_model
-                for row in range(len(model)):
-                    if plugin == model[row][0]:
-                        model.remove(model.get_iter((row, 0)))
-                        break
+        if need_restart:
+            txt = _('All plugins downloaded.\nThe updates will '
+                    'be installed the next time Gajim is started.')
+        else:
+            txt = _('All selected plugins downloaded and activated')
+        if not auto_update:
+            dialog = HigDialog(
+                self.window, Gtk.MessageType.INFO, Gtk.ButtonsType.OK, '', txt)
+            dialog.set_modal(False)
+            dialog.popup()
+        if auto_update and self.config['auto_update_feedback']:
+            def on_ok(checked):
+                if checked:
+                    self.config['auto_update_feedback'] = False
+            # Hide cancel button to mimic InfoDialogCheck
+            ConfirmationDialogCheck(_('Plugins updated'),
+                                    _('Plugin updates have successfully been downloaded.'
+                                      'Updates will be installed on next Gajim restart.'),
+                                    _('Do not show this message again'),
+                                    on_response_ok=on_ok).get_widget_for_response(
+                                        Gtk.ResponseType.CANCEL).hide()
+        if auto_update and not self.config['auto_update_feedback']:
+            log.info('Updates downloaded, will install on next restart')
 
-            log.info('Load Plugin from: %s', plugin_dir)
-            plugins = app.plugin_manager.scan_dir_for_plugins(
-                plugin_dir, package=True)
-            if not plugins:
-                log.warning('Loading Plugin failed')
-                continue
-            app.plugin_manager.add_plugin(plugins[0])
-            plugin = app.plugin_manager.plugins[-1]
-            log.info('Loading successful')
-            for row in range(len(self.available_plugins_model)):
-                model_row = self.available_plugins_model[row]
-                if plugin.name == model_row[Column.NAME]:
-                    model_row[Column.LOCAL_VERSION] = plugin.version
-                    model_row[Column.UPGRADE] = False
-            if is_active:
-                log.info('Activate Plugin: %s', plugin)
-                app.plugin_manager.activate_plugin(plugin)
-            # get plugin icon
-            icon_file = os.path.join(plugin.__path__, os.path.split(
-                plugin.__path__)[1]) + '.png'
-            icon = FALLBACK_ICON
-            if os.path.isfile(icon_file):
-                icon = GdkPixbuf.Pixbuf.new_from_file_at_size(icon_file, 16, 16)
-            row = [plugin, plugin.name, is_active, plugin.activatable, icon]
-            self.installed_plugins_model.append(row)
-
-        dialog = HigDialog(
-            self.window, Gtk.MessageType.INFO, Gtk.ButtonsType.OK,
-            '', _('All selected plugins downloaded'))
-        dialog.set_modal(False)
-        dialog.popup()
 
     def available_plugins_treeview_selection_changed(self, treeview_selection):
         model, iter_ = treeview_selection.get_selected()
-        self.description_textview.get_buffer().set_text('')
         if not iter_:
-            self.name_label.set_text('')
-            self.version_label.set_text('')
-            self.authors_label.set_text('')
-            self.homepage_linkbutton.set_uri('')
-            self.homepage_linkbutton.set_label('')
-            self.install_button.set_sensitive(False)
+            self.plugin_name_label.set_text('')
+            self.plugin_version_label.set_text('')
+            self.plugin_authors_label.set_text('')
+            self.plugin_homepage_linkbutton.set_text('')
+            self.install_plugin_button.set_sensitive(False)
             return
-        self.install_button.set_sensitive(True)
-        self.name_label.set_text(model.get_value(iter_, Column.NAME))
-        self.version_label.set_text(model.get_value(iter_, Column.VERSION))
-        self.authors_label.set_text(model.get_value(iter_, Column.AUTHORS))
-        self.homepage_linkbutton.set_uri(
-            model.get_value(iter_, Column.HOMEPAGE))
-        self.homepage_linkbutton.set_label(
-            model.get_value(iter_, Column.HOMEPAGE))
-        link_label = self.homepage_linkbutton.get_children()[0]
-        link_label.set_ellipsize(Pango.EllipsizeMode.END)
-        desc = _(model.get_value(iter_, Column.DESCRIPTION))
-        if not desc.startswith('<body '):
-            desc = ('<body xmlns=\'http://www.w3.org/1999/xhtml\'>'
-                    '%s</body>') % desc
-            desc = desc.replace('\n', '<br/>')
-        self.description_textview.display_html(
-            desc, self.description_textview, None)
+        self.plugin_name_label.set_text(model.get_value(iter_, Column.NAME))
+        self.plugin_version_label.set_text(model.get_value(iter_, Column.VERSION))
+        self.plugin_authors_label.set_text(model.get_value(iter_, Column.AUTHORS))
+        homepage = model.get_value(iter_, Column.HOMEPAGE)
+        markup = '<a href="%s">%s</a>' % (homepage, homepage)
+        self.plugin_homepage_linkbutton.set_markup(markup)
+        self.plugin_description.set_text(model.get_value(iter_, Column.DESCRIPTION))
 
     def select_root_iter(self):
-        selection = self.available_treeview.get_selection()
+        selection = self.available_plugins_treeview.get_selection()
         model, iter_ = selection.get_selected()
         if not iter_:
             iter_ = self.available_plugins_model.get_iter_first()
             selection.select_iter(iter_)
-        self.name_label.show()
-        self.homepage_linkbutton.show()
+        self.plugin_name_label.show()
+        self.plugin_homepage_linkbutton.show()
         path = self.available_plugins_model.get_path(iter_)
-        self.available_treeview.scroll_to_cell(path)
+        self.available_plugins_treeview.scroll_to_cell(path)
 
 
 class DownloadAsync(threading.Thread):
-    def __init__(self, plugin, secure, remote_dirs, upgrading, check_update):
+    def __init__(self, plugin, secure, remote_dirs,
+                 upgrading, check_update, auto_update):
         threading.Thread.__init__(self)
         self.plugin = plugin
         self.window = plugin.window
@@ -383,6 +359,7 @@ class DownloadAsync(threading.Thread):
         self.upgrading = upgrading
         self.secure = secure
         self.check_update = check_update
+        self.auto_update = auto_update
         self.pulse = None
 
     def model_append(self, row):
@@ -403,9 +380,10 @@ class DownloadAsync(threading.Thread):
             if self.check_update:
                 self.run_check_update()
             else:
-                GLib.idle_add(self.progressbar.show)
-                self.pulse = GLib.timeout_add(150, self.progressbar_pulse)
-                self.run_download_plugin_list()
+                if not self.auto_update:
+                    GLib.idle_add(self.progressbar.show)
+                    self.pulse = GLib.timeout_add(150, self.progressbar_pulse)
+                    self.run_download_plugin_list()
         except urllib.error.URLError as exc:
             if isinstance(exc.reason, ssl.SSLError):
                 ssl_reason = exc.reason.reason
@@ -461,7 +439,7 @@ class DownloadAsync(threading.Thread):
                 pix.close()
                 icon = pix.get_pixbuf()
 
-            # transform to dictonary
+            # Transform to dictonary
             config_dict = {}
             for key, value in config.items('info'):
                 config_dict[key] = value
@@ -493,38 +471,59 @@ class DownloadAsync(threading.Thread):
 
         return io.BytesIO(request.read())
 
+    def plugin_is_valid(self, plugin):
+        gajim_v = V(app.config.get('version'))
+        min_v = plugin.get('min_gajim_version', None)
+        min_v = V(min_v) if min_v else gajim_v
+        max_v = plugin.get('max_gajim_version', None)
+        max_v = V(max_v) if max_v else gajim_v
+        if (gajim_v >= min_v) and (gajim_v <= max_v):
+            return True
+        return False
+
     def run_check_update(self):
         to_update = []
+        auto_update_list = []
         zipbuf = self.download_url(MANIFEST_URL)
         plugin_list = self.parse_manifest(zipbuf)
         for plugin in plugin_list:
             local_version = get_local_version(plugin)
             if local_version:
-                gajim_v = V(app.config.get('version'))
-                min_v = plugin.get('min_gajim_version', None)
-                min_v = V(min_v) if min_v else gajim_v
-                max_v = plugin.get('max_gajim_version', None)
-                max_v = V(max_v) if max_v else gajim_v
                 if (V(plugin['version']) > V(local_version)) and \
-                (gajim_v >= min_v) and (gajim_v <= max_v):
+                self.plugin_is_valid(plugin):
                     to_update.append(plugin['name'])
-        GLib.idle_add(self.plugin.warn_update, to_update)
+                    auto_update_list.append(plugin['remote_dir'])
+        if not self.auto_update:
+            GLib.idle_add(self.plugin.warn_update, to_update)
+        else:
+            if auto_update_list:
+                self.remote_dirs = auto_update_list
+                GLib.idle_add(self.download_plugin)
+            else:
+                log.info('No updates found')
+                if hasattr(self.plugin, 'thread'):
+                    del self.plugin.thread
 
     def run_download_plugin_list(self):
         if not self.remote_dirs:
             log.info('Downloading Pluginlist...')
             zipbuf = self.download_url(MANIFEST_IMAGE_URL)
             plugin_list = self.parse_manifest(zipbuf)
+            nb_plugins = 0
             for plugin in plugin_list:
+                if not self.plugin_is_valid(plugin):
+                    continue
+                nb_plugins += 1
                 plugin['local_version'] = get_local_version(plugin)
                 if self.upgrading and plugin['local_version']:
                     if V(plugin['version']) > V(plugin['local_version']):
                         plugin['upgrade'] = True
                         GLib.idle_add(
-                            self.plugin.install_button.set_property,
+                            self.plugin.install_plugin_button.set_property,
                             'sensitive', True)
                 GLib.idle_add(self.model_append, plugin)
-            GLib.idle_add(self.plugin.select_root_iter)
+            if nb_plugins:
+                GLib.idle_add(self.plugin.select_root_iter)
         else:
             self.download_plugin()
 
@@ -532,15 +531,14 @@ class DownloadAsync(threading.Thread):
         for remote_dir in self.remote_dirs:
             filename = remote_dir + '.zip'
             log.info('Download: %s', filename)
-            base_dir, user_dir = app.PLUGINS_DIRS
-            if not os.path.isdir(user_dir):
-                os.mkdir(user_dir)
+
+            user_dir = configpaths.get('PLUGINS_DOWNLOAD')
             local_dir = os.path.join(user_dir, remote_dir)
             if not os.path.isdir(local_dir):
                 os.mkdir(local_dir)
             local_dir = os.path.dirname(local_dir)
 
-            # downloading zip file
+            # Downloading zip file
             try:
                 plugin = posixpath.join(PLUGINS_URL, filename)
                 buf = self.download_url(plugin)
@@ -549,23 +547,37 @@ class DownloadAsync(threading.Thread):
                 continue
             with ZipFile(buf) as zip_file:
                 zip_file.extractall(local_dir)
-        GLib.idle_add(self.plugin.on_plugin_downloaded, self.remote_dirs)
+        GLib.idle_add(self.plugin.on_plugin_downloaded,
+                      self.remote_dirs, self.auto_update)
 
 
 class PluginInstallerPluginConfigDialog(GajimPluginConfigDialog):
     def init(self):
-        glade_file_path = self.plugin.local_file_path('config_dialog.ui')
-        self.xml = Gtk.Builder()
-        self.xml.set_translation_domain('gajim_plugins')
-        self.xml.add_objects_from_file(glade_file_path, ['config_grid'])
-        grid = self.xml.get_object('config_grid')
-        self.get_child().pack_start(grid, True, True, 0)
+        glade_file_path = self.plugin.local_file_path('config.ui')
+        self._ui = get_builder(glade_file_path)
+        self.get_child().pack_start(self._ui.config_grid, True, True, 0)
 
-        self.xml.connect_signals(self)
+        self._ui.connect_signals(self)
 
     def on_run(self):
-        self.xml.get_object('check_update').set_active(
-            self.plugin.config['check_update'])
+        self._ui.check_update.set_active(self.plugin.config['check_update'])
+        self._ui.auto_update.set_sensitive(self.plugin.config['check_update'])
+        self._ui.auto_update.set_active(self.plugin.config['auto_update'])
+        self._ui.auto_update_feedback.set_sensitive(self.plugin.config['auto_update'])
+        self._ui.auto_update_feedback.set_active(self.plugin.config['auto_update_feedback'])
 
     def on_check_update_toggled(self, widget):
         self.plugin.config['check_update'] = widget.get_active()
+        if not self.plugin.config['check_update']:
+            self.plugin.config['auto_update'] = False
+        self._ui.auto_update.set_sensitive(self.plugin.config['check_update'])
+        self._ui.auto_update.set_active(self.plugin.config['auto_update'])
+        self._ui.auto_update_feedback.set_sensitive(self.plugin.config['auto_update'])
+        self._ui.auto_update_feedback.set_active(self.plugin.config['auto_update_feedback'])
+
+    def on_auto_update_toggled(self, widget):
+        self.plugin.config['auto_update'] = widget.get_active()
+        self._ui.auto_update_feedback.set_sensitive(self.plugin.config['auto_update'])
+
+    def on_auto_update_feedback_toggled(self, widget):
+        self.plugin.config['auto_update_feedback'] = widget.get_active()
