@@ -1,27 +1,23 @@
-# -*- coding: utf-8 -*-
-## src/disco.py
-##
-## Copyright (C) 2005-2006 Stéphan Kochen <stephan AT kochen.nl>
-## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
-## Copyright (C) 2005-2014 Yann Leboulanger <asterix AT lagaule.org>
-## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
-## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
-##
-## This file is part of Gajim.
-##
-## Gajim is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 3 only.
-##
-## Gajim is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
-##
+# Copyright (C) 2005-2006 Stéphan Kochen <stephan AT kochen.nl>
+# Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+# Copyright (C) 2005-2014 Yann Leboulanger <asterix AT lagaule.org>
+# Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+# Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+# Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
+#
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 # The appearance of the treeview, and parts of the dialog, are controlled by
 # AgentBrowser (sub-)classes. Methods that probably should be overridden when
@@ -46,24 +42,26 @@
 
 import types
 import weakref
+
+import nbxmpp
 from gi.repository import GLib
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
-from gi.repository import Pango
 
-from gajim import dialogs
 from gajim import gtkgui_helpers
 from gajim import groups
 from gajim import adhoc_commands
 from gajim import search_window
-from gajim import gui_menu_builder
 
 from gajim.common import app
-import nbxmpp
-from gajim.common.exceptions import GajimGeneralException
+from gajim.common.i18n import _
 from gajim.common import helpers
-from gajim.common import ged
+from gajim.common.const import StyleAttr
+
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.dialogs import InformationDialog
+from gajim.gtk.service_registration import ServiceRegistration
 
 LABELS = {
     1: _('This service has not yet responded with detailed information'),
@@ -102,20 +100,14 @@ def _gen_agent_type_info():
         ('conference', 'irc'):          (ToplevelAgentBrowser, 'irc'),
         ('_jid', 'irc'):                (False, 'irc'),
         ('gateway', 'irc'):             (False, 'irc'),
-        ('gateway', 'aim'):             (False, 'aim'),
-        ('_jid', 'aim'):                (False, 'aim'),
         ('gateway', 'gadu-gadu'):       (False, 'gadu-gadu'),
         ('_jid', 'gadugadu'):           (False, 'gadu-gadu'),
         ('gateway', 'http-ws'):         (False, 'http-ws'),
         ('gateway', 'icq'):             (False, 'icq'),
         ('_jid', 'icq'):                (False, 'icq'),
-        ('gateway', 'msn'):             (False, 'msn'),
-        ('_jid', 'msn'):                (False, 'msn'),
         ('gateway', 'sms'):             (False, 'sms'),
         ('_jid', 'sms'):                (False, 'sms'),
         ('gateway', 'smtp'):            (False, 'mail'),
-        ('gateway', 'yahoo'):           (False, 'yahoo'),
-        ('_jid', 'yahoo'):              (False, 'yahoo'),
         ('gateway', 'mrim'):            (False, 'mrim'),
         ('_jid', 'mrim'):               (False, 'mrim'),
         ('gateway', 'facebook'):        (False, 'facebook'),
@@ -138,10 +130,10 @@ class CacheDictionary:
     """
     A dictionary that keeps items around for only a specific time.  Lifetime is
     in minutes. Getrefresh specifies whether to refresh when an item is merely
-    accessed instead of set aswell
+    accessed instead of set as well
     """
 
-    def __init__(self, lifetime, getrefresh = True):
+    def __init__(self, lifetime, getrefresh=True):
         self.lifetime = lifetime * 1000 * 60
         self.getrefresh = getrefresh
         self.cache = {}
@@ -205,27 +197,26 @@ class CacheDictionary:
 
 _icon_cache = CacheDictionary(15)
 
-def get_agent_address(jid, node = None):
+def get_agent_address(jid, node=None):
     """
     Get an agent's address for displaying in the GUI
     """
     if node:
         return '%s@%s' % (node, str(jid))
-    else:
-        return str(jid)
+    return str(jid)
 
-class Closure(object):
+class Closure:
     """
     A weak reference to a callback with arguments as an object
 
-    Weak references to methods immediatly die, even if the object is still
+    Weak references to methods immediately die, even if the object is still
     alive. Besides a handy way to store a callback, this provides a workaround
     that keeps a reference to the object instead.
 
     Userargs and removeargs must be tuples.
     """
 
-    def __init__(self, cb, userargs = (), remove = None, removeargs = ()):
+    def __init__(self, cb, userargs=(), remove=None, removeargs=()):
         self.userargs = userargs
         self.remove = remove
         self.removeargs = removeargs
@@ -260,28 +251,10 @@ class ServicesCache:
 
     def __init__(self, account):
         self.account = account
-        self._items = CacheDictionary(0, getrefresh = False)
-        self._info = CacheDictionary(0, getrefresh = False)
+        self._items = CacheDictionary(0, getrefresh=False)
+        self._info = CacheDictionary(0, getrefresh=False)
         self._subscriptions = CacheDictionary(5, getrefresh=False)
         self._cbs = {}
-        app.ged.register_event_handler('agent-items-received', ged.GUI1,
-            self._nec_agent_items_received)
-        app.ged.register_event_handler('agent-items-error-received', ged.GUI1,
-            self._nec_agent_items_error_received)
-        app.ged.register_event_handler('agent-info-received', ged.GUI1,
-                self._nec_agent_info_received)
-        app.ged.register_event_handler('agent-info-error-received', ged.GUI1,
-                self._nec_agent_info_error_received)
-
-    def __del__(self):
-        app.ged.remove_event_handler('agent-items-received', ged.GUI1,
-            self._nec_agent_items_received)
-        app.ged.remove_event_handler('agent-items-error-received', ged.GUI1,
-            self._nec_agent_items_error_received)
-        app.ged.remove_event_handler('agent-info-received', ged.GUI1,
-                self._nec_agent_info_received)
-        app.ged.remove_event_handler('agent-info-error-received', ged.GUI1,
-                self._nec_agent_info_error_received)
 
     def cleanup(self):
         self._items.cleanup()
@@ -369,7 +342,7 @@ class ServicesCache:
         # NS_BROWSE is deprecated, but we check for it anyways.
         # Some services list it in features and respond to
         # NS_DISCO_ITEMS anyways.
-        # Allow browsing for unknown types aswell.
+        # Allow browsing for unknown types as well.
         if (not features and not identities) or \
         nbxmpp.NS_DISCO_ITEMS in features or nbxmpp.NS_BROWSE in features:
             return ToplevelAgentBrowser
@@ -397,7 +370,9 @@ class ServicesCache:
             self._cbs[cbkey].append(cb)
         else:
             self._cbs[cbkey] = [cb]
-            app.connections[self.account].discoverInfo(jid, node)
+            con = app.connections[self.account]
+            con.get_module('Discovery').disco_info(
+                jid, node, self._disco_info_received, self._disco_info_error)
 
     def get_items(self, jid, node, cb, force=False, nofetch=False, args=()):
         """
@@ -415,24 +390,42 @@ class ServicesCache:
         # Create a closure object
         cbkey = ('items', addr)
         cb = Closure(cb, userargs=args, remove=self._clean_closure,
-            removeargs=cbkey)
+                     removeargs=cbkey)
         # Are we already fetching this?
         if cbkey in self._cbs:
             self._cbs[cbkey].append(cb)
         else:
             self._cbs[cbkey] = [cb]
-            app.connections[self.account].discoverItems(jid, node)
+            con = app.connections[self.account]
+            con.get_module('Discovery').disco_items(
+                jid, node, self._disco_items_received, self._disco_items_error)
 
-    def _nec_agent_info_received(self, obj):
+    def _disco_info_received(self, from_, identities, features, data, node):
         """
         Callback for when we receive an agent's info
         array is (agent, node, identities, features, data)
         """
-        # We receive events from all accounts from GED
-        if obj.conn.name != self.account:
-            return
-        self._on_agent_info(obj.fjid, obj.node, obj.identities, obj.features,
-            obj.data)
+        if not identities:
+            # Ejabberd doesn't send identities when using admin nodes
+            identities = [{'category': 'server', 'type': 'im', 'name': node}]
+
+        self._on_agent_info(str(from_), node, identities, features, data)
+
+    def _disco_info_error(self, from_, error):
+        """
+        Callback for when a query fails. Even after the browse and agents
+        namespaces
+        """
+        addr = get_agent_address(from_)
+
+        # Call callbacks
+        cbkey = ('info', addr)
+        if cbkey in self._cbs:
+            for cb in self._cbs[cbkey]:
+                cb(str(from_), '', 0, 0, 0)
+            # clean_closure may have beaten us to it
+            if cbkey in self._cbs:
+                del self._cbs[cbkey]
 
     def _on_agent_info(self, fjid, node, identities, features, data):
         addr = get_agent_address(fjid, node)
@@ -449,69 +442,43 @@ class ServicesCache:
             if cbkey in self._cbs:
                 del self._cbs[cbkey]
 
-    def _nec_agent_items_received(self, obj):
+    def _disco_items_received(self, from_, node, items):
         """
         Callback for when we receive an agent's items
         array is (agent, node, items)
         """
-        # We receive events from all accounts from GED
-        if obj.conn.name != self.account:
-            return
-
-        addr = get_agent_address(obj.fjid, obj.node)
+        addr = get_agent_address(from_, node)
 
         # Store in cache
-        self._items[addr] = obj.items
+        self._items[addr] = items
 
         # Call callbacks
         cbkey = ('items', addr)
         if cbkey in self._cbs:
             for cb in self._cbs[cbkey]:
-                cb(obj.fjid, obj.node, obj.items)
+                cb(str(from_), node, items)
             # clean_closure may have beaten us to it
             if cbkey in self._cbs:
                 del self._cbs[cbkey]
 
-    def _nec_agent_info_error_received(self, obj):
+    def _disco_items_error(self, from_, error):
         """
         Callback for when a query fails. Even after the browse and agents
         namespaces
         """
-        # We receive events from all accounts from GED
-        if obj.conn.name != self.account:
-            return
-        addr = get_agent_address(obj.fjid)
-
-        # Call callbacks
-        cbkey = ('info', addr)
-        if cbkey in self._cbs:
-            for cb in self._cbs[cbkey]:
-                cb(obj.fjid, '', 0, 0, 0)
-            # clean_closure may have beaten us to it
-            if cbkey in self._cbs:
-                del self._cbs[cbkey]
-
-    def _nec_agent_items_error_received(self, obj):
-        """
-        Callback for when a query fails. Even after the browse and agents
-        namespaces
-        """
-        # We receive events from all accounts from GED
-        if obj.conn.name != self.account:
-            return
-        addr = get_agent_address(obj.fjid)
+        addr = get_agent_address(from_)
 
         # Call callbacks
         cbkey = ('items', addr)
         if cbkey in self._cbs:
             for cb in self._cbs[cbkey]:
-                cb(obj.fjid, '', 0)
+                cb(str(from_), '', 0)
             # clean_closure may have beaten us to it
             if cbkey in self._cbs:
                 del self._cbs[cbkey]
 
-# object is needed so that @property works
-class ServiceDiscoveryWindow(object):
+
+class ServiceDiscoveryWindow:
     """
     Class that represents the Services Discovery window
     """
@@ -533,7 +500,7 @@ class ServiceDiscoveryWindow(object):
 
         # Check connection
         if app.connections[account].connected < 2:
-            dialogs.ErrorDialog(_('You are not connected to the server'),
+            ErrorDialog(_('You are not connected to the server'),
 _('Without a connection, you can not browse available services'))
             raise RuntimeError('You must be connected to browse services')
 
@@ -629,37 +596,13 @@ _('Without a connection, you can not browse available services'))
         self.banner_icon.clear()
         self.banner_icon.hide() # Just clearing it doesn't work
 
-    def _set_window_banner_text(self, text, text_after = None):
-        theme = app.config.get('roster_theme')
-        bannerfont = app.config.get_per('themes', theme, 'bannerfont')
-        bannerfontattrs = app.config.get_per('themes', theme,
-                'bannerfontattrs')
-
-        if bannerfont:
-            font = Pango.FontDescription(bannerfont)
-        else:
-            font = Pango.FontDescription('Normal')
-        if bannerfontattrs:
-            # B is attribute set by default
-            if 'B' in bannerfontattrs:
-                font.set_weight(Pango.Weight.HEAVY)
-            if 'I' in bannerfontattrs:
-                font.set_style(Pango.Style.ITALIC)
-
-        font_attrs = 'font_desc="%s"' % font.to_string()
-        font_size = font.get_size()
-
-        # in case there is no font specified we use x-large font size
-        if font_size == 0:
-            font_attrs = '%s size="large"' % font_attrs
-        markup = '<span %s>%s</span>' % (font_attrs, text)
-        if text_after:
-            font.set_weight(Pango.Weight.NORMAL)
-            markup = '%s\n<span font_desc="%s" size="small">%s</span>' % \
-                (markup, font.to_string(), text_after)
+    def _set_window_banner_text(self, text, text_after=None):
+        markup = '<span>%s</span>' % (text)
+        if text_after is not None:
+            markup += '\n<span size="x-small">%s</span>' % text_after
         self.banner.set_markup(markup)
 
-    def destroy(self, chain = False):
+    def destroy(self, chain=False):
         """
         Close the browser. This can optionally close its children and propagate
         to the parent. This should happen on actions like register, or join to
@@ -683,13 +626,13 @@ _('Without a connection, you can not browse available services'))
         for child in self.children[:]:
             child.parent = None
             if chain:
-                child.destroy(chain = chain)
+                child.destroy(chain=chain)
                 self.children.remove(child)
         if self.parent:
             if self in self.parent.children:
                 self.parent.children.remove(self)
             if chain and not self.parent.children:
-                self.parent.destroy(chain = chain)
+                self.parent.destroy(chain=chain)
                 self.parent = None
         else:
             self.cache.cleanup()
@@ -729,18 +672,20 @@ _('Without a connection, you can not browse available services'))
             if not self.address_comboboxtext:
                 # We can't travel anywhere else.
                 self.destroy()
-            dialogs.ErrorDialog(_('The service could not be found'),
+            ErrorDialog(
+                _('The service could not be found'),
                 _('There is no service at the address you entered, or it is '
-                'not responding. Check the address and try again.'),
+                  'not responding. Check the address and try again.'),
                 transient_for=self.window)
             return
         klass = self.cache.get_browser(identities, features)
         if not klass:
-            dialogs.ErrorDialog(_('The service is not browsable'),
+            ErrorDialog(
+                _('The service is not browsable'),
                 _('This type of service does not contain any items to browse.'),
                 transient_for=self.window)
             return
-        elif klass is None:
+        if klass is None:
             klass = AgentBrowser
         self.browser = klass(self.account, jid, node)
         self.browser.prepare_window(self)
@@ -779,7 +724,7 @@ _('Without a connection, you can not browse available services'))
                 jid = helpers.parse_jid(jid)
             except helpers.InvalidFormat as s:
                 pritext = _('Invalid Server Name')
-                dialogs.ErrorDialog(pritext, str(s))
+                ErrorDialog(pritext, str(s))
                 return
             self.travel(jid, '')
 
@@ -789,7 +734,7 @@ _('Without a connection, you can not browse available services'))
             jid = helpers.parse_jid(jid)
         except helpers.InvalidFormat as s:
             pritext = _('Invalid Server Name')
-            dialogs.ErrorDialog(pritext, str(s),
+            ErrorDialog(pritext, str(s),
                 transient_for=self.window)
             return
         if jid == self.jid: # jid has not changed
@@ -806,7 +751,7 @@ _('Without a connection, you can not browse available services'))
                 ' '.join(self.latest_addresses))
         self.travel(jid, '')
 
-    def on_services_treeview_row_activated(self, widget, path, col = 0):
+    def on_services_treeview_row_activated(self, widget, path, col=0):
         if self.browser:
             self.browser.default_action()
 
@@ -913,9 +858,9 @@ class AgentBrowser:
         name = ''
         if len(identities) > 1:
             # Check if an identity with server category is present
-            for i, _identity in enumerate(identities):
-                if _identity['category'] == 'server' and 'name' in _identity:
-                    name = _identity['name']
+            for _index, identity in enumerate(identities):
+                if identity['category'] == 'server' and 'name' in identity:
+                    name = identity['name']
                     break
                 elif 'name' in identities[0]:
                     name = identities[0]['name']
@@ -975,7 +920,7 @@ class AgentBrowser:
         """
         pass
 
-    def on_browse_button_clicked(self, widget = None):
+    def on_browse_button_clicked(self, widget=None):
         """
         When we want to browse an agent: open a new services window with a
         browser for the agent type
@@ -1000,7 +945,7 @@ class AgentBrowser:
         jid = model[iter_][0]
         node = model[iter_][1]
         if jid:
-            self.cache.get_info(jid, node, self._update_actions, nofetch = True)
+            self.cache.get_info(jid, node, self._update_actions, nofetch=True)
 
     def _update_actions(self, jid, node, identities, features, data):
         """
@@ -1023,7 +968,7 @@ class AgentBrowser:
         jid = model[iter_][0]
         node = model[iter_][1]
         if jid:
-            self.cache.get_info(jid, node, self._default_action, nofetch = True)
+            self.cache.get_info(jid, node, self._default_action, nofetch=True)
 
     def _default_action(self, jid, node, identities, features, data):
         """
@@ -1088,7 +1033,7 @@ class AgentBrowser:
             if not self.window.address_comboboxtext:
                 # We can't travel anywhere else.
                 self.window.destroy()
-            dialogs.ErrorDialog(_('The service is not browsable'),
+            ErrorDialog(_('The service is not browsable'),
                 _('This service does not contain any items to browse.'),
                 transient_for=self.window.window)
             return
@@ -1139,7 +1084,7 @@ class AgentBrowser:
         """
         self.model.append((jid, node, item.get('name', ''),
                 get_agent_address(jid, node)))
-        self.cache.get_info(jid, node, self._agent_info, force = force)
+        self.cache.get_info(jid, node, self._agent_info, force=force)
 
     def _update_item(self, iter_, jid, node, item):
         """
@@ -1224,11 +1169,10 @@ class ToplevelAgentBrowser(AgentBrowser):
                 # fetching or error
                 cell.set_property('foreground_set', True)
             else:
-                # Normal/succes
+                # Normal/success
                 cell.set_property('foreground_set', False)
         else:
-            theme = app.config.get('roster_theme')
-            bgcolor = app.config.get_per('themes', theme, 'groupbgcolor')
+            bgcolor = app.css_config.get_value('.gajim-group-row', StyleAttr.BACKGROUND)
             if bgcolor:
                 cell.set_property('cell_background_set', True)
             cell.set_property('foreground_set', False)
@@ -1255,7 +1199,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 
     def _create_treemodel(self):
         # JID, node, icon, description, state
-        # state is None on sucess or has a string
+        # state is None on success or has a string
         # from LABELS on error or while fetching
         view = self.window.services_treeview
         self.model = Gtk.TreeStore(str, str, GdkPixbuf.Pixbuf, str, str)
@@ -1342,7 +1286,7 @@ class ToplevelAgentBrowser(AgentBrowser):
             self.search_button = None
         AgentBrowser._clean_actions(self)
 
-    def on_search_button_clicked(self, widget = None):
+    def on_search_button_clicked(self, widget=None):
         """
         When we want to search something: open search window
         """
@@ -1361,8 +1305,7 @@ class ToplevelAgentBrowser(AgentBrowser):
         AgentBrowser.cleanup(self)
 
     def update_theme(self):
-        theme = app.config.get('roster_theme')
-        bgcolor = app.config.get_per('themes', theme, 'groupbgcolor')
+        bgcolor = app.css_config.get_value('.gajim-group-row', StyleAttr.BACKGROUND)
         if bgcolor:
             self._renderer.set_property('cell-background', bgcolor)
         self.window.services_treeview.queue_draw()
@@ -1378,7 +1321,7 @@ class ToplevelAgentBrowser(AgentBrowser):
         node = model[iter_][1]
         adhoc_commands.CommandWindow(self.account, service, commandnode=node)
 
-    def on_register_button_clicked(self, widget = None):
+    def on_register_button_clicked(self, widget=None):
         """
         When we want to register an agent: request information about registering
         with the agent and close the window
@@ -1388,8 +1331,8 @@ class ToplevelAgentBrowser(AgentBrowser):
             return
         jid = model[iter_][0]
         if jid:
-            app.connections[self.account].request_register_agent_info(jid)
-            self.window.destroy(chain = True)
+            ServiceRegistration(self.account, jid)
+            self.window.destroy(chain=True)
 
     def on_join_button_clicked(self, widget):
         """
@@ -1428,8 +1371,8 @@ class ToplevelAgentBrowser(AgentBrowser):
             # Guess what kind of service we're dealing with
             if self.browse_button:
                 jid = model[iter_][0]
-                type_ = app.get_transport_name_from_jid(jid,
-                                        use_config_setting = False)
+                type_ = app.get_transport_name_from_jid(
+                    jid, use_config_setting=False)
                 if type_:
                     identity = {'category': '_jid', 'type': type_}
                     klass = self.cache.get_browser([identity])
@@ -1448,7 +1391,7 @@ class ToplevelAgentBrowser(AgentBrowser):
             self.execute_button.set_sensitive(True)
         if self.search_button and nbxmpp.NS_SEARCH in features:
             self.search_button.set_sensitive(True)
-        # Don't autorize to register with a server via disco
+        # Don't authorize to register with a server via disco
         if self.register_button and nbxmpp.NS_REGISTER in features and \
         jid != self.jid:
             # We can register this agent
@@ -1479,7 +1422,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 
     def browse(self, force=False):
         self._progress = 0
-        AgentBrowser.browse(self, force = force)
+        AgentBrowser.browse(self, force=force)
 
     def _expand_all(self):
         """
@@ -1595,8 +1538,8 @@ class ToplevelAgentBrowser(AgentBrowser):
             descr = "<b>%s</b>" % addr
         # Guess which kind of service this is
         identities = []
-        type_ = app.get_transport_name_from_jid(jid,
-                                use_config_setting = False)
+        type_ = app.get_transport_name_from_jid(
+            jid, use_config_setting=False)
         if type_:
             identity = {'category': '_jid', 'type': type_}
             identities.append(identity)
@@ -1626,11 +1569,14 @@ class ToplevelAgentBrowser(AgentBrowser):
 
     def _update_info(self, iter_, jid, node, identities, features, data):
         addr = get_agent_address(jid, node)
-        name = identities[0].get('name', '')
-        if name:
-            descr = "<b>%s</b>\n%s" % (name, addr)
-        else:
+        if not identities:
             descr = "<b>%s</b>" % addr
+        else:
+            name = identities[0].get('name', '')
+            if name:
+                descr = "<b>%s</b>\n%s" % (name, addr)
+            else:
+                descr = "<b>%s</b>" % addr
 
         # Update progress
         self._progress += 1
@@ -1764,33 +1710,27 @@ class MucBrowser(AgentBrowser):
             self.join_button = None
 
     def on_bookmark_button_clicked(self, *args):
-        model, iter = self.window.services_treeview.get_selection().get_selected()
-        if not iter:
+        con = app.connections[self.account]
+        model, iter_ = self.window.services_treeview.get_selection().get_selected()
+        if not iter_:
             return
-        name = app.config.get_per('accounts', self.account, 'name')
-        room_jid = model[iter][0]
-        bm = {
-                'name': room_jid.split('@')[0],
-                'jid': room_jid,
-                'autojoin': '0',
-                'minimize': '0',
-                'password': '',
-                'nick': name
-        }
 
-        for bookmark in app.connections[self.account].bookmarks:
-            if bookmark['jid'] == bm['jid']:
-                dialogs.ErrorDialog( _('Bookmark already set'),
-                _('Group Chat "%s" is already in your bookmarks.') % bm['jid'],
+        room_jid = model[iter_][0]
+        if room_jid in con.get_module('Bookmarks').bookmarks:
+            ErrorDialog(
+                _('Bookmark already set'),
+                _('Group Chat "%s" is already in your bookmarks.') % room_jid,
                 transient_for=self.window.window)
-                return
+            return
 
-        app.connections[self.account].bookmarks.append(bm)
-        app.connections[self.account].store_bookmarks()
+        con.get_module('Bookmarks').add_bookmark(room_jid.split('@')[0],
+                                                 room_jid,
+                                                 False,
+                                                 False,
+                                                 ''
+                                                 '')
 
-        gui_menu_builder.build_bookmark_menu(self.account)
-
-        dialogs.InformationDialog(
+        InformationDialog(
             _('Bookmark has been added successfully'),
             _('You can manage your bookmarks via Actions menu in your roster.'),
             transient_for=self.window.window)
@@ -1804,11 +1744,12 @@ class MucBrowser(AgentBrowser):
         if not iter_:
             return
         service = model[iter_][0]
-        if 'join_gc' not in app.interface.instances[self.account]:
+        window = app.get_app_window('JoinGroupchatWindow')
+        if window is None:
             app.interface.join_gc_minimal(self.account, service)
         else:
-            app.interface.instances[self.account]['join_gc'].set_room(service)
-            app.interface.instances[self.account]['join_gc'].present()
+            window.set_room(service)
+            window.present()
             self.window.destroy()
 
     def update_actions(self):
@@ -1831,7 +1772,7 @@ class MucBrowser(AgentBrowser):
 
     def on_scroll(self, *args):
         """
-        Scrollwindow callback to trigger new queries on scolling
+        Scrollwindow callback to trigger new queries on scrolling
         """
         # This apparently happens when inactive sometimes
         self._query_visible()
@@ -1865,9 +1806,9 @@ class MucBrowser(AgentBrowser):
             iter_ = self.model.iter_next(iter_)
         self._fetch_source = None
 
-    def _channel_altinfo(self, jid, node, items, name = None):
+    def _channel_altinfo(self, jid, node, items, name=None):
         """
-        Callback for the alternate disco#items query. We try to atleast get the
+        Callback for the alternate disco#items query. We try to at least get the
         amount of users in the room if the service does not support MUC dataforms
         """
         if items == 0:
@@ -1921,7 +1862,7 @@ class MucBrowser(AgentBrowser):
                 break
         else:
             # We didn't find a form, switch to alternate query mode
-            self.cache.get_items(jid, node, self._channel_altinfo, args = (name,))
+            self.cache.get_items(jid, node, self._channel_altinfo, args=(name,))
             return
         # Continue with the next
         self._fetch_source = None
@@ -1958,14 +1899,15 @@ class DiscussionGroupsBrowser(AgentBrowser):
         self.subscribe_button = None
         self.unsubscribe_button = None
 
-        app.connections[account].send_pb_subscription_query(jid,
-            self._on_pep_subscriptions)
+        con = app.connections[account]
+        con.get_module('PubSub').send_pb_subscription_query(
+            jid, self._on_pep_subscriptions)
 
     def _create_treemodel(self):
         """
         Create treemodel for the window
         """
-        # JID, node, name (with description) - pango markup, dont have info?, subscribed?
+        # JID, node, name (with description) - pango markup, don't have info?, subscribed?
         self.model = Gtk.TreeStore(str, str, str, bool, bool)
         # sort by name
         self.model.set_sort_column_id(2, Gtk.SortType.ASCENDING)
@@ -2038,8 +1980,8 @@ class DiscussionGroupsBrowser(AgentBrowser):
             parent_iter = None
         if not node or not self._in_list(node):
             self.model.append(parent_iter, (jid, node, name, dunno, subscribed))
-            self.cache.get_items(jid, node, self._add_items, force = force,
-                    args = (force,))
+            self.cache.get_items(
+                jid, node, self._add_items, force=force, args=(force,))
 
     def _get_child_iter(self, parent_iter, node):
         child_iter = self.model.iter_children(parent_iter)
@@ -2052,9 +1994,9 @@ class DiscussionGroupsBrowser(AgentBrowser):
     def _get_iter(self, node):
         ''' Look for an iter with the given node '''
         self.found_iter = None
-        def is_node(model, path, iter, node):
-            if model[iter][1] == node:
-                self.found_iter = iter
+        def is_node(model, path, iter_, node):
+            if model[iter_][1] == node:
+                self.found_iter = iter_
                 return True
         self.model.foreach(is_node, node)
         return self.found_iter
@@ -2094,10 +2036,11 @@ class DiscussionGroupsBrowser(AgentBrowser):
     def update_actions(self):
         """
         Called when user selected a row. Make subscribe/unsubscribe buttons
-        sensitive appropriatelly
+        sensitive appropriately
         """
         # we have nothing to do if we don't have buttons...
-        if self.subscribe_button is None: return
+        if self.subscribe_button is None:
+            return
 
         model, iter_ = self.window.services_treeview.get_selection().get_selected()
         if not iter_ or self.subscriptions is None:
@@ -2116,7 +2059,8 @@ class DiscussionGroupsBrowser(AgentBrowser):
         Called when 'post' button is pressed. Open window to create post
         """
         model, iter_ = self.window.services_treeview.get_selection().get_selected()
-        if iter_ is None: return
+        if iter_ is None:
+            return
 
         groupnode = model.get_value(iter_, 1)   # 1 = groupnode
 
@@ -2127,24 +2071,28 @@ class DiscussionGroupsBrowser(AgentBrowser):
         Called when 'subscribe' button is pressed. Send subscribtion request
         """
         model, iter_ = self.window.services_treeview.get_selection().get_selected()
-        if iter_ is None: return
+        if iter_ is None:
+            return
 
-        groupnode = model.get_value(iter_, 1)   # 1 = groupnode
+        node = model.get_value(iter_, 1)   # 1 = groupnode
 
-        app.connections[self.account].send_pb_subscribe(self.jid, groupnode,
-            self._on_pep_subscribe, groupnode)
+        con = app.connections[self.account]
+        con.get_module('PubSub').send_pb_subscribe(
+            self.jid, node, self._on_pep_subscribe, groupnode=node)
 
     def on_unsubscribe_button_clicked(self, widget):
         """
         Called when 'unsubscribe' button is pressed. Send unsubscription request
         """
         model, iter_ = self.window.services_treeview.get_selection().get_selected()
-        if iter_ is None: return
+        if iter_ is None:
+            return
 
-        groupnode = model.get_value(iter_, 1) # 1 = groupnode
+        node = model.get_value(iter_, 1) # 1 = groupnode
 
-        app.connections[self.account].send_pb_unsubscribe(self.jid, groupnode,
-            self._on_pep_unsubscribe, groupnode)
+        con = app.connections[self.account]
+        con.get_module('PubSub').send_pb_unsubscribe(
+            self.jid, node, self._on_pep_unsubscribe, groupnode=node)
 
     def _on_pep_subscriptions(self, conn, request):
         """
@@ -2156,11 +2104,11 @@ class DiscussionGroupsBrowser(AgentBrowser):
         except Exception:
             return
 
-        groups = set()
+        groups_ = set()
         for child in subscriptions.getTags('subscription'):
-            groups.add(child['node'])
+            groups_.add(child['node'])
 
-        self.subscriptions = groups
+        self.subscriptions = groups_
 
         # try to setup existing items in model
         model = self.window.services_treeview.get_model()
@@ -2170,13 +2118,11 @@ class DiscussionGroupsBrowser(AgentBrowser):
             # 4 = subscribed?
             groupnode = row[1]
             row[3] = False
-            row[4] = groupnode in groups
+            row[4] = groupnode in groups_
 
         # we now know subscriptions, update button states
         self.update_actions()
 
-        raise nbxmpp.NodeProcessed
-
     def _on_pep_subscribe(self, conn, request, groupnode):
         """
         We have just subscribed to a node. Update UI
@@ -2191,8 +2137,6 @@ class DiscussionGroupsBrowser(AgentBrowser):
 
         self.update_actions()
 
-        raise nbxmpp.NodeProcessed
-
     def _on_pep_unsubscribe(self, conn, request, groupnode):
         """
         We have just unsubscribed from a node. Update UI
@@ -2202,12 +2146,10 @@ class DiscussionGroupsBrowser(AgentBrowser):
         model = self.window.services_treeview.get_model()
         for row in model:
             if row[1] == groupnode: # 1 = groupnode
-                row[4]=False
+                row[4] = False
                 break
 
         self.update_actions()
 
-        raise nbxmpp.NodeProcessed
-
 # Fill the global agent type info dictionary
 _agent_type_info = _gen_agent_type_info()
